<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * MultipleProjectImportPresenter presents forms for multiple uploads
 * of project files. It should handle projects importing from SAP
 * files, but now it only handles uploads of files and validates
 * input and then redirects to old app2 for actual importing 
 * and actualization of projects.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class MultipleProjectImportPresenter extends SecuredPresenter
{

	/** directory name in TEMP_DIR where uploaded files will be stored */
	private $uploadRootDir = '';
	
	/** @var array */
	private $uploadedFiles = NULL;

	/** @var int Default number of input fields for file upload. */
	private $numberOfInputs = 9;

	/** @var array */
	private $uploadDirName = '';

	/** @var string */
	private $sapImportDirName = 'import_sap';
	
    public function  __construct(DIContainer $context)
    {
		parent::__construct($context);
		$this->sapImportDirName = $context->parameters['sap_import_dir'];
        $this->uploadRootDir = Environment::getVariable('tempDir') . '/' . $this->sapImportDirName;
		$this->uploadedFiles = array();
		$php_max_file_uploads = 0 + ini_get('max_file_uploads');
		if (($php_max_file_uploads > 0) && ($php_max_file_uploads < $this->numberOfInputs)) 
			 $this->numberOfInputs = $php_max_file_uploads;
    }

	/**
	 * Overloads startup of presenter.
	 *
	 * @see Nette\Application\Presenter#startup()
	 */
	protected function startup()
	{
		parent::startup();
	}

	/**
	 * Default renderer. Not Used - redirects to renderUploadSAPDataFilesForm.
	 */
	public function renderDefault()
	{
 		if ($this->testUserRoles('PMO'))
			$this->redirect('uploadDataFilesForm');
		// otherwise blank template is shown
	}

	/**
	 * Creates upload form with multiple input fields.
	 * 
	 * @param string $button_name Name of the upload submit button.
	 * @param string $button_label Label of the upload submit button.
	 * @param array $input_file_labels Assoc array of name => label for file inputs.
	 * @return AppForm
	 */
	private function createMultipleFilesUploadForm(
			$form_name = 'multipleFilesUploadForm', $button_name = 'upload', $button_label = 'Upload', 
			$input_file_labels = array('files' => 'Input File:'))
	{
		$form = new AppForm($this, $form_name);
		foreach ($input_file_labels as $name => $label)
			$form->addUpload($name, $label);
		$form->addSubmit($button_name, $button_label);
		return $form;
	}

	/**
	 * Factory method for creating component into template.
	 * Creates form for multiple file uploading. 
	 */
	public function createComponentMultipleFilesUploadForm()
	{
		for ($i = 1; $i <= $this->numberOfInputs; $i++)
			$input_fields['input'.$i] = 'Input SAP file:';
		$form = $this->createMultipleFilesUploadForm('multipleFilesUploadForm', 'upload', 'Upload Files', $input_fields);
		$form->getElementPrototype()->class('multipleFileUpload');
		$form->onSuccess[] = array($this, 'uploadDataFiles');
		return $form;
	}
	
	/**
	 * Creates array of files to upload from upload form and returns it as array.
	 * @param AppForm $form
	 * @return array
	 */
	private function getFilesToProcess(AppForm $form)
	{
		$filesToUpload = array();
		$values = $form->getValues();
		for ($i = 1; $i <= $this->numberOfInputs; $i++)
		{
			$file = $values['input'.$i];
			if ($file->Name != '')
				if ($file->isOk()) 
				{
					if (!isset($filesToUpload[$file->Name]))
						$filesToUpload[$file->Name] = $file;
					else
						$this->flashMessage('File ' . $file->Name . ' was uploaded multiple times. Only one copy will be used.', 'warning');
				}
				else
					$this->flashMessage('File ' . $file->Name . ' failed to upload (error: ' . $file->error . ', see http://php.net/manual/en/features.file-upload.errors.php).', 'error');
		}
		return $filesToUpload;
	}
	
	/**
	 * Creates working directory in root for SAP imports with given dir name suffix.
	 * @param string $dir_name_suffix
	 * @return mixed Returns path to created directory or FALSE on error.
	 */
	private function createWorkDirectory($dir_name_suffix = NULL)
	{
		if (!file_exists($this->uploadRootDir))
			if (!mkdir($this->uploadRootDir))
			{
				Debugger::log('Unable to create temporary upload root directory: ' . $this->uploadRootDir, Debugger::ERROR);
				return false;
			}
		/* create brand new temporary directory and move uploaded files */
		do {
			$this->uploadDirName = date('Ymd-His-') . rand(1, 100) . $dir_name_suffix;
			$upload_dir = $this->uploadRootDir . '/' . $this->uploadDirName; 
		} while (file_exists($upload_dir));
		if (!mkdir($upload_dir))
		{
			Debugger::log('Unable to create temporary upload root directory: ' . $upload_dir, Debugger::ERROR);
			return false;
		}
		return $upload_dir;
	}

	/**
	 * Factory method for creating component into template.
	 * Creates form for multiple file uploading. 
	 */
	public function createComponentProcessFilesForm()
	{
		$form = new AppForm();
		$form->addHidden('directory')->value = $this->uploadDirName;
		$form->addSubmit('processSapFiles', 'Update Projects');
		$form->onSuccess[] = array($this, 'importFromFiles');
		return $form;
	}
	
	/**
	 * Renders form for uploaded file processing. 
	 */
	public function renderShowImportLog($directory, $batch_number, $error_count, $warning_count, $notice_count, $info_count, $updated_count)
	{
		// TODO: better link generation (not fixed path)
		//$this->template->logfileUrl = str_replace('index.php', '', $_SERVER["SCRIPT_NAME"]) . 'temp/' . $this->sapImportDirName . '/' . $directory . '/logfile.html';
		$this->template->batchNumber = $batch_number;
		if ($error_count > 0)
			$this->flashMessage('Process finished with errors!', 'error');
		elseif ($warning_count > 0)
			$this->flashMessage('Process finished successfully with warnings.', 'warning');
		elseif ($notice_count > 0)
			$this->flashMessage('Process finished successfully with notices.', 'success');
		else if ($updated_count > 0)
			$this->flashMessage('Process finished successfully with changes.', 'success');
		else
			$this->flashMessage('Process finished successfully without changes.', 'success');
	}

	/**
	 * Renders form for multiple file upload. 
	 */
	public function renderUploadDataFilesForm()
	{
 		if (!$this->testUserRoles('PMO'))
		{
			$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
			$this->redirect('default');
		}
	}

	/*
	 * Uploads uploaded files and move them into working directory for 
	 * checks if they are ok and prepares array of names of files for later processing.
	 * 
	 * @param AppForm $form Sent form for processing.
	 */
	public function uploadDataFiles(AppForm $form)
	{
		$filesToUpload = $this->getFilesToProcess($form);
		if (count($filesToUpload) == 0)
		{
			$this->flashMessage('No files uploaded. Check server limits! (max total files size :'.ini_get('upload_max_filesize').' , max files to upload: ' . ini_get('max_file_uploads') . ')', 'info');
			$this->redirect('uploadDataFilesForm');
		}
		else
		{
			if (!$this->createWorkDirectory())
			{
				$this->flashMessage('Failed to create temporary working directory.', 'error');
				$this->redirect('uploadDataFilesForm');
			}
			/* ... move uploaded files there */
			$fhandle = false;
			$log = new MemoryLogger();
			$importer = new MultipleProjectSAPDataImportService($this->context, $this->uploadRootDir . '/' . $this->uploadDirName . "/", $log);
			$PMBFilePresent = $tasksFilePresent = FALSE;
			foreach ($filesToUpload as $file)
			{
				// check names (if ok then file might be ok)
				$file_info = MultipleProjectSAPDataImportService::guessTransactionFileType($file->getName());
				if ($file_info == FALSE)
				{
					$this->flashMessage('File ' . $file->getName() . ' was ignored. Unacceptible filename.', 'warning');
					continue;
				}
				$tmp_file_name = $file->getTemporaryFile();
				$fhandle = fopen($tmp_file_name, 'r');
				if ($fhandle === false)
				{
					$this->flashMessage('Cannot read file ' . $file->getName() . '. Ignored.', 'error');
					continue;
				}
				$file_creation_timestamp = '';
				$log->setPrefix($file->getName() . ': ');
				switch ($file_info['type']) 
				{
					case MultipleProjectSAPDataImportService::SAP_CN43N_OUTPUT :
						$file_creation_timestamp = $importer->checkCN43NTransactionFile($fhandle);
						break;
					case MultipleProjectSAPDataImportService::SAP_CN47N_OUTPUT:
						$file_creation_timestamp = $importer->checkCN47NTransactionFile($fhandle);
						break;
					case MultipleProjectSAPDataImportService::SAP_YPSDSNPLNYR_OC_OUTPUT:
						$file_creation_timestamp = $importer->checkYPSDSNPLNYR_OCTransactionFile($fhandle);
						break;
					default:
						{
							$this->flashMessage('Failed to guess type of file ' . $file->getName() . '. Ignored.', 'error');
							continue 2;
						}
				}

				if ($file_creation_timestamp === FALSE)
					continue;
				// check if exchange rates are available for whole year
				$year = 'N/A';
				if ($file_info['type'] == MultipleProjectSAPDataImportService::SAP_YPSDSNPLNYR_OC_OUTPUT)
				{
					$year = 0 + $file_info['year'];
					if ($year < (0 + date('Y')))
						$this->flashMessage('File ' . $file->getName() . ' seems to be from history (older than actual year). Check it out!', 'warning');
					$currency = MultipleProjectSAPDataImportService::getPMBCurrency($fhandle);
					if(!$currency)
					{
						$this->flashMessage("PMB file has unknown currency.", 'error');
						$this->redirect('uploadDataFilesForm');
					}
					if($currency == ExchangeRatePresenter::CZK)
					{
						$ex_rates = $this->context->ratesRepository->findYearRates('CZK', 'USD', $year);
						foreach ($ex_rates as $rate) 
							if ($rate->isUndefined())
							{
								$this->flashMessage('Exchange rates from CZK to USD are not defined for whole year '.$year.'! Skipping file ' . $file->getName(), 'warning');
								continue 2;
							}
					}
					if($currency == ExchangeRatePresenter::EUR)
					{
						$ex_rates = $this->context->ratesRepository->findYearRates('EUR', 'USD', $year);
						foreach ($ex_rates as $rate) 
							if ($rate->isUndefined())
							{
								$this->flashMessage('Exchange rates from EUR to USD are not defined for whole year '.$year.'! Skipping file ' . $file->getName(), 'warning');
								continue 2;
							}
					}
				}
				fclose($fhandle);
				$path = $this->uploadRootDir . '/' . $this->uploadDirName . "/" . $file->getName();
				try
				{
					$file->move($path);
					$this->uploadedFiles[] = array('year' => $year, 'created' => $file_creation_timestamp, 'file' => $file, 'type' => $file_info['type']);
					$this->flashMessage('File ' . $file->getName() . ' uploaded successfully.', 'success');
				}
				catch (InvalidStateException $e)
				{
					$this->flashMessage('File ' . $file->getName() . ' upload error: ' . $e->Message, 'error');
				}

				if ($file_info['type'] === MultipleProjectSAPDataImportService::SAP_YPSDSNPLNYR_OC_OUTPUT)
					$PMBFilePresent = TRUE;
				else if ($file_info['type'] === MultipleProjectSAPDataImportService::SAP_CN47N_OUTPUT)
					$tasksFilePresent = TRUE;
			}
			if (!$PMBFilePresent || !$tasksFilePresent)
			{
				$this->flashMessage('This operation requires both PMB and Task files.', 'error');
				$this->redirect('uploadDataFilesForm');
			}
			
			// show messages generated by service checks
			foreach ($log->getMessages() as $message_log)
				$this->flashMessage($message_log['msg'], $message_log['level']);
			
			if (count($this->uploadedFiles) == 0)
			{
				$this->flashMessage('No files to process.', 'info');
				$this->redirect('uploadDataFilesForm');
			}

			/* show it to user */
			$this->setView('processUploadedFilesForm');
		}
	}
	
	/**
	 * Renders form for uploaded file processing. 
	 */
	public function renderProcessUploadedFilesForm()
	{
		$this->template->uploadedFiles = array();
		foreach ($this->uploadedFiles as $uploaded_file_info)
			$this->template->uploadedFiles[] = array(
						'year' => $uploaded_file_info['year'], 
						'name' => $uploaded_file_info['file']->Name, 
						'created' => (DateTime::createFromFormat('d.m.Y', $uploaded_file_info['created'])) ? $uploaded_file_info['created'] : '',
						'type' => $uploaded_file_info['type']
			);
		$this->template->form = $this['processFilesForm'];
		$this->template->form->onSuccess[] = array($this, 'importFromFiles');
	}
	
	/**
	 * Renders form for uploaded file processing. 
	 */
	public function importFromFiles(AppForm $form)
	{
		$exec_time = 300;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
			$this->redirect('uploadDataFilesForm');
		}
		$values = $form->getValues();
		$working_dir = $this->uploadRootDir . '/' . $values['directory'];
		if (isset($values['directory']) && file_exists($working_dir) && is_dir($working_dir))
		{
			/*$logfile = new HTMLTableLogger($working_dir . '/logfile.html');
			$logfile->open('Tasks and WBS Element names Import Log');*/
			// factory method
			$batch_number = 'import-'.time();
			$logger = $this->context->createLogger($batch_number, "PMB IMPORT");
			Stopwatch::start("Multiple Project Import");
			$importer = new MultipleProjectSAPDataImportService($this->context, $working_dir, $logger);
			try
			{
				$importer->runFilesImport();
			}
			catch (Exception $e)
			{
				$this->flashMessage($e->getMessage(), 'error');
				Debugger::log($e->getMessage(), Debugger::ERROR);
				$this->redirect('uploadDataFilesForm');
			}
			$stats = $logger->getStats();
			$importer_stats = $importer->getProjectImportStats();
			Stopwatch::stop("Multiple Project Import");
			$this->redirect('showImportLog', 
					array(	'directory' => $values['directory'], 
							'batch_number' => $batch_number, 
							'error_count' => $stats[ILogger::ERROR],
							'warning_count' => $stats[ILogger::WARNING],
							'notice_count' => $stats[ILogger::NOTICE],
							'info_count' => $stats[ILogger::INFO],
						    'updated_count' => $importer_stats[MultipleProjectSAPDataImportService::STATS_PROJECT_UPDATED]
						)
			);
		}
		else
		{
			Stopwatch::stop("Multiple Project Import");
			$this->flashMessage('Working dir not found.', 'error');
			Debugger::log('Working dir not found: ' . $working_dir, Debugger::ERROR);
			$this->redirect('uploadDataFilesForm');
		}
	}

	/**
	 * Renders list of newly found project available for creation or blacklisting.
	 */
	public function renderNewProjects()
	{
        if (!$this->isAllowed(self::RESOURCE_PROJECT, self::ACTION_CREATE))
		{
			$form = $this['newProjectActionForm'];
			$form['Create']->setDisabled(TRUE);
			$form['Blacklist']->setDisabled(TRUE);
		}
		
		$this->template->projects = $this->context->projectFoundRepository->findAllInfos();
	}
	
	/**
     * Creates action form for newly found project.
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
	public function createComponentNewProjectActionForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addHidden('SAP_ID');
		
		$form->addSubmit('Create', 'Create New Project')
				->onClick[] = array($this, 'newProjectActionFormCreateSubmitted');
		
		$form->addSubmit('Blacklist', 'Blacklist Project')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'newProjectActionFormBlacklistSubmitted');
		
		return $form;
	}

	/**
	 * Service for Create button of NewProjectAction
	 * @param SubmitButton $form
	 */
	public function  newProjectActionFormCreateSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			if (!$this->isAllowed(self::RESOURCE_PROJECT, self::ACTION_CREATE))
			{
				$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
				$this->redirect('default');
			}
			// avoid unnecessary error in case there is some inconsistency
			$project = $this->context->projectRepository->findProjectBySAPID($entry->SAP_ID);
			if ($project != NULL)
			{
				$this->flashMessage("Project with SAP ID \"$entry->SAP_ID\" already exists in the system! Cannot create new.", 'warning');
				$this->redirect('newProjects');
			}
			$project_stub = $this->context->projectFoundRepository->findBySAPID($entry->SAP_ID);
			if (!$project_stub)
			{
				$this->flashMessage("Project with SAP ID \"$entry->SAP_ID\" not found.", 'error');
				$this->redirect('newProjects');
			}
			
			$logfile = $this->context->createLogger('new-project-'.time(), "NEW PROJECT");
			$import_service = new MultipleProjectSAPDataImportService($this->context, '', $logfile);
			$project_id = $import_service->runNewProjectImport($project_stub);
			if ($project_id !== FALSE)
			{
				$this->flashMessage("Project \"$project_stub->Name\" with SAP ID \"$project_stub->SAP_ID\" was created.", 'success');
				$this->flashMessage("For correct upload of project details and plan run bulk import again.", 'warning');
				if (!$this->context->projectFoundRepository->delete($entry->SAP_ID))
					Debugger::log('Failed to delete stub of project with SAP ID='.$project_stub->SAP_ID, Debugger::ERROR);
				$this->redirect('Project:detail', array('id' => $project_id));
			}
			else
			{
				$this->flashMessage("Failed to create project \"$project_stub->Name\" with SAP ID \"$project_stub->SAP_ID\"!", 'error');
				$this->redirect('newProjects');
			}
		}
	}

	/**
	 * Service for Create button of NewProjectAction
	 * @param SubmitButton $form
	 */
	public function  newProjectActionFormBlacklistSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues();

		if (!$this->isAllowed(self::RESOURCE_PROJECT, self::ACTION_CREATE))
		{
			$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
			$this->redirect('default');
		}
		$project = $this->context->projectFoundRepository->findInfoBySAPID($entry->SAP_ID);
		if ($project)
		{
			if ($this->context->projectFoundRepository->putOnBlacklist($project->SAPID))
			{
				$logger = $this->context->createLogger(NULL, "PROJECT BLACKLIST");
				$logger->log('Project "'.$project->Name.'" with SAP ID "'.$project->SAPID.'" was blacklisted.', ILogger::NOTICE);
				$this->flashMessage("Project \"$project->Name\" with SAP ID \"$project->SAPID\" was blaclisted.", 'success');
			}
			else
				$this->flashMessage("Failed to blacklist project \"$project->Name\" with SAP ID \"$project->SAPID\" !", 'error');
		}
		else
			$this->flashMessage("Project with SAP ID \"$entry->SAP_ID\" was not found!", 'error');
		$this->redirect('newProjects');
	}

	/**
	 * Renders list of newly found project available for creation or blacklisting.
	 */
	public function renderBlacklistedProjects()
	{
        /* Viewing for all */
 		/*if (!$this->testUserRoles('ADMIN'))
		{
			$this->flashMessage("You are not authorized to use this functionality. ADMIN role required.", 'info');
			$this->redirect('Error:default');
		}*/
		$this->template->projects = $this->context->projectFoundRepository->findAllBlacklistedInfos();
	}
    
	/**
	 * Renders list of newly found project available for creation or blacklisting.
	 */
	public function actionUnblacklistProject($sap_id)
	{
 		if (!$this->testUserRoles('PMO'))
		{
			$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
			$this->redirect('default');
		}
        // project data does not have to be acual => deletion (now) and reimporting (by user) required
        if ($this->context->projectFoundRepository->delete($sap_id))
        {
            $logger = $this->context->createLogger(NULL, "PROJECT BLACKLIST");
            $logger->log('Project with SAP ID "'.$sap_id.'" was removed from blacklist.', ILogger::NOTICE);
            $this->flashMessage("Project with SAP ID \"$sap_id\" was removed from blacklist. You need to import it again to be able to create it!", 'success');
        }
        else
            $this->flashMessage("Failed to remove project with SAP ID \"$sap_id\" from blacklist!", 'error');
		$this->setView('blacklistedProjects');
	}
    
}
