<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presents interface for importing data from iASAP.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PersonImportPresenter extends SecuredPresenter
{
    /**
     * Service for Person import.
     * @var PersonImportService
     */
	private $personImportService;
    
    /**
     * Nette default startup method 
     */
    public function startup()
    {
        parent::startup();
		
        if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_VIEW))
        {
            $this->flashMessage('You don\'t have permission to use Person import. You were redirected.');
            $this->redirect('Dashboard:');
        }
        
    }
    
    /**
     * Nette default render method 
     */
	public function renderDefault()
	{
        $this->template->maxExecTime = IASAPActualsImportServiceTLC::MAXIMUM_EXECUTION_TIME;
		$lock = $this->context->createLock(IASAPActualsImportServiceTLC::LOCK_KEY);
		$this->template->running = $lock->locked();
	}
	
    /**
     * Nette default render method 
     */
	public function renderNewPersons()
	{
		$persons = $this->context->personFoundRepository->findAllNonBlacklisted();
		$this->template->personArray = array();
		foreach ($persons as $eid => $person)
		{
			// avoid unnecessary error in case there is some inconsistency
			$existingPerson = $this->context->personRepository->find($person->EID);
			if ($existingPerson != NULL)
			{
				$this->context->personFoundRepository->delete($person->EID);
				continue;
			}
			$this->template->personArray[$eid]['EID'] = $person->EID;
			$this->template->personArray[$eid]['Name'] = $person->Name;
			$this->template->personArray[$eid]['Surname'] = $person->Surname;
			$this->template->personArray[$eid]['WorkCenterID'] = $person->WorkCenterID;
			$this->template->personArray[$eid]['SupervisorName'] = $person->SupervisorName;
			$this->template->personArray[$eid]['SupervisorEID'] = $person->SupervisorEID;
			$this->template->personArray[$eid]['SupervisorSurname'] = $person->SupervisorSurname;
			$this->template->personArray[$eid]['OrganizationUnitID'] = $person->OrganizationUnitID;
			$this->template->personArray[$eid]['WCFound'] = $this->context->costCenterRepository->findCostCenter($person->WorkCenterID);
			$this->template->personArray[$eid]['OrgUnitFound'] = $this->context->organizationUnitRepository->find($person->OrganizationUnitID);
			$this->template->personArray[$eid]['LeaderFound'] = $this->context->personRepository->find($person->SupervisorEID);
			
		}
	}
	
    /**
     * Nette default render method 
     */
	public function renderBlacklistedPersons()
	{
		$persons = $this->context->personFoundRepository->findAllBlacklisted();
		$this->template->personArray = array();
		foreach ($persons as $eid => $person)
		{
			// avoid unnecessary error in case there is some inconsistency
			$existingPerson = $this->context->personRepository->find($person->EID);
			if ($existingPerson != NULL)
			{
				$this->context->personFoundRepository->delete($person->EID);
				continue;
			}
			$this->template->personArray[$eid]['EID'] = $person->EID;
			$this->template->personArray[$eid]['Name'] = $person->Name;
			$this->template->personArray[$eid]['Surname'] = $person->Surname;
			$this->template->personArray[$eid]['WorkCenterID'] = $person->WorkCenterID;
			$this->template->personArray[$eid]['SupervisorName'] = $person->SupervisorName;
			$this->template->personArray[$eid]['SupervisorEID'] = $person->SupervisorEID;
			$this->template->personArray[$eid]['SupervisorSurname'] = $person->SupervisorSurname;
			$this->template->personArray[$eid]['OrganizationUnitID'] = $person->OrganizationUnitID;
			$this->template->personArray[$eid]['WCFound'] = $this->context->costCenterRepository->findCostCenter($person->WorkCenterID);
			$this->template->personArray[$eid]['OrgUnitFound'] = $this->context->organizationUnitRepository->find($person->OrganizationUnitID);
			$this->template->personArray[$eid]['LeaderFound'] = $this->context->personRepository->find($person->SupervisorEID);
		}
		//dump($this->template->personArray);
	}
    
	/**
     * Creates action form for newly found person.
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
	public function createComponentNewPersonActionForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addHidden('EID');
		
		$form->addSubmit('Create', 'Create New Person')
				->onClick[] = array($this, 'newPersonActionFormCreateSubmitted');
		
		$form->addSubmit('Blacklist', 'Blacklist Person')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'newPersonActionFormBlacklistSubmitted');
		
		return $form;
	}
	
	/**
	 * Service for Create button of NewPersonAction
	 * @param SubmitButton $form
	 */
	public function  newPersonActionFormCreateSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_CREATE))
			{
				$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
				$this->redirect('default');
			}
			$person = $this->context->personFoundRepository->findByEID($entry->EID);
			if (!$person)
			{
				$this->flashMessage("Person with EID \"$entry->EID\" not found.", 'error');
				$this->redirect('newPersons');
			}
			
			$this->redirect('Person:create', $person->EID);
		}
	}

	/**
	 * Service for Create button of NewPersonAction
	 * @param SubmitButton $form
	 */
	public function  newPersonActionFormBlacklistSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues();

		if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_CREATE))
		{
			$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
			$this->redirect('default');
		}
		$person = $this->context->personFoundRepository->findByEID($entry->EID);
		if ($person)
		{
			$person->Blacklisted = 1;
			if ($this->context->personFoundRepository->update($person))
			{
				$this->flashMessage("Person " . $person->Name . " " . $person->Surname . " with EID " . $person->EID . " was blaclisted.", 'success');
			}
			else
			{
				$this->flashMessage("Failed to blacklist person with EID \"$entry->EID\"!", 'error');
			}
		}
		else
		{
			$this->flashMessage("Person with EID \"$entry->EID\" was not found!", 'error');
		}
		$this->redirect('newPersons');
	}
	
	/**
	 * Renders list of newly found person available for creation or blacklisting.
	 */
	public function actionUnblacklistPerson($eid)
	{
 		if (!$this->testUserRoles('PMO'))
		{
			$this->flashMessage("You are not authorized to use this functionality. PMO role required.", 'info');
			$this->redirect('default');
		}
        // person data does not have to be acual => deletion (now) and reimporting (by user) required
        if ($this->context->personFoundRepository->delete($eid))
        {
            $this->flashMessage("Person with EID \"$eid\" was removed from blacklist. You need to import it again to be able to create it!", 'success');
        }
        else
		{
            $this->flashMessage("Failed to remove person with EID \"$eid\" from blacklist!", 'error');
		}
		$this->setView('blacklistedPersons');
	}
	
    /**
     * Run method 
     */
    public function renderRun()
    {
		if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_RUN))
        {
            $this->flashMessage('You don\'t have permission to use Person import. You were redirected.');
            $this->redirect('Dashboard:');
        }
		
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'person_import-'.$logs_suffix.'.html';
		$batch_id = 'person_import-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "PERSON IMPORT") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "PERSON IMPORT"));

		//  Person import
        $this->personImportService = new PersonImportService($this->context, $multilog);
        
        try
        {
			switch ($this->personImportService->runPersonImport())
			{
				case PersonImportService::RESULT_SUCCESS:
					$this->flashMessage('Person import was successful.', 'success');
					break;
				case PersonImportService::RESULT_NO_DATA:
					$this->flashMessage('Person import was successful but no person to import was found.', 'warning');
					break;
				case PersonImportService::RESULT_WARNING:
					$this->flashMessage('Person import was successful with warnings.', 'warning');
					break;
				case PersonImportService::RESULT_ERROR:
					$this->flashMessage('Person import failed!', 'error');
					break;
				default: break;
			}
        }
        catch (IASAPActualsImportServiceTLCException $e)
        {
			$this->flashMessage("Person import failed: " . $e->getMessage(), 'error');
			$multilog->log('Person import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('Person import failed: ' . $e->getMessage(), Debugger::ERROR);
		}

        $filelog->close();
		
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
		$this->template->batchNumber	= $batch_id;
    }
}
