<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of Customers.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class CustomerPresenter extends SecuredPresenter
{
    /**
     * Nette default render method.
     */
    public function renderDefault()
	{
		$customers = $this->context->customerRepository->findAll();
		$custArray = array();
		foreach($customers as $customer)
		{
			$custArray[$customer->ID]['Name'] = $customer->Name;
			if($customer->Responsible)
				$custArray[$customer->ID]['Responsible'] = $this->context->personRepository->find($customer->Responsible)->ReversedFullName;
			else
				$custArray[$customer->ID]['Responsible'] = '-';
			if($customer->Remarks)
				$custArray[$customer->ID]['Remarks'] = $customer->Remarks;
			else
				$custArray[$customer->ID]['Remarks'] = '-';
		}
		$this->template->customers = $custArray;
	}
	
	/**
     * Renders form for creating new Customer.
	 */
	public function renderCreate()
	{
		$form = $this['customerForm'];
		$this->template->form = $form;
	}
	
	/**
     * Shows detail of Customer.
	 * 
	 * @param type $customerID ID of Customer
	 */
	public function renderShowDetail($customerID)
	{
		$customer = $this->context->customerRepository->find($customerID);
		if(is_null($customer))
		{
			$this->flashMessage("Selected Customer doesnt exist.", 'notice');
			$this->redirect('default');
		}
		
		$formDefault = array();
		$formDefault['CustomerID'] = $customer->ID;
		$formDefault['Name'] = $customer->Name;
		$formDefault['Responsible'] = $customer->Responsible;
		if($customer->Responsible)
			$formDefault['Responsible_info'] = $this->context->personRepository->find($customer->Responsible)->ReversedFullName;
		$formDefault['Remarks'] = $customer->Remarks;
		$form = $this['customerForm'];
		$form->setDefaults($formDefault);
		
		$this->template->form = $form;
	}
	
    /**
     * Creates general Customer form.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentCustomerForm($name)
    {
        $form = new AppForm($this, $name);

		$form->addHidden('CustomerID');
		
        $form->addText('Name', 'Name')
				->addRule(FORM::FILLED, 'Name is mandatory.')
                ->setAttribute('maxlength', 50);

        $form->addText('Responsible', 'Responsible', 7)
			->getControlPrototype()
				->addClass('autocomplete');
        $form->addText('Responsible_info', 'Responsible_info')
			->setAttribute('readonly', 'readonly');

        $form->addTextArea('Remarks', 'Remarks', 28, 3);

        $form->addSubmit('back', 'Back to Customers')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'backClicked');
		
        $form->addSubmit('Create', 'Create Customer')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'createClicked');
		
        $form->addSubmit('save', 'Save Customer')
                ->onClick[] = callback($this, 'createCustomerFormSubmitted');
		
		$form->addSubmit('update', 'Update Customer')
                ->onClick[] = callback($this, 'updateCustomerFormSubmitted');
		
        return $form;
    }
	
    public function handleAutoComplete()
    {
		$this->payload->autoComplete = array();

		$text = trim($_GET['q']);

		if ($text !== '')
		{
			$filter = new PersonFilter();
			$filter->setFirstname($text)->setSurname($text)->setID($text);
			$persons = $this->context->personRepository->getPersonsByOrFilter($filter);
			foreach ($persons as $person) 
			{
				$person_item = array($person->getReversedFullNameWithEID(), strtoupper(trim($person->ID)), $person->getReversedFullName());
				$this->payload->autoComplete[] = $person_item;
			}
		}

		$this->terminate();
	}
	
    public function backClicked(SubmitButton $button)
    {
        $this->redirect('default');
    }
	
    public function createClicked(SubmitButton $button)
    {
        $this->redirect('create');
    }
	
    /**
     * Customer Form Submitted - Save.
     * 
     * @param SubmitButton $button 
     */
    public function createCustomerFormSubmitted(SubmitButton $button)
    {
        $entry = $button->form->getValues();
		if ($button->form->isValid())
		{
			$customer = new Customer();
			$customer->Name = $entry['Name'];
			$customer->Responsible = $entry['Responsible'];
			$customer->Remarks = $entry['Remarks'];
			if ($this->context->customerRepository->save($customer))
			{
				$this->flashMessage("Customer " . $customer->Name . " saved.", 'success');
				$this->redirect('Customer:showDetail', $customer->ID);
			}
			else
			{
				$this->flashMessage("Saving failed.", 'error');
				$this->redirect('Customer:create');
			}
		}
		else
		{
			$this->flashMessage("Invalid form values.");
		}
        $this->redirect('Customer:default');
    }
	
    /**
     * Customer Form Submitted - Update.
     * 
     * @param SubmitButton $button 
     */
    public function updateCustomerFormSubmitted(SubmitButton $button)
    {
        $entry = $button->form->getValues();
		if ($button->form->isValid())
		{
			$customer = $this->context->customerRepository->find($entry['CustomerID']);
			$customer->Name = $entry['Name'];
			$customer->Responsible = $entry['Responsible'];
			$customer->Remarks = $entry['Remarks'];
			if ($this->context->customerRepository->update($customer))
			{
				$this->flashMessage("Customer " . $customer->Name . " updated.", 'success');
				$this->redirect('Customer:showDetail', $customer->ID);
			}
			else
			{
				$this->flashMessage("Updating failed.", 'error');
				$this->redirect('Customer:showDetail', $entry['CustomerID']);
			}
		}
		else
		{
			$this->flashMessage("Invalid form values.");
		}
        $this->redirect('Customer:showDetail', $entry['CustomerID']);
    }
}
