<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for exporting STRAP data from database to simple MS EXCEL (xlsx) file.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class StrapReportPresenter extends SecuredPresenter
{
	
    /**
     * Nette default render method.
     */
    public function renderDefault()
    {
        $formDefaultReport = array();
		$lock = $this->context->createLock('STRAP_Report');
        $downloadReport = $this['downloadForm'];
		if($lock->locked())
		{
			$downloadReport['download']->setDisabled(TRUE);
			$this->flashMessage('Report is generated right now.', 'notice');
		}
        $downloadReport->setDefaults($formDefaultReport);
        $this->template->downloadReport = $downloadReport;
		
		$uploadFile = $this['uploadForm'];
		$this->template->uploadForm = $uploadFile;
		
		$dir = null;
        if (file_exists(Environment::getConfig('strap_storage_dir'))) 
			$dir = opendir(Environment::getConfig('strap_storage_dir'));
		$files = array();
		//List files in directory
		if($dir != null)
		{
			while (($file = readdir($dir)) !== false)
			{
				if(preg_match('#.xlsx#', $file) || preg_match('#.xls#', $file) || preg_match('#.xlsm#', $file))
				{
					$files[$file]['file'] = $file;
					$files[$file]['modDate'] = date("Y-d-m H:i:s.", filemtime(Environment::getConfig('strap_storage_dir') . $file));
				}
			}
			closedir($dir); 
		}
		
		$this->template->path = Environment::getConfig('strap_storage_dir');
		$this->template->files = $files;
    }

    /**
     * Creates form for download.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentDownloadForm($name)
    {
        $form = new AppForm($this, $name);
        $form->addHidden('download_token_value_id');
        $form->addSubmit('download', 'Regenerate');
        $form->onSuccess[] = array($this, 'downloadFormSubmitted');
    }
	
    /**
     * Creates form for download.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentUploadForm($name)
    {
        $form = new AppForm($this, $name);
		$submit_button = $form->addSubmit('upload', 'Upload');
		$submit_button->setAttribute('onclick', '
					if (!$("#frmuploadForm-file")[0].value)
					{
					    alert("Select uploaded file!");
						return false;
					}');
		if (!$this->testUserRoles('STRAP-ADMIN'))
			$submit_button->setDisabled(true);
        $form->addUpload('file', 'Upload new template file:');
		$form->onSuccess[] = array($this, 'uploadFormSubmitted');
    }

    /**
     * Component downloadForm submit download tracking table.
     * 
     * @param AppForm $form 
     */
    public function uploadFormSubmitted(AppForm $form) 
	{
        $entry = $form->getValues();
		//Only STRAP-ADMIN user can update file
		if ($this->testUserRoles('STRAP-ADMIN'))
		{
            if ($entry['file']->isOk())
            {
				//check if imported file is xlsx type
				if(preg_match('#.xlsx$#', $entry['file']->name) || preg_match('#.xlsm$#', $entry['file']->name) || preg_match('#.xls$#', $entry['file']->name))
				{
					try
					{
						$entry['file']->move(Environment::getConfig('strap_storage_dir') . Environment::getConfig('strap_pivotTable_file') . '.' . pathinfo($entry['file']->name, PATHINFO_EXTENSION));
						unset($entry['file']);
						$this->flashMessage("File was uploaded.", 'success');
					}
					catch (Exception $e)
					{
						$this->flashMessage("Updating file failed.", 'error');
					}
				}
				else
				{
					$this->flashMessage("Uploaded file has to be in xls, xlsx or xlsm format!", 'error');
				}
				$this->redirect('default');
            }
            else
            {
                $this->flashMessage("Updating file failed.", 'error');
				$this->redirect('default');
            }
		}
		else
		{
			$this->flashMessage("You are not authorized to update pivot table. STRAP-ADMIN role required!", 'error');
			$this->redirect('default');
		}
	}
	
    /**
     * Component downloadForm submit download tracking table.
     * 
     * @param AppForm $form 
     */
    public function downloadFormSubmitted(AppForm $form)       
    {
        $values = $form->getValues();
		$multilog = new MultiLogger();
		$batch_id = 'STRAP-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "STRAP manual generation"));
        $STRAPReportService = new STRAPReportService($this->context, $multilog, $values['download_token_value_id']);
		$importResult = true;
        try
        {
			$STRAPReportService->generateReport();
		}
        catch (Exception $e)
        {
			$importResult = false;
            Debugger::log('STRAP manual generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('STRAP manual generation failed: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage('STRAP manual generation failed: ' . $e->getMessage(), 'error');
		}
		if($importResult)
		{
			$multilog->log('STRAP manual generation completed.', ILogger::NOTICE);
			$this->flashMessage('Report regeneration was successful.', 'success');
		}
		$this->redirect('default');
    }
}

class StrapReportGenerationException extends Exception {}