<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Customer selector
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class CustomerSelectorContainer extends BaseContainer
{
	/** @var int Width of container fields */
	private $width;
	/** @var boolean Cut short each field in hierarchy */
	private $showHierarchy;
	
	/**
	 * @param int $width=200 Width of container fields. Min value = 60
	 * @param boolean $showHierarchy=TRUE Cut short each field in hierarchy
	 */
	public function __construct($width = 200, $showHierarchy = TRUE)
    {
		parent::__construct();
		
		$this->width = ($width >= 60) ? $width : 60;
		$this->showHierarchy = $showHierarchy;
	}
	
	/**
	 * Handels change of value in customer selector
	 * @param int $id Should be organization unit id
	 * @return array
	 *     $array(
	 *         [Customer|Program|Call] => array(
	 *             id => value,     // selected value
	 *             values => array( // list of programs|calls
	 *                 id => name
	 *             )
	 *         )
	 *     )
	 */
	public function ajaxHandle($id, $part_name)
	{
		$customer_id = $program_id = $call_id = NULL;
		
		if ($id && ($part_name == 'Customer'))
		{
			$customer_id = $id;
			$related_programs = $this->context->customerProgramRepository->findByCustomer($id);
			$related_calls = $this->context->customerProgramCallRepository->findByCustomer($id);
		}
		elseif ($id && ($part_name == 'Program'))
		{
			$program = $this->context->customerProgramRepository->find($id);
			$customer_id = ($program) ? $program->CustomerID : NULL;
			$program_id = $id;
			$related_programs = $this->context->customerProgramRepository->findByCustomer($customer_id);
			$related_calls = $this->context->customerProgramCallRepository->findByProgram($id);
		}
		elseif ($id && ($part_name == 'Call'))
		{
			$call = $this->context->customerProgramCallRepository->find($id);
			$program_id = ($call) ? $call->ProgramID : NULL;
			$program = $this->context->customerProgramRepository->find($program_id);
			$customer_id = ($program) ? $program->CustomerID : NULL;
			$call_id = $id;
			$related_programs = $this->context->customerProgramRepository->findByCustomer($customer_id);
			$related_calls = $this->context->customerProgramCallRepository->findByProgram($program_id);
		}
		else
		{
			$related_programs = $this->context->customerProgramRepository->findAll();
			$related_calls = $this->context->customerProgramCallRepository->findAll();
		}
		
		// Programs
		$programs = array();
		foreach ($related_programs as $program)
			$programs[$program->ProgramID] = $program->Program;
		
		// Calls
		$calls = array();
		foreach ($related_calls as $call)
			$calls[$call->CallID] = $call->Call;
		
		return	array(
			'Customer' => array(
				'id' => $customer_id,
				'values' => array()
			),
			'Program' => array(
				'id' => $program_id,
				'values' => $programs
			),
			'Call' => array(
				'id' => $call_id,
				'values' => $calls
			)
		);
	}
	
	/**
	 * Adds container content to form
	 */
	protected function configure()
	{
		// Here place link to ajax handle method
		$this->addHidden('ajaxHandle');
		
		$all_customers = $this->context->customerRepository->findAll();
        $customers = array();
        foreach ($all_customers as $customer)
            $customers[$customer->ID] = $customer->Name;
        $this->addSelect('Customer', 'Customer', $customers)
				->setPrompt('')
				->setAttribute('style', 'width: ' . $this->width . 'px;');
		
		$all_programs = $this->context->customerProgramRepository->findAll();
		$programs = array();
		foreach ($all_programs as $program)
			$programs[$program->ProgramID] = $program->Program;
        $this->addSelect('Program', 'Program', $programs)
				->setPrompt('');
		
		$all_calls = $this->context->customerProgramCallRepository->findAll();
		$calls = array('' => '*');
		foreach ($all_calls as $call)
			$calls[$call->CallID] = $call->Call;
        $this->addSelect('Call', 'Call', $calls)
				->setPrompt('');
		
		if ($this->showHierarchy)
		{
			$this['Program']->setAttribute('style', 'width: ' . ($this->width - 10) . 'px;');
			$this['Call']->setAttribute('style', 'width: ' . ($this->width - 20) . 'px;');
		}
		else
		{
			$this['Program']->setAttribute('style', 'width: ' . $this->width . 'px;');
			$this['Call']->setAttribute('style', 'width: ' . $this->width . 'px;');
		}
	}
}
