<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing actuals from database using dibi.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class ActualsDibiMapper extends DibiMapper implements IActualsMapper
{

    /**
     * Creates Actuals instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return	Actuals 
     * @todo rewrite using annotations
     */
    private function load(DibiRow $row)
    {
        $actuals = new Actuals();
        $actuals->Eid             = $row->EID;
        $actuals->Charge_number   = $row->ChargeNumber;
        $actuals->Year            = $row->Year;
        $actuals->Fiscal_month    = $row->FiscalMonth;
        $actuals->Hours           = $row->Hours;
        $actuals->Date            = $row->Date;
        $actuals->Project_sap_id  = $row->Project;
        $actuals->Source          = $row->Source;
        return $actuals;
    }	

    /**
     * Returns all actuals.
     * 
     * @return array Associative array of actuals.
     */
    public function findAll()
    {
        $actuals_db_result = $this->conn->query('SELECT * FROM [Actuals]')->fetchAll();
        $actuals = array();
        foreach ($actuals_db_result as $row)
        {
                $actual = $this->load($row);
                $actuals[] = $actual;
        }
        return $actuals;
    }
	
    /**
     * Returns number all actuals.
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param int $source Should be in range of ActualsTemp::SOURCE_*
     * @return int Number of stored actuals.
     */
    public function countActualsByRangeAndType(DateTime $startDate, DateTime $endDate, $source)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		
        $actuals_db_result = $this->conn->query('SELECT * FROM [Actuals] WHERE
												(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i 
												AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i AND
												[Source] = %i', 
												$start_month_idx,
												$end_month_idx,
												$source)->fetchAll();
		$count = 0;
        foreach ($actuals_db_result as $row)
        {
            $count++;
        }
        return $count;
    }

    /**
     * Returns single actuals record.
     * --- MOSTLY FOR DEBUGGING ---
     * 
     * @return Actuals Single Actuals record.
     */
    public function findSingle()
    {
        $actuals_db_row = $this->conn->query('SELECT * FROM [Actuals]')->fetch();
        $actuals = $this->load($actuals_db_row);
        return $actuals;
    }
    
    /**
	 * Returns all task years by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Task duration according to actuals.
	 */
	public function findTaskYears($id)
	{
		$db_result = $this->conn->query('SELECT DISTINCT [Year] FROM [Actuals] WHERE [ChargeNumber] = %s ORDER BY [Year] ASC', $id)->fetchAll();
		
        $years = array();
		foreach ($db_result as $row)
		{
			$years[] = $row->Year;
		}
        
		return $years;
	}
    
    /**
     * Deletes records from actuals by range (year & fiscal month) INCLUSIVELY.
	 * Dates are "rounded" to month indexes, so all data for all complete and partial
	 * months within date range are included. 
	 * 
	 * Methods translates $startDate and $endDate to indexes/numbers of months
	 * from year 0 using formula:
	 * (Year - 1) * 12 + Month 
	 * or (for fiscal months indexed in range <0,11> as in database
	 * (Year - 1) * 12 + FiscalMonth + 1
	 * 
	 * Examples of translation of date:
	 * "1st Jan 1" => 1
	 * "2nd Jan 1" => 1
	 * "5th May 10" => 113
	 * "30th Dec 2012" => 24144
	 * "1st Jan 2013" => 24145
	 * 
	 * Example of deleted data:
	 * $startDate is "2nd Feb 2012" and $endDate is "20th Feb 2012" => data for Feb 2012 are deleted
	 * $startDate is "1nd Feb 2012" and $endDate is "30th November 2012" => data from Feb 2012 to Nov 2012 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "30th November 2013" => data from Feb 2012 to Dec 2012 and data from Jan 2013 to Nov 2013 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "3rd Feb 2012" => data for Feb 2012 are deleted
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param int $source Should be in range of Actuals::SOURCE_*
     */    
    public function deleteByRange(DateTime $startDate, DateTime $endDate, $source)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
		// start
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		try
		{
		    $result = $this->conn->query('DELETE FROM [Actuals] WHERE
					(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i AND 
					(([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i AND 
					[Source] = %i', $start_month_idx, $end_month_idx, $source);
			Debugger::log("Deleting actuals by month indexes from $start_month_idx ($start_month, $start_year) to $end_month_idx ($end_month, $end_year).", Debugger::DEBUG);
			if ($result >= 0)
			{
				Debugger::log("Actuals was deleted.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Deleting of Actuals failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of actuals failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
	
    /**
     * Deletes records from actuals by range (year & fiscal month) INCLUSIVELY only for given task.
	 * Dates are "rounded" to month indexes, so all data for all complete and partial
	 * months within date range are included. 
	 * 
	 * Methods translates $startDate and $endDate to indexes/numbers of months
	 * from year 0 using formula:
	 * (Year - 1) * 12 + Month 
	 * or (for fiscal months indexed in range <0,11> as in database
	 * (Year - 1) * 12 + FiscalMonth + 1
	 * 
	 * Examples of translation of date:
	 * "1st Jan 1" => 1
	 * "2nd Jan 1" => 1
	 * "5th May 10" => 113
	 * "30th Dec 2012" => 24144
	 * "1st Jan 2013" => 24145
	 * 
	 * Example of deleted data:
	 * $startDate is "2nd Feb 2012" and $endDate is "20th Feb 2012" => data for Feb 2012 are deleted
	 * $startDate is "1nd Feb 2012" and $endDate is "30th November 2012" => data from Feb 2012 to Nov 2012 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "30th November 2013" => data from Feb 2012 to Dec 2012 and data from Jan 2013 to Nov 2013 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "3rd Feb 2012" => data for Feb 2012 are deleted
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param Task $task
	 * @param int $source Should be in range of Actuals::SOURCE_*
     */    
    public function deleteByRangeForTask(DateTime $startDate, DateTime $endDate, Task $task, $source)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
		// start
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;

		$result = $this->conn->query(
			'DELETE FROM [Actuals] WHERE (([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i 
				AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i AND [Source] = %i AND [ChargeNumber] LIKE %~like~',
			$start_month_idx,
			$end_month_idx,
            $source,
			$task->getSAPID()
		);
		if ($result >= 0)
		{
			Debugger::log("Actuals was deleted.", Debugger::INFO);
			$result = true;
		}
		else
			Debugger::log("Deleting of Actuals failed.", Debugger::ERROR);
		return $result;
    }
	
	/**
	 * Deletes records associated with given charge numbers in given range.
	 * 
	 * @param DateTime $startDate Delete from.
	 * @param DateTime $endDate Delete to.
	 * @param array $charge_numbers List of charge numbers.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteIASAPImport(DateTime $startDate, DateTime $endDate, array $charge_numbers)
	{
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U')))
			return TRUE;
		if (!count($charge_numbers))
			return TRUE;
		
		$start_year = $startDate->format('Y');
		$end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		
		try
		{
			$db_result = $this->conn->delete('Actuals')
					->where('(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i', $start_month_idx, $end_month_idx)
					->where('[ChargeNumber] IN %in', $charge_numbers)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting iASAP actuals hour import from ' . $start_year . '-' . $start_month . ' to ' . $end_year . '-' . $end_month, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting iASAP actuals hour import from ' . $start_year . '-' . $start_month . ' to ' . $end_year . '-' . $end_month . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting iASAP actuals hour import from ' . $start_year . '-' . $start_month . ' to ' . $end_year . '-' . $end_month . ' failed: ' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
    
    /**
     * Saves new Actuals record.
     * 
     * @param Actuals $actual
     * @return boolean Returns true if save successfull, otherwiste returns false.
     */
    public function save(Actuals $actual)
    {
        try
        {
            $array = array();
            $array['EID'] = $actual->Eid;
            $array['ChargeNumber'] = $actual->Charge_number;
            $array['Year'] = $actual->Year;
            $array['FiscalMonth'] = $actual->Fiscal_month;
            $array['Hours'] = $actual->Hours;
            $array['Project'] = $actual->Project_sap_id;
            $array['Source'] = ($actual->Source) ? $actual->Source : Actuals::SOURCE_ECHARGE;
            $db_result = $this->conn->insert('Actuals', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving actuals record failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
    }

    /**
     * Adds new Actuals record.
     * If Actuals record does not exist, new one is inserted.
     * If Actuals record exists, it is updated (hours are added).
     * 
     * @param Actuals $actual
     * @return boolean Returns true if adding successfull, otherwiste returns false.
     */
    public function add(Actuals $actual)
    {
        try
        {
            $db_result = $this->conn->query(
                'SELECT * FROM [Actuals] WHERE [EID] = %s AND [ChargeNumber] = %s AND [Year] = %i AND [FiscalMonth] = %i',
                $actual->Eid,
                $actual->Charge_number,
                $actual->Year,
                $actual->Fiscal_month
            )->fetch();
            if ($db_result)
            {
                $array = $db_result->toArray();
                unset($array['Hours']);
                $this->conn->query(
                    'UPDATE [Actuals] SET',
                    array('Hours' => $db_result->Hours + $actual->Hours),
                    'WHERE [EID] = %s AND [ChargeNumber] = %s AND [Year] = %i AND [FiscalMonth] = %i',
                    $actual->Eid,
                    $actual->Charge_number,
                    $actual->Year,
                    $actual->Fiscal_month
                );
            }
            else
            {
                $array = array();
                $array['EID'] = $actual->Eid;
                $array['ChargeNumber'] = $actual->Charge_number;
                $array['Year'] = $actual->Year;
                $array['FiscalMonth'] = $actual->Fiscal_month;
                $array['Hours'] = $actual->Hours;
                $array['Project'] = $actual->Project_sap_id;
				$array['Source'] = ($actual->Source) ? $actual->Source : Actuals::SOURCE_ECHARGE;
                $db_result = $this->conn->insert('Actuals', $array)->execute();
            }
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Adding actuals record failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
    
    /**
     * Gets date of the oldest record from given source.
     * 
	 * @param int $source Should be in range of Actuals::SOURCE_*
     * @return DibiDateTime
	 * @throws InvalidArgumentException
     */
    public function getLastImportDate($source)
    {
		if (!in_array($source, Actuals::getSources()))
		    throw new InvalidArgumentException('Source must be in range of Actuals::SOURCE_*. Wrong param: ' . $source);
				
        return $this->conn->select('MAX([Date])')
				->from('[Actuals]')
				->where('[Source] = %i', $source)
				->fetchSingle();
    }
	
	/**
	 * Deletes records associated with given task SAP ID.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPID($sapID)
	{
		try
		{
			$db_result = $this->conn->delete('Actuals')
					->where('[ChargeNumber] = %s', $sapID)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
	
	/**
	 * Deletes records associated with given task SAP ID.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @param int $source Should be in range of Actuals::SOURCE_*
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPIDAndSource($sapID, $source)
	{
		try
		{
			$db_result = $this->conn->delete('Actuals')
					->where('[ChargeNumber] = %s AND [Source] = %i', $sapID, $source)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
	
	/**
	 * Deletes records associated with given task SAP ID and year.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @param int $year Year.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPIDAndYear($sapID, $year)
	{
		try
		{
			$db_result = $this->conn->delete('Actuals')
					->where('[ChargeNumber] = %s AND [Year] = %i', $sapID, $year)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting iASAP actuals hour for task with SAP ID: ' . $sapID, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting iASAP actuals hour for task with SAP ID: ' . $sapID . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
}
