<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing cost centers from database using dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class CostCenterDibiMapper extends DibiMapper implements ICostCenterMapper
{
	/**
	 * Creates CostCenter instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	CostCenter 
	 */
    private function loadCostCenter(DibiRow $row)
    {
		$center = new CostCenter();
		$center->ID = $row->CostCenter_ID;
		$center->Name = $row->Name;
		$center->TaskNamePrefix = $row->TaskNamePrefix;
		$center->Plant = $row->Plant;
		$center->Status = $row->Status;
		return $center;
    }

	/**
	 * Creates CostCenterLabourRate instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	CostCenterLabourRate 
	 */
    private function loadCostCenterLabourRates(DibiRow $row)
    {
		$rates = new CostCenterLabourRates();
		$rates->Year = $row->Year;

		$column_names = array(
			'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
		);
		
        for ($i = 0; $i < count($column_names); $i++) 
			$rates->setMonthlyHourRate($i + 1, $row->$column_names[$i]);
		return $rates;
    }	
	
    /**
     * Saves cost center.
     * 
     * @param CostCenter $costCenter cost center.
     * @return boolean Returns true if cost center is successfully saved, otherwise false.
     */
    public function save(CostCenter $costCenter)
    {
        try
        {
            $array = array();
            $array['CostCenter_ID'] = $costCenter->ID;
			$array['Name'] = $costCenter->Name;
//			$array['TaskNamePrefix'] = $costCenter->TaskNamePrefix;
			$array['Plant'] = $costCenter->Plant;
			$array['Status'] = 1;
            $db_result = $this->conn->insert('CostCenter', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving Work Center with id = " . $costCenter->ID . " failed. ".$e->getMessage());
            return false;
        }
    }

    /**
     * Updates cost center.
     * 
     * @param CostCenter $costCenter cost center.
     * @return boolean Returns true if cost center is successfully updated, otherwise false.
     */
    public function update(CostCenter $costCenter)
    {
        try
        {
            $array = array();
            $array['Name'] = $costCenter->Name;
//			$array['TaskNamePrefix'] = $costCenter->TaskNamePrefix;
			$array['Plant'] = $costCenter->Plant;
			$array['Status'] = $costCenter->Status;
            $cost_center_db_result = $this->conn->update('CostCenter', $array)->where('[CostCenter_ID] = %s', $costCenter->ID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Work Center update with id = ".$costCenter->ID." failed. ".$e->getMessage());
            return false;
        }
    }
	
    /**
     * Saves cost center labour rates.
     * 
     * @param CostCenterLabourRates $costCenterLabourRates
     * @return boolean Returns true if cost center labour rates is successfully saved, otherwise false.
     */
    public function saveRates(CostCenterLabourRates $costCenterLabourRates)
    {
        try
        {
			$array = array();
			$array['CostCenter_ID'] = $costCenterLabourRates->getID();
			$array['Year'] = $costCenterLabourRates->getYear();
			$array['Jan'] = $costCenterLabourRates->getMonthlyHourRate( 1);
			$array['Feb'] = $costCenterLabourRates->getMonthlyHourRate( 2);
			$array['Mar'] = $costCenterLabourRates->getMonthlyHourRate( 3);
			$array['Apr'] = $costCenterLabourRates->getMonthlyHourRate( 4);
			$array['May'] = $costCenterLabourRates->getMonthlyHourRate( 5);
			$array['Jun'] = $costCenterLabourRates->getMonthlyHourRate( 6);
			$array['Jul'] = $costCenterLabourRates->getMonthlyHourRate( 7);
			$array['Aug'] = $costCenterLabourRates->getMonthlyHourRate( 8);
			$array['Sep'] = $costCenterLabourRates->getMonthlyHourRate( 9);
			$array['Oct'] = $costCenterLabourRates->getMonthlyHourRate(10);
			$array['Nov'] = $costCenterLabourRates->getMonthlyHourRate(11);
			$array['Dec'] = $costCenterLabourRates->getMonthlyHourRate(12);
			$cost_center_db_result = $this->conn->insert('Rate', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Work Center Labour Rates update with id = ".$costCenter->ID." and year = ". $costCenterLabourRates->Year ." failed. ".$e->getMessage());
            return false;
        }
    }

    /**
     * Updates cost center labour rates.
     * 
     * @param CostCenter $costCenter cost center.
     * @return boolean Returns true if cost center labour rates is successfully updated, otherwise false.
     */
    public function updateRates(CostCenter $costCenter)
    {
        try
        {
            foreach ($costCenter->getLabourRateYears() as $year)
            {
                $costCenterLabourRates = $costCenter->getYearLabourRate($year);
                $array = array();
                $array['Jan'] = $costCenterLabourRates->getMonthlyHourRate( 1);
                $array['Feb'] = $costCenterLabourRates->getMonthlyHourRate( 2);
                $array['Mar'] = $costCenterLabourRates->getMonthlyHourRate( 3);
                $array['Apr'] = $costCenterLabourRates->getMonthlyHourRate( 4);
                $array['May'] = $costCenterLabourRates->getMonthlyHourRate( 5);
                $array['Jun'] = $costCenterLabourRates->getMonthlyHourRate( 6);
                $array['Jul'] = $costCenterLabourRates->getMonthlyHourRate( 7);
                $array['Aug'] = $costCenterLabourRates->getMonthlyHourRate( 8);
                $array['Sep'] = $costCenterLabourRates->getMonthlyHourRate( 9);
                $array['Oct'] = $costCenterLabourRates->getMonthlyHourRate(10);
                $array['Nov'] = $costCenterLabourRates->getMonthlyHourRate(11);
                $array['Dec'] = $costCenterLabourRates->getMonthlyHourRate(12);
                $cost_center_db_result = $this->conn->update('Rate', $array)->where('[CostCenter_ID] = %s AND [Year] = %i', $costCenter->ID, $costCenterLabourRates->Year)->execute();
            }
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Work Center Labour Rates update with id = ".$costCenter->ID." and year = ". $costCenterLabourRates->Year ." failed. ".$e->getMessage());
            return false;
        }
    }
	
    /**
     * Returns cost center labour rate by cost center id and year.
     * 
     * @param string $id Cost center ID.
     * @param int $year Year of whitch rates should be found.
     * @return	CostCenterLabourRate or empty CostCenterLabourRate if not found.
     */
    public function findCostCenterLabourRates($id, $year)
	{
		$rates_result = $this->conn->query('SELECT * FROM [Rate] WHERE [CostCenter_ID] = %s AND [Year] = %u', $id, $year)->fetch();
		if($rates_result)
		{
			if ($rates_result->count() > 0)
				return $this->loadCostCenterLabourRates($rates_result);
		}
		else
		{
			$rate = new CostCenterLabourRates();
			$rate->Year = $year;
			Debugger::log("Labour rate for work center with ID: " . $id . " and year: " . $year . " doesnt exists!", Debugger::ERROR);
			return $rate;
		}
	}

	/**
	 * Returns cost center labour rates array by cost center id.
	 * 
	 * @param string $id Cost center ID.
	 * @return	array of CostCenterLabourRate.
	 */
	public function findAllCostCenterLabourRates($id)
	{
		$rates_db_result = $this->conn->query('SELECT * FROM [Rate] WHERE [CostCenter_ID] = %s ORDER BY [Year]', $id)->fetchAll();
		$rates = array();
		foreach ($rates_db_result as $row)
			$rates[] = $this->loadCostCenterLabourRates($row);
		return $rates;
	}

	/**
	 * Returns cost center with defined labour rate for every year.
	 * 
	 * @param string $id Cost center ID.
	 * @param int $year Year of whitch rates should be found.
	 * @return	CostCenterLabourRate or NULL if not found.
	 */
	public function findCostCenter($id)
	{
		$center_db_result = $this->conn->query('SELECT * FROM [CostCenter] WHERE [CostCenter_ID] = %s', $id)->fetch();
		if ($center_db_result)
		{
			$center = $this->loadCostCenter($center_db_result);
			$rates = $this->findAllCostCenterLabourRates($id);
			foreach ($rates as $rate)
				$center->setYearLabourRate($rate->Year, $rate);
			return $center;
		}
		else
			return NULL;
	}

	/**
	 * Returns all cost centers as an array sorted by name.
	 * 
	 * @return	array associative array of CostCenter as value and its ID as a key
	 */
	public function findAllCostCenters()
	{
		$centers_db_result = $this->conn->query('SELECT * FROM [CostCenter] ORDER BY [Name] ASC')->fetchAll();
		$centers = array();
		foreach ($centers_db_result as $row)
		{
			$costCenter = $this->loadCostCenter($row);
			$rates = $this->findAllCostCenterLabourRates($costCenter->ID);
			foreach ($rates as $rate)
			{
				$costCenter->setYearLabourRate($rate->Year, $rate);
			}
			$centers[$costCenter->ID] = $costCenter;
		}
		return $centers;
	}

	/**
	 * Returns sum of costs on current task in month in a year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $month Month index (0-11).
	 * @param int $year Year.
	 * @param string $currency Currency code (only USD or CZK accepted)
	 * @return	float
	 */
	public function findActualTaskCostsMonthSummary($task_id, $month, $year, $currency)
	{
		if (!(($month >= 0) && ($month <= 11)))
			throw new InvalidArgumentException('Month index must be integer in range <0,11>. Invalid value:'.$month);
		$curr_code = strtoupper($currency);
		if (!(($curr_code == 'USD') || ($curr_code == 'CZK')))
			throw new InvalidArgumentException('Unsupported currency code. Only USD and CZK supported. Invalid value:'.$currency);
		$costs = 0.0 + $this->conn->query('SELECT SUM([Value_'.$curr_code.']) AS [costs] FROM [Task],[ActualsCost] 
			WHERE [Task].[Task_ID]=[ActualsCost].[Task_ID]
			AND [Task].[Task_ID] = %u
			AND [ActualsCost].[FiscalMonth] = %u
			AND [ActualsCost].[Year] = %u', $task_id, $month, $year)->fetchSingle();
		return $costs;
	}
    
    /**
     * Returns sum of costs on current task in a year.
     * 
     * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @param string $currency Currency code (only USD or CZK accepted)
     * @return MonthArray with float values with hours for every month in a year.
     */
    public function findActualCostsYearSummary($task_id, $year, $currency)
    {
		$curr_code = strtoupper($currency);
		if (!(($curr_code == 'USD') || ($curr_code == 'CZK')))
			throw new InvalidArgumentException('Unsupported currency code. Only USD and CZK supported. Invalid value:'.$currency);
		$costs_result = $this->conn->query('SELECT [ActualsCost].[FiscalMonth] AS [month], SUM([Value_'.$curr_code.']) AS [costs] FROM [Task],[ActualsCost]
            WHERE [Task].[Task_ID]=[ActualsCost].[Task_ID]
            AND [Task].[Task_ID] = %u
            AND [ActualsCost].[Year] = %u
            GROUP BY [ActualsCost].[FiscalMonth]', $task_id, $year)->fetchAll();
        $costs = new MonthArray(0.0);
        foreach ($costs_result as $row)
            $costs[$row->month + 1] = $row->costs;
		return $costs;
    }	
    
    /**
     * Returns all cost centers prefixes as an array.
     * 
     * @return array of strings
     */
    public function findAllCostCentersPrefix()
    {
        $cost_centers = $this->findAllCostCenters();
        $prefix = array();
        foreach ($cost_centers as $center)
        {
            $prefix[] = substr($center->taskNamePrefix, 0, 3);
        }
        $prefix = array_unique($prefix);
        return $prefix;
    }
	
	/**
     * Gets CostCenters by CostCenterFilter. Returns every cost center that matches all set conditions.
     * @param CostCenterFilter $filter
     * @return array Of CostCenters or empty array
    */
	public function findByFilter(CostCenterFilter $filter)
	{
		$db_result = $this->conn->select('*')->from('[CostCenter]');
		
		if ($filter->Name)
            $db_result->where("[Name] LIKE %~like~", $filter->Name);
		
		if ($filter->ID)
            $db_result->where("[CostCenter_ID] LIKE %~like~", $filter->ID);
		
		if ($filter->Plant)
            $db_result->where("[Plant]LIKE %~like~", $filter->Plant);

        if ($filter->Status)
        {
            $statuses = $filter->getStatus();
			if(count($statuses) != 2)
				foreach ($statuses as $value => $status)
				{
					if($status == 'Active')
						$db_result->where("[Status] = %i", 1);
					else
						$db_result->where("[Status] = %i", 0);
				}
        }
		else
		{
			//No status selected, no wc should be displayed.
			$db_result->where("[Status] = %i", 2);
		}
		
		// get result
		$db_result->orderBy('[Name] ASC');
        $db_result->fetchAll();

        $array_costCenter = array();
        foreach($db_result as $row)
		{
			$costCenter = $this->loadCostCenter($row);
			$rates = $this->findAllCostCenterLabourRates($costCenter->ID);
			foreach ($rates as $rate)
			{
				$costCenter->setYearLabourRate($rate->Year, $rate);
			}
			$array_costCenter[$costCenter->ID] = $costCenter;
		}

        return $array_costCenter;
	}
	
	/**
	 * Returns cost center.
	 * 
	 * @param string $taskNamePrefix Task name prefix.
	 * @return	CostCenterLabourRate or NULL if not found.
	 */
	public function findCostCenterByTaskNamePrefix($taskNamePrefix)
	{
		$center_db_result = $this->conn->query('SELECT * FROM [CostCenter] WHERE [TaskNamePrefix] = %s', $taskNamePrefix)->fetch();
		if ($center_db_result)
		{
			$center = $this->loadCostCenter($center_db_result);
			return $center;
		}
		else
			return NULL;
	}
}
