<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for importing data - actual hours - from iASAP.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class IASAPActualsImportServiceTLC extends Object
{
	/** @const php.ini maximum execution time */
	const MAXIMUM_EXECUTION_TIME = 720; // 12 minutes
	/** @const Lock key */
	const LOCK_KEY = 'IASAP_ACTUALS_IMPORT_TLC';
	/** @const Possible results of import */
	const RESULT_SUCCESS = 0,
		  RESULT_WARNING = 1,
		  RESULT_ERROR = 2,
		  RESULT_NO_DATA = 3;
	
	/** @var DIContainer DI context with application repositories access */
    private $context;
	/** @var ILogger Logging facility */
    private $logger;
	/** @var ActualsReaderTLC iASAP access class */
	private $reader;
	/** @var DateTime Creation date of oldest requested data. Default value is stored in db table GlobalSetting. */
	private $start_date;
	/** @var DateTime Creation date of latest requested data. Default value is time of import invocation. */
	private $end_date;
	/** Array Array of all cost centers*/
	private $cachedWWorkCenters;
	
	/**
	 * @param DIContainer $context
	 * @param ILogger $logger
	 */
	public function __construct(DIContainer $context, ILogger $logger = NULL)
    {
		$this->context  = $context;
		$this->logger   = ($logger === NULL) ? (new NullLogger('')) : $logger;
		
		$this->reader = new ActualsReaderTLC($logger);
		
		$this->start_date = new DateTime(GlobalSettingReaderHelper::get('TLC.actuals.update.startDate'));
		$this->end_date = new DateTime();
		
		$this->cachedWWorkCenters = $this->context->costCenterRepository->findAllCostCenters();
	}
	
	/**
	 * Sets window between oldest and latest requested data item
	 * @param string $start_date Date in format YYYY-MM-DD
	 * @param string $end_date Optional date in format YYYY-MM-DD
	 * @throws InvalidArgumentException
	 */
	public function setRange($start_date, $end_date = NULL)
	{
		if ($this->validateDate($start_date) && $this->validateDate($end_date))
		{
		    $this->start_date = new DateTime($start_date);
		    $this->end_date = new DateTime($end_date);
		}
        else
		    throw new InvalidArgumentException('Start date and End date must be in format YYYY-MM-DD. Wrong params: ' . $start_date . ', ' . $end_date);
	}
	
	/**
	 * Runs actuals import from iASAP.
	 * Import is divided into time blocks. Each block represents fiscal month in given time range.
	 * @return int Result of operation in range of IASAPActualsImportServiceTLC::RESULT_*
	 * @throws IASAPActualsImportServiceTLCException
	 */
	public function runTimeAdminImport()
	{
		// adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new IASAPActualsImportServiceTLCException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
		$lock = $this->context->createLock(self::LOCK_KEY);
		if (!$lock->lock())
            throw new IASAPActualsImportServiceTLCException("TLC actuals import is already running.");
        // open logfile
        $this->logger->log('TLC (time admin) actuals import start: ' . date('Y-m-d H:i:s'), ILogger::INFO);

		try
		{
			$succ = 0; // number of successfully imported records
            $fail = 0; // number of records that couldn't be stored in db
			
			$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->start_date->format('U'));
			$end_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->end_date->format('U'));
			
			// get all tasks sap ids. We store actuals only for task from HON PT DB with flags ActualsImport = 0 
			$distinctTaskSAPIDs = $this->context->taskRepository->findDistinctLaborTaskSAPIDs();
			// we want to work just with tasks from Plant 3320 and 3330
			//TODO - for better performance cache work centers
			foreach ($distinctTaskSAPIDs as $task)
			{
				if($this->cachedWWorkCenters[$task->CostCenterID])
				{
					if(($this->cachedWWorkCenters[$task->CostCenterID]->Plant != 3320) && ($this->cachedWWorkCenters[$task->CostCenterID]->Plant != 3330))
						unset($distinctTaskSAPIDs[$task->SAPID]);
				}
				else
				{
					$this->logger->log('Task ' . $task->Name . ' (' . $task->SAPID . ') doesn\'t have defined  or has unknown work center!', ILogger::WARNING);
				}
			}
			
			//Deletes actuals from temp table
			if($this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN))
			{
				$this->logger->log('Temp actuals were deleted successfully.', ILogger::INFO);
			}
			else
			{
				$lock->unlock();
				throw new EChargeImportServiceException('Temp actuals were not deleted successfully.');
			}

			while ($fiscal_month->ToDateTime <= $end_fiscal_month->ToDateTime)
			{
				foreach ($this->reader->readTimeAdminActuals($fiscal_month->FromDateTime, $fiscal_month->ToDateTime) as $taskSAP_ID => $assigned)
				{
					if (array_key_exists($taskSAP_ID, $distinctTaskSAPIDs))// in array
					{
						foreach ($assigned as $eid => $months)
						{
							foreach ($months as $month => $hours)
							{
								
								$actuals = new ActualsTemp();
								$actuals->Eid = $eid;
								$actuals->Charge_number = $taskSAP_ID;
								$actuals->Year = $fiscal_month->year;
								$actuals->Fiscal_month = $month;
								$actuals->Hours = $hours;
								$actuals->Project_sap_id = ''; // TODO add project SAP ID
								$actuals->Source = ActualsTemp::SOURCE_IASAP_TIMEADMIN;
								if ($this->context->actualsTempRepository->add($actuals))
									$succ++;
								else
								    $fail++;
							}
						}
					}
				} // TODO - this while cycle doesnt need that many foreach cycles
				
				$fiscal_month = FiscalCalendar::getNextFiscalMonth($fiscal_month);
				if ($fiscal_month === NULL)
				{
					$msg = "IASAPActualsImportServiceTLC(time admin): TLC actuals import failed. Cannot get next fiscal month.";
					$this->logger->log($msg, ILogger::ERROR);
					Debugger::log($msg, Debugger::ERROR);
					$lock->unlock();
					return self::RESULT_ERROR;
				}
			}
			$actualsTemp = $this->context->actualsTempRepository->countActualsByRangeAndType($this->start_date, $this->end_date, ActualsTemp::SOURCE_IASAP_TIMEADMIN);
			$actualsCurrent = $this->context->actualsRepository->countActualsByRangeAndType($this->start_date, $this->end_date, ActualsTemp::SOURCE_IASAP_TIMEADMIN);
			$this->logger->log('Current actuals records: ' . $actualsCurrent, ILogger::INFO);
			$this->logger->log('Temp actuals records: ' . $actualsTemp, ILogger::INFO);
			// Compares temp actuals to current actuals
			if($actualsCurrent <= $actualsTemp)
			{
				//Deletes actuals from current table by range
				if($this->context->actualsRepository->deleteByRange($this->start_date, $this->end_date, Actuals::SOURCE_IASAP_TIMEADMIN))
				{
					$this->logger->log('Current actuals were deleted successfully.', ILogger::INFO);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN);
					throw new EChargeImportServiceException('Current actuals were not deleted successfully.');
				}
				//Moves actuals
				if($this->context->actualsTempRepository->moveAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN))
				{
					$this->logger->log('Temp actuals were moved successfully. TLC import ends.', ILogger::NOTICE);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN);
					throw new EChargeImportServiceException('Temp actuals were not moved successfully.');
				}
			}
			else
			{
				$lock->unlock();
				$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN);
				throw new EChargeImportServiceException('Count of current actuals is greater than count of temp actuals.');
			}
			$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN);
			$lock->unlock();

			if ($fail == 0)
			{
				$msg = "IASAPActualsImportServiceTLC(time admin): All $succ records from " . $this->start_date->format('Y-m-d') . " till " . $this->end_date->format('Y-m-d') . " imported successfully";
				$this->logger->log($msg, ILogger::NOTICE);
				Debugger::log($msg, Debugger::INFO);
			}
			else
			{
				$msg = "IASAPActualsImportServiceTLC(time admin): $succ actuals imported, $fail actuals thrown away (" . $this->start_date->format('Y-m-d') . " - " . $this->end_date->format('Y-m-d') . ")";
				$this->logger->log($msg, ILogger::WARNING);
				Debugger::log($msg, Debugger::WARNING);
				return self::RESULT_WARNING;
			}
		}
        catch (Exception $e)
		{
			$this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			return self::RESULT_ERROR;
		}

		if($succ == 0)
		{
			return self::RESULT_NO_DATA;
		}
		return self::RESULT_SUCCESS;
	}
	
	/**
	 * Runs actuals import from iASAP.
	 * Import is divided into time blocks. Each block represents fiscal month in given time range.
	 * @return int Result of operation in range of IASAPActualsImportServiceTLC::RESULT_*
	 * @throws IASAPActualsImportServiceTLCException
	 */
	public function runYPSDetailImport()
	{
		// adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new IASAPActualsImportServiceTLCException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
		$lock = $this->context->createLock(self::LOCK_KEY);
		if (!$lock->lock())
            throw new IASAPActualsImportServiceTLCException("TLC actuals import is already running.");
        // open logfile
        $this->logger->log('TLC (ysp detail) actuals import start: ' . date('Y-m-d H:i:s'), ILogger::INFO);
		
		try
		{
			$succ = 0; // number of successfully imported records
            $fail = 0; // number of records that couldn't be stored in db
			
			$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->start_date->format('U'));
			$end_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->end_date->format('U'));
			
			// get all tasks sap ids. We store actuals only for task from HON PT DB with flags ActualsImport = 0 
			$distinctTaskSAPIDs = $this->context->taskRepository->findDistinctLaborTaskSAPIDs();
			// we want to work with tasks not from Plant 3320 and 3330
			//TODO - for better performance cache work centers
			foreach ($distinctTaskSAPIDs as $task)
			{
				if($this->cachedWWorkCenters[$task->CostCenterID])
				{
					if(($this->cachedWWorkCenters[$task->CostCenterID]->Plant == 3320) || ($this->cachedWWorkCenters[$task->CostCenterID]->Plant == 3330))
						unset($distinctTaskSAPIDs[$task->SAPID]);
				}
				else
				{
					$this->logger->log('Task ' . $task->Name . ' (' . $task->SAPID . ') doesn\'t have defined or has unkown work center!', ILogger::WARNING);
				}
			}

			//Deletes actuals from temp table
			if($this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_YPSDETAIL))
			{
				$this->logger->log('Temp actuals were deleted successfully.', ILogger::INFO);
			}
			else
			{
				$lock->unlock();
				throw new EChargeImportServiceException('Temp actuals were not deleted successfully.');
			}

			while ($fiscal_month->ToDateTime <= $end_fiscal_month->ToDateTime)
			{
				foreach ($this->reader->readYPSDetailActuals($fiscal_month->FromDateTime, $fiscal_month->ToDateTime) as $taskSAP_ID => $assigned)
				{
					if (array_key_exists($taskSAP_ID, $distinctTaskSAPIDs))// in array
					{
						foreach ($assigned as $eid => $months)
						{
							foreach ($months as $month => $hours)
							{
								
								$actuals = new ActualsTemp();
								$actuals->Eid = $eid;
								$actuals->Charge_number = $taskSAP_ID;
								$actuals->Year = $fiscal_month->year;
								$actuals->Fiscal_month = $month;
								$actuals->Hours = $hours;
								$actuals->Project_sap_id = ''; // TODO add project SAP ID
								$actuals->Source = ActualsTemp::SOURCE_IASAP_YPSDETAIL;
								if ($this->context->actualsTempRepository->add($actuals))
									$succ++;
								else
								    $fail++;
							}
						}
					}
				} // TODO - this while cycle doesnt need that many foreach cycles
				
				$fiscal_month = FiscalCalendar::getNextFiscalMonth($fiscal_month);
				if ($fiscal_month === NULL)
				{
					$msg = "IASAPActualsImportServiceTLC(ysp detail): TLC actuals import failed. Cannot get next fiscal month.";
					$this->logger->log($msg, ILogger::ERROR);
					Debugger::log($msg, Debugger::ERROR);
					$lock->unlock();
					return self::RESULT_ERROR;
				}
			}
			$actualsTemp = $this->context->actualsTempRepository->countActualsByRangeAndType($this->start_date, $this->end_date, ActualsTemp::SOURCE_IASAP_YPSDETAIL);
			$actualsCurrent = $this->context->actualsRepository->countActualsByRangeAndType($this->start_date, $this->end_date, ActualsTemp::SOURCE_IASAP_YPSDETAIL);
			$this->logger->log('Current actuals records: ' . $actualsCurrent, ILogger::INFO);
			$this->logger->log('Temp actuals records: ' . $actualsTemp, ILogger::INFO);
			// Compares temp actuals to current actuals
			if($actualsCurrent <= $actualsTemp)
			{
				//Deletes actuals from current table by range
				if($this->context->actualsRepository->deleteByRange($this->start_date, $this->end_date, Actuals::SOURCE_IASAP_YPSDETAIL))
				{
					$this->logger->log('Current actuals were deleted successfully.', ILogger::INFO);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_YPSDETAIL);
					throw new EChargeImportServiceException('Current actuals were not deleted successfully.');
				}
				//Moves actuals
				if($this->context->actualsTempRepository->moveAll(ActualsTemp::SOURCE_IASAP_YPSDETAIL))
				{
					$this->logger->log('Temp actuals were moved successfully. TLC import ends.', ILogger::NOTICE);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_YPSDETAIL);
					throw new EChargeImportServiceException('Temp actuals were not moved successfully.');
				}
			}
			else
			{
				$lock->unlock();
				$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_YPSDETAIL);
				throw new EChargeImportServiceException('Count of current actuals is greater than count of temp actuals.');
			}
			$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_YPSDETAIL);
			$lock->unlock();

			if ($fail == 0)
			{
				$msg = "IASAPActualsImportServiceTLC(ysp detail): All $succ records from " . $this->start_date->format('Y-m-d') . " till " . $this->end_date->format('Y-m-d') . " imported successfully";
				$this->logger->log($msg, ILogger::NOTICE);
				Debugger::log($msg, Debugger::INFO);
			}
			else
			{
				$msg = "IASAPActualsImportServiceTLC(ysp detail): $succ actuals imported, $fail actuals thrown away (" . $this->start_date->format('Y-m-d') . " - " . $this->end_date->format('Y-m-d') . ")";
				$this->logger->log($msg, ILogger::WARNING);
				Debugger::log($msg, Debugger::WARNING);
				return self::RESULT_WARNING;
			}
		}
        catch (Exception $e)
		{
			$this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			return self::RESULT_ERROR;
		}
		
		if($succ == 0)
		{
			return self::RESULT_NO_DATA;
		}
		return self::RESULT_SUCCESS;
	}
	
	/**
	 * Runs actuals import from iASAP.
	 * Import is divided into time blocks. Each block represents fiscal month in given time range.
	 * @return int Result of operation in range of IASAPActualsImportServiceTLC::RESULT_*
	 * @throws IASAPActualsImportServiceTLCException
	 */
	public function runTaskImport($task)
	{
		// adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new IASAPActualsImportServiceTLCException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
        // open logfile
        $this->logger->log('TLC task actuals import start: ' . date('Y-m-d H:i:s'), ILogger::INFO);

		try
		{
			$succ = 0; // number of successfully imported records
            $fail = 0; // number of records that couldn't be stored in db
			
			$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->start_date->format('U'));
			$end_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->end_date->format('U'));
			
			$workCenter = $this->context->costCenterRepository->findCostCenter($task->CostCenterID);
			if(!$workCenter)
				throw new EChargeImportServiceException('Work center with ID:' . $task->CostCenterID . ' was not found!');
			
			//Deletes actuals from current table by range
			if($this->context->actualsRepository->deleteByRangeForTask($this->start_date, $this->end_date, $task, Actuals::SOURCE_IASAP_TIMEADMIN) && $this->context->actualsRepository->deleteByRangeForTask($this->start_date, $this->end_date, $task, Actuals::SOURCE_IASAP_YPSDETAIL))
			{
				$this->logger->log('Current actuals were deleted successfully.', ILogger::INFO);
			}
			else
			{
				$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_IASAP_TIMEADMIN);
				throw new EChargeImportServiceException('Current actuals were not deleted successfully.');
			}

			while ($fiscal_month->ToDateTime <= $end_fiscal_month->ToDateTime)
			{
				if(($workCenter->Plant != 3320) && ($workCenter->Plant != 3330))
				{
					foreach ($this->reader->readYPSDetailTaskActuals($fiscal_month->FromDateTime, $fiscal_month->ToDateTime, $task) as $taskSAP_ID => $assigned)
					{
						if ($taskSAP_ID == $task->getSAPID())// in array
						{
							foreach ($assigned as $eid => $months)
							{
								foreach ($months as $month => $hours)
								{

									$actuals = new Actuals();
									$actuals->Eid = $eid;
									$actuals->Charge_number = $taskSAP_ID;
									$actuals->Year = $fiscal_month->year;
									$actuals->Fiscal_month = $month;
									$actuals->Hours = $hours;
									$actuals->Project_sap_id = ''; // TODO add project SAP ID
									$actuals->Source = ActualsTemp::SOURCE_IASAP_YPSDETAIL;
									if ($this->context->actualsRepository->add($actuals))
										$succ++;
									else
										$fail++;
								}
							}
						}
					} // TODO - this while cycle doesnt need that many foreach cycles
				}
				else
				{
					foreach ($this->reader->readTimeAdminTaskActuals($fiscal_month->FromDateTime, $fiscal_month->ToDateTime, $task) as $taskSAP_ID => $assigned)
					{
						if ($taskSAP_ID == $task->getSAPID())// in array
						{
							foreach ($assigned as $eid => $months)
							{
								foreach ($months as $month => $hours)
								{

									$actuals = new Actuals();
									$actuals->Eid = $eid;
									$actuals->Charge_number = $taskSAP_ID;
									$actuals->Year = $fiscal_month->year;
									$actuals->Fiscal_month = $month;
									$actuals->Hours = $hours;
									$actuals->Project_sap_id = ''; // TODO add project SAP ID
									$actuals->Source = ActualsTemp::SOURCE_IASAP_TIMEADMIN;
									if ($this->context->actualsRepository->add($actuals))
										$succ++;
									else
										$fail++;
								}
							}
						}
					} // TODO - this while cycle doesnt need that many foreach cycles
				}
				
				$fiscal_month = FiscalCalendar::getNextFiscalMonth($fiscal_month);
				if ($fiscal_month === NULL)
				{
					$msg = "IASAPActualsImportServiceTLC: TLC task actuals import failed. Cannot get next fiscal month.";
					$this->logger->log($msg, ILogger::ERROR);
					Debugger::log($msg, Debugger::ERROR);
					return self::RESULT_ERROR;
				}
			}

			if ($fail == 0)
			{
				$msg = "IASAPActualsImportServiceTLC: All $succ records from " . $this->start_date->format('Y-m-d') . " till " . $this->end_date->format('Y-m-d') . " imported successfully";
				$this->logger->log($msg, ILogger::NOTICE);
				Debugger::log($msg, Debugger::INFO);
			}
			else
			{
				$msg = "IASAPActualsImportServiceTLC: $succ actuals imported, $fail actuals thrown away (" . $this->start_date->format('Y-m-d') . " - " . $this->end_date->format('Y-m-d') . ")";
				$this->logger->log($msg, ILogger::WARNING);
				Debugger::log($msg, Debugger::WARNING);
				return self::RESULT_WARNING;
			}
		}
        catch (Exception $e)
		{
			$this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			return self::RESULT_ERROR;
		}

		if($succ == 0)
		{
			return self::RESULT_NO_DATA;
		}
		return self::RESULT_SUCCESS;
	}
	
	/**
	 * Validates date format
	 * @param string $date To be validated
	 * @param string $format Validation pattern
	 * @return bool TRUE if date match format. FALSE otherwise.
	 */
	protected function validateDate($date, $format = 'Y-m-d')
	{
		if ($date === NULL)
			return TRUE;
		
	    $d = DateTime::createFromFormat($format, $date);
	    return $d && ($d->format($format) == $date);
	}
}

class IASAPActualsImportServiceTLCException extends Exception {}