<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing news from database using Dibi library.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class NewsDibiMapper extends DibiMapper implements INewsMapper
{

	/**
	 * Creates News instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	News 
	 */
    private function loadNews(DibiRow $row)
    {
		$news = new News();
		$news->ID = $row->News_ID;
		$news->Title = $row->Title;
		$news->Annotation = $row->Annotation;
		$news->PublishDate = $row->PublishDate;
		$news->Text = $row->Text;
		return $news;
    }	
	
	/**
	 * Saves new news.
	 * 
	 * @param News $news News.
	 */
	public function save(News $news)
	{
		Debugger::log('Not implemented yet.', Debugger::CRITICAL);
	}

	/**
	 * Returns cost center labour rate by cost center id and year.
	 * 
	 * @param string $id Cost center ID.
	 * @param int $year Year of whitch rates should be found.
	 * @return	News or NULL if not found.
	 */
	public function find($id)
	{
		$row = $this->conn->query('SELECT * FROM [News] 
			WHERE [News_ID] = %u', $id)->fetch();
		if ($row->count() > 0)
			return $this->loadNews($row);
		else
			return NULL;
	}
	
	/**
	 * Returns news that are were published before $date
	 * and max $count of news.
	 * 
	 * @param int $date Unix timestamp of date and time.
	 * @param int $count How many news should be returned.
	 * @return	array of News
	 */
	public function findLatestNews($date, $count)
	{
		$date_string = date('Y-m-d H:i:s', $date);
		$db_rows = $this->conn->query('SELECT * FROM [News] 
			WHERE [PublishDate] <= %s ORDER BY [PublishDate] DESC LIMIT %u', $date_string, $count)->fetchAll();
		$news_list = array();
		foreach ($db_rows as $row)
		{
			$news = $this->loadNews($row);
			$news_list[$news->ID] = $news;
		}
		return $news_list;
	}

	/**
	 * Returns all news published in given time range.
	 * 
	 * @param int $date_from Unix timestamp of from date and time.
	 * @param int $date_to Unix timestamp of from date and time.
	 * @param int $count How many news should be returned. 0 means all.
	 * @param int $offset How may news to skip.
	 * @return	array of News
	 */
	public function findNewsInRange($date_from, $date_to, $count = 0, $offset = 0)
	{
		$date_from_string = date('Y-m-d H:i:s', $date_from);
		$date_to_string = date('Y-m-d H:i:s', $date_to);
		if ($count > 0)
			$db_rows = $this->conn->query('SELECT * FROM [News] 
				WHERE [PublishDate] >= %s AND [PublishDate] <= %s ORDER BY [PublishDate] DESC LIMIT %u OFFSET %u', $date_from_string, $date_to_string, $count, $offset)->fetchAll();
		else
			$db_rows = $this->conn->query('SELECT * FROM [News] 
				WHERE [PublishDate] >= %s AND [PublishDate] <= %s ORDER BY [PublishDate] DESC', $date_from_string, $date_to_string)->fetchAll();
		$news_list = array();
		foreach ($db_rows as $row)
		{
			$news = $this->loadNews($row);
			$news_list[$news->ID] = $news;
		}
		return $news_list;
	}

	/**
	 * Returns sum of costs on current task in month in a year.
	 * 
	 * @param array $filter Attributes to filter.
	 * @param int $count Count of news.
	 * @param int $offset Skip this number of news.
	 * @return	array of News 
	 */
	public function findNewsByFilter($filter, $count, $offset = 0)
	{
		Debugger::log('Not implemented yet.', Debugger::CRITICAL);
	}
	
}

