<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * POAttributesAndBudget repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class POAttributesAndBudgetRepository extends Object
{
	/** @var IPOAttributesAndBudgetMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IPOAttributesAndBudgetMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'POAttributesAndBudget-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "POAttributesAndBudget"));
    }
    
	/**
	 * Saves POAttributesAndBudget.
	 * @param POAttributesAndBudget $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(POAttributesAndBudget $poAttr)
	{
		if($this->mapper->save($poAttr))
		{
			$this->multilog->log('POAttributesAndBudget with ID: ' . $poAttr->POAttributesAndBudgetID . ' and Budget: ' . $poAttr->Budget . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates POAttributesAndBudget.
	 * @param POAttributesAndBudget $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(POAttributesAndBudget $poAttr)
	{
		$oldObject = $this->find($poAttr->POAttributesAndBudgetID);
		if($this->mapper->update($poAttr))
		{
			$this->multilog->log('POAttributesAndBudget with ID: ' . $poAttr->POAttributesAndBudgetID . ' and Budget: ' . $poAttr->Budget . ' updated. ' . $this->multilog->differences($oldObject, $poAttr), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing POAttributesAndBudget.
	 * @param int $id POAttributesAndBudget ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		if($this->mapper->delete($id))
		{
			$this->multilog->log('POAttributesAndBudget with ID: ' . $id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds POAttributesAndBudget by ID.
	 * @param int $id POAttributesAndBudget ID.
	 * @return POAttributesAndBudget or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Finds all POAttributesAndBudgets.
	 * @return array Array of POAttributesAndBudget or empty array.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Finds all POAttributesAndBudgets.
	 * @param int $id Project ID
	 * @return array Array of POAttributesAndBudget or empty array.
	 */
	public function findByProjectID($id)
	{
		return $this->mapper->findByProjectID($id);
	}
	
	/**
	 * Sum of budget for project in current year.
	 * @param int $id Project ID
	 * @return float Or NULL if error occured
	 */
	public function budgetInCurrentYearByProjectID($id)
	{
		return $this->mapper->budgetByProjectIDAndYear($id, date('Y'));
	}
	
	/**
	 * Sum of budget for project in given year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return float Or NULL if error occured
	 */
	public function budgetByProjectIDAndYear($id, $year)
	{
		return $this->mapper->budgetByProjectIDAndYear($id, $year);
	}
	
	/**
	 * Sum of PO value for project in current year.
	 * @param int $id Project ID
	 * @return float Or NULL if error occured
	 */
	public function poValueInCurrentYearByProjectID($id)
    {
		return $this->mapper->poValueByProjectIDAndYear($id, date('Y'));
	}
	
	/**
	 * Sum of PO value for project in given year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return float Or NULL if error occured
	 */
	public function poValueByProjectIDAndYear($id, $year)
	{
		return $this->mapper->poValueByProjectIDAndYear($id, $year);
	}
	
	/**
	 * Sum of PO number for project in current year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return string
	 */
	public function poNumberInCurrentYearByProjectID($id)
	{
		return $this->mapper->poNumberByProjectIDAndYear($id, date('Y'));
	}
	
	/**
	 * Sum of PO number for project in given year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return string
	 */
	public function poNumberByProjectIDAndYear($id, $year)
	{
		return $this->mapper->poNumberByProjectIDAndYear($id, $year);
	}
}
