<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing Person - ECLicence relation from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class PersonECLicenceDibiMapper extends DibiMapper implements IPersonECLicenceMapper
{
	/**
	 * Creates PersonECLicence instance from DibiRow.
	 * @param DibiRow $row
	 * @return PersonECLicence
	 */
    private function load(DibiRow $row)
    {
		$relation = new PersonECLicence();
		$relation->EID = $row->EID;
		$relation->ECLicenceID = $row->ECLicence_ID;
		return $relation;
    }
	
	/**
	 * Saves Person - ECLicence relation
	 * @param PersonECLicence $relation
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(PersonECLicence $relation)
	{
		try
        {
            $array = array();
            $array['EID'] = $relation->EID;
            $array['ECLicence_ID'] = $relation->ECLicenceID;
            $db_result = $this->conn->insert('Person_ECLicence', $array)->execute();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving PersonECLicence relation: " . $relation->EID . "-" . $relation->ECLicenceID .  " failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Updates Person - ECLicence relation
	 * @param PersonECLicence $relation
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(PersonECLicence $relation)
	{
		try
        {
            $array = array();
            $array['EID'] = $relation->EID;
            $array['ECLicence_ID'] = $relation->ECLicenceID;
            $db_result = $this->conn->update('Person_ECLicence', $array)->where('[EID] = %i AND [ECLicence_ID] = %i', $relation->EID, $relation->ECLicenceID)->execute();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("PersonECLicence relation: " . $relation->EID . "-" . $relation->ECLicenceID . " update failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Deletes Person - ECLicence relation
	 * @param string $eid Person id.
	 * @param int $licence_id ECLicence id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($eid, $licence_id)
	{
		try
		{
		    $db_result = $this->conn->delete('Person_ECLicence')->where('[EID] = %s AND [ECLicence_ID] = %i', $eid, $licence_id)->execute();
			if ($db_result)
				Debugger::log("PersonECLicence relation: " . $eid ."-" . $licence_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of PersonECLicence relation: " . $eid ."-" . $licence_id . " failed.", Debugger::ERROR);
			return ($db_result) ? TRUE : FALSE;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of PersonECLicence relation: " . $eid ."-" . $licence_id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Deletes Person - ECLicence relations for given eid. 
	 * @param string $eid Person id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteByEID($eid)
	{
		try
		{
		    $db_result = $this->conn->delete('Person_ECLicence')->where('[EID] = %i', $eid)->execute();
			if ($db_result)
				Debugger::log("PersonECLicence relations for person EID = " . $eid . " were deleted.", Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log("Deleting of PersonECLicence relations for person EID = " . $eid . " failed.", Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of PersonECLicence relations for person EID = " . $eid . " failed: " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Finds Person - ECLicence relation by project id and ECLicence id.
	 * @param string $eid Person id.
	 * @param int $licence_id ECLicence id.
	 * @return PersonECLicence or NULL if not found.
	 */
	public function find($eid, $licence_id)
	{
		$db_result = $this->conn->query('SELECT * FROM [Person_ECLicence] WHERE [EID] = %s AND [ECLicence_ID] = %i', $eid , $licence_id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all Person - ECLicence relations
	 * @return array Array of PersonECLicence or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('[Person_ECLicence]')->fetchAll();     
		$relation_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $relation_array[] = $this->load($row);
        }		
		return $relation_array;
	}
	
	/**
	 * Finds all Person - ECLicence relations by person id
	 * @param string $eid Person id.
	 * @return array Array of PersonECLicence or empty array.
	 */
	public function findByEID($eid)
	{
		$db_result = $this->conn->select('*')->from('[Person_ECLicence]')->where('[EID] = %s', $eid)->fetchAll();     
		$relation_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $relation_array[$row->ECLicence_ID] = $this->load($row);
        }		
		return $relation_array;
	}
	
	/**
	 * Finds all Person - ECLicence relations by ECLicence id
	 * @param int $licence_id ECLicence id.
	 * @return array Array of PersonECLicence or empty array.
	 */
	public function findByECLicenceID($licence_id)
	{
		$db_result = $this->conn->select('*')->from('[Person_ECLicence]')->where('[ECLicence_ID] = %i', $licence_id)->fetchAll();     
		$relation_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $relation_array[] = $this->load($row);
        }		
		return $relation_array;
	}
}
