<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of task assignments
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class PersonTaskAssignmentRepository extends Object
{
	/** @var IPersonTaskAssignmentMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IPersonTaskAssignmentMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'WorkerAssignments-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "WorkerAssignments"));
    }	

	/**
	 * Finds month assignment by id of assignment.
	 * 
	 * @param int $id task assignment ID.
	 * @return	PersonTaskMonthAssignment 
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}

	/**
	 * Stores month assignment.
	 * 
	 * @param int $id task assignment ID.
	 * @return	boolean Returns TRUE if success, otherwise false.
	 */
	public function save(PersonTaskMonthAssignment $assignment)
	{
		$oldObject = $this->find($assignment->ID);
		if ($this->mapper->save($assignment))
		{
			if ($oldObject)
			{
				if($oldObject != $assignment)
				{
					$person = Environment::getContext()->personRepository->find($assignment->PersonID);
					$task = Environment::getContext()->taskRepository->find($assignment->TaskID);
					$this->multilog->log('Assignments ID: ' . $assignment->ID . ' for person: ' . $person->getReversedFullNameWithEID() . ', task(' . $task->SAPID . '): ' . $task->Name . ' and month: ' . $assignment->Month . ' updated. ' . $this->multilog->differences($oldObject, $assignment), ILogger::INFO);
				}
			}
			else
			{
				if ($assignment->getHours() > 0) // we do not save new empty assignments
				{
					$person = Environment::getContext()->personRepository->find($assignment->PersonID);
					$task = Environment::getContext()->taskRepository->find($assignment->TaskID);
					$this->multilog->log('Assignments ID: ' . $assignment->ID . ' for person: ' . $person->getReversedFullNameWithEID() . ', task(' . $task->SAPID . '): ' . $task->Name . ' and month: ' . $assignment->Month . ' saved.', ILogger::INFO);
				}
			}
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Deletes all person assignment from task in given year.
	 * 
	 * @param int $person_id Person ID.
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 */
	public function deleteByPersonAndYear($person_id, $task_id, $year)
	{
		if($this->mapper->deleteByPersonAndYear($person_id, $task_id, $year))
		{
			$this->multilog->log('Person: ' . $person_id . ' assignment for task: ' . $task_id . ' and year: ' . $year . ' was deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deleteserson assignment from task in given year and month.
	 * 
	 * @param int $person_id Person ID.
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @param int $month Month.
	 */
	public function deleteByPersonAndYearAndMonth($person_id, $task_id, $year, $month)
	{
		if($this->mapper->deleteByPersonAndYearAndMonth($person_id, $task_id, $year, $month))
		{
			$this->multilog->log('Person: ' . $person_id . ' assignment for task: ' . $task_id . ' and year: ' . $year . ' and month: ' . $month . ' was deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes all assignment from task in given year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 */
	public function deleteByTaskAndYearAssignment($task_id, $year)
	{
		if($this->mapper->deleteByTaskAndYearAssignment($task_id, $year))
		{
			$this->multilog->log('Assignment for task: ' . $task_id . ' and year: ' . $year . ' was deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Returns twelve task assigments for a person for given task ID in given year.
	 * Each for every fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array of 12 PersonTaskMonthAssignment where key is a month index.
	 */
	public function findPersonTaskYearAssignment($task_id, $year, $person_id)
	{
		return $this->mapper->findPersonTaskYearAssignment($task_id, $year, $person_id);
	}
	
	/**
	 * Returns twelve task assigments for a person for given task ID in given year.
	 * Each for every fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array of 12 PersonTaskMonthAssigment where key is a month index.
	 */
	public function findPersonTaskYearAndFutureAssignment($task_id, $year, $person_id)
	{
		return $this->mapper->findPersonTaskYearAndFutureAssignment($task_id, $year, $person_id);
	}

	/**
	 * Returns array of arrays of twelve task assigments 
	 * for every person that is assigned to given task ID in given year.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @return	array or arrays of 12 PersonTaskMonthAssignment 
	 *					where outer array keys are person IDs
	 *					and inner arrays keys are month indexes.
	 */
	public function findTaskYearAssignments($task_id, $year)
	{
		return $this->mapper->findTaskYearAssignments($task_id, $year);
	}

	/**
	 * Returns array twelve numbers with totals assignments for
	 * for every month in given year. It retuns sum of all person
	 * assignments to this task.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @return	MonthArray
	 */
	public function findTaskYearAssignmentTotals($task_id, $year)
	{
		$all_assignments = $this->mapper->findTaskYearAssignments($task_id, $year);
		$totals = new MonthArray((int) 0, false);
		foreach ($all_assignments as $eid => $assignments)
			for ($i = 1; $i <= 12; $i++)
				$totals[$i] += $assignments[$i]->getHours();
		return $totals;
	}

	/**
	 * Returns whether there was a change in task assignments after
	 * given datetime.
	 * Date/time parameter must be given as a Unix timestamp.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param int $ts_from date/time to compare whether there were changes after given date/time.
	 * @return	boolean
	 */
	public function isTaskYearAssignmentChangedSince($task_id, $year, $ts_from)
	{
		$all_assignments = $this->mapper->findTaskYearAssignments($task_id, $year);
		$date_time_from = date('Y-m-d H:i:s', $ts_from);
		$task_modified = false;
		foreach ($all_assignments as $eid => $assignments)
			for ($i = 1; $i <= 12; $i++)
			{
				// because of date time database format YYYY-MM-DD hh:mm:ss we can compare lexicographically
				if (strcmp($date_time_from, $assignments[$i]->getModified()) <= 0)
				{
					$task_modified = true;
					continue 2; // jump out of cycles
				}
			}
		return $task_modified;
	}
	
	/**
	 * Returns array task assigments 
	 * for every task person is assigned to in given month only if it
	 * is more than 0 hours.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month.
	 * Month indexes are 1 (January) - 12 (December). 
	 * 
	 * @param int $person_id Person ID.
	 * @param int $month month index (1-12).
	 * @param int $year year.
	 * @return	array or PersonTaskMonthAssignment 
	 *					where outer array keys are task IDs.
	 */
    public function findPersonMonthAssignments($person_id, $month, $year)
	{
		return $this->mapper->findPersonMonthAssignments($person_id, $month, $year);
	}

	/**
	 * Returns array of twelve values with total hours assigned every month
	 * to given person.
	 * 
	 * Month indexes are 1 (January) - 12 (December). 
	 * 
	 * @param int $person_id Person ID.
	 * @param int $year year.
	 * @return	array or integers where keys are month indexes.
	 */
    public function findPersonTotalYearAssignments($person_id, $year)
	{
		return $this->mapper->findPersonTotalYearAssignments($person_id, $year);
	}
	
	/**
	 * Returns wheter person is assigned to to any task year.
	 * 
	 * @param int $person_id Person ID.
	 * @return	boolean
	 */
    public function isPersonAssignedToAnyTasks($person_id)
    {
        return $this->mapper->isPersonAssignedToAnyTasks($person_id);
    }

	/**
	 * Returns wheter person is assigned to to task in given year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @param int $person_id Person ID.
	 * @return	boolean
	 */
    public function isPersonAssignedToTaskInYear($task_id, $year, $person_id)
    {
        $assignments = $this->mapper->findPersonTaskYearAssignment($task_id, $year, $person_id);
		// to check whether ther were assignment of the person already
		// we need to check modificaton dates (if none is set, than no assignments were set up)
		foreach ($assignments as $assignment)
			if ($assignment->Modified != '')
				return true;
		return false;
    }

	/**
	 * Returns wether task has any assignment in all years
	 * 
	 * @param int $task_id Tasl ID.
	 * @return	boolean
	 */
    public function hasTaskAssignmentAtAll($task_id)
    {
        return $this->mapper->hasTaskAssignmentAtAll($task_id);
    }
	
	/**
	 * Returns whether task assignment was transfered
	 * 
	 * @param int $fromTaskID Task ID.
	 * @param int $toTaskID Task ID.
	 * @param int $year Year.
	 * @return	boolean
	 */
	public function transferTaskAssignment($fromTaskID, $toTaskID, $year)
	{
		return $this->mapper->transferTaskAssignment($fromTaskID, $toTaskID, $year);
	}
	
	/**
	 * Returns whether person task assignment was transfered.
	 * 
	 * @param int $fromTaskID Task ID.
	 * @param int $toTaskID Task ID.
	 * @param int $eid Person EID.
	 * @param int $year Year.
	 * @return	boolean
	 */
	public function transferPersonTaskAssignment($fromTaskID, $toTaskID, $eid, $year)
	{
		return $this->mapper->transferPersonTaskAssignment($fromTaskID, $toTaskID, $eid, $year);
	}
}
