<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ProjectCharterBudgetHistory repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterBudgetHistoryRepository extends Object
{
    /** @var IProjectCharterBudgetHistoryMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IProjectCharterBudgetHistoryMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectCharterBudgetHistory-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Charter Budget History"));
    }
    
        /**
	 * Creates new project charter budget  history record.
	 * 
	 * @param ProjectCharterBudgetHistory $projectCharterBudgetHistory
	 * @return boolean
	 */
	public function save(ProjectCharterBudgetHistory $projectCharterBudgetHistory)
	{
		if($this->mapper->save($projectCharterBudgetHistory))
		{
			$this->multilog->log('Project Charter Budget History for charter with ID: ' . $projectCharterBudgetHistory->ProjectCharterID . ' and version: ' . $projectCharterBudgetHistory->Version . ' and Responsible EID: ' . $projectCharterBudgetHistory->Responsible . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
        
        /**
	 * Updates existing project charter budget  history record.
	 * 
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterBudgetHistory $projectCharterBudgetHistory
	 * @return boolean
	 */
	public function update(ProjectCharterBudgetHistory $projectCharterBudgetHistory)
	{
		$oldObject = $this->find($projectCharterBudgetHistory->ProjectCharterID, $projectCharterBudgetHistory->Version);
		if($this->mapper->update($projectCharterBudgetHistory))
		{
			$this->multilog->log('Project Charter Budget History for charter with ID: ' . $projectCharterBudgetHistory->ProjectCharterID . ' and version: ' . $projectCharterBudgetHistory->Version . ' and Responsible EID: ' . $projectCharterBudgetHistory->Responsible . ' updated. ' . $this->multilog->differences($oldObject, $projectCharterBudgetHistory), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Deletes existing project charter budget  history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		if($this->mapper->delete($projectCharterID, $version))
		{
			$this->multilog->log('Project Charter Budget History for charter with ID: ' . $projectCharterID . ' and version: ' . $version . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets project charter budget  history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return ProjectCharterBudgetHistory
	 */
	public function find($projectCharterID, $version)
	{
		return $this->mapper->find($projectCharterID, $version);
	}    

	/**
	 * Gets project charter budget history record, which is the last one
	 * submitted by certain person (indentified by EID). It's possible this record
	 * won't be the latest one from all histories for this charter,
	 * as someone else (CL - line manager) could have created a new budget
	 * version and attain the latest budget status.
	 * 
	 * But for the EID supplied, it will.
	 * 
	 * @param int $projectCharterID
	 * @param int $EID
	 * @return ProjectCharterBudgetHistory
	 */
	public function findLatestVersionByEID($projectCharterID, $EID)
	{
		return $this->mapper->findLatestVersionByEID($projectCharterID, $EID);
	}
	
        /**
	 * Gets project charter budget history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterBudgetHistory
	 */
	public function findLatestVersion($projectCharterID)
	{
		return $this->mapper->findLatestVersion($projectCharterID);
	}
	
	/**
	 * Gets all project charter history records by project charter id.
	 * 
	 * @param int $projectCharterID
	 * @return array Array of ProjectCharterBudgetHistory or empty array if error occured.
	 */
	public function findByProjectCharterID($projectCharterID)
	{
		return $this->mapper->findByProjectCharterID($projectCharterID);
	}
        
	/**
	 * Initializes flag LatestVersion
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID)
	{
			return $this->mapper->initLatestVersion($projectCharterID);
	}
}
