<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project planning charter from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
  */
class ProjectPlanningCharterDibiMapper extends DibiMapper implements IProjectPlanningCharterMapper
{
	/**
	 * Creates ProjectPlanningCharter instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectPlanningCharter
	 */
	private function load(DibiRow $row)
	{
		$projectPlanningCharter = new ProjectPlanningCharter();
		$projectPlanningCharter->ID = $row->ProjectPlanningCharter_ID;
		$projectPlanningCharter->ProjectID = $row->Project_ID;
		$projectPlanningCharter->Year = $row->Year;
		$projectPlanningCharter->Name = $row->Name;
		$projectPlanningCharter->Type = $row->Type;
		return $projectPlanningCharter;
	}

	/**
	 * Saves project planning charter.
	 * 
     * @param ProjectPlanningCharter $projectPlanningCharter Project planning charter
	 * @return boolean
	 */
	public function save(ProjectPlanningCharter $projectPlanningCharter)
	{
		try
		{
			$array = array();
			$array['Project_ID'] = $projectPlanningCharter->ProjectID;
			$array['Year'] = $projectPlanningCharter->Year;
			$array['Name'] = $projectPlanningCharter->Name;
			$array['Type'] = $projectPlanningCharter->Type;
			$budget_db_result = $this->conn->insert('ProjectPlanningCharter', $array)->execute();
			$projectPlanningCharter->ID = $this->conn->insertId();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Saving ProjectPlanningCharter failed. " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Updates project planning charter.
	 * 
	 * @param ProjectPlanningCharter $projectPlanningCharter Project planning charter
	 * @return boolean
	 */
	public function update(ProjectPlanningCharter $projectPlanningCharter)
	{
		try
		{
			$array = array();
			$array['Project_ID'] = $projectPlanningCharter->ProjectID;
			$array['Year'] = $projectPlanningCharter->Year;
			$array['Name'] = $projectPlanningCharter->Name;
			$array['Type'] = $projectPlanningCharter->Type;
			$budget_db_result = $this->conn->update('ProjectPlanningCharter', $array)->where('[ProjectPlanningCharter_ID] = %i', $projectPlanningCharter->getID())->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("ProjectPlanningCharter update with ID = " . $projectPlanningCharter->getID() . " failed. " . $e->getMessage());
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charter.
	 * 
	 * @param int $projectPlanningCharter_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharter_id)
	{
		try
		{
			$result = $this->conn->delete('ProjectPlanningCharter')->where('[ProjectPlanningCharter_ID] = %i', $projectPlanningCharter_id)->execute();
			if ($result)
				Debugger::log("Project planning charter with ID = " . $projectPlanningCharter_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project planning charter with ID = " . $projectPlanningCharter_id . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project planning charter with ID = " . $projectPlanningCharter_id . " failed: " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charters by project id and by planning year.
	 * 
	 * @param int $projectID Project ID.
	 * @param int $planningYear Year.
	 * @return boolean
	 */
	public function deleteByProjectIDAndYear($projectID, $planningYear)
	{
		try
		{
			$result = $this->conn->delete('ProjectPlanningCharter')->where('[Project_ID] = %i AND [Year] = %i', $projectID, $planningYear)->execute();
			if ($result)
				Debugger::log("Project planning charters with Project_ID = " . $projectID . " and Year = " . $planningYear . " were deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project planning charters with Project_ID = " . $projectID . " and Year = " . $planningYear . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project planning charters with Project_ID = " . $projectID . " and Year = " . $planningYear . " failed: " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Returns project planning charter by its id.
	 * 
	 * @param int $id ProjectPlanningCharter ID.
	 * @return ProjectPlanningCharter
	 */
	public function find($id)
	{
		$projectPlanningCharter_db_result = $this->conn->select('*')->from('ProjectPlanningCharter')->where('[ProjectPlanningCharter_ID] = %i', $id)->fetch();
		if ($projectPlanningCharter_db_result !== FALSE)
			return $this->load($projectPlanningCharter_db_result);
		else
			return NULL;
	}

	/**
	 * Returns project planning charter by project id.
	 * 
	 * @param int $projectID Project ID.
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findByProjectID($projectID)
	{
		$projectPlanningCharter_array = array();
		$projectPlanningCharter_db_result = $this->conn->select('*')->from('ProjectPlanningCharter')->where('[Project_ID] = %i', $projectID)->fetchAll();
		foreach ($projectPlanningCharter_db_result as $projectPlanningCharter)
		{
			$projectPlanningCharter_array[] = $this->load($projectPlanningCharter);
		}
		return $projectPlanningCharter_array;
	}
	
	/**
	 * Returns project planning charter by project id and by planning year.
	 * 
	 * @param int $projectID Project ID.
	 * @param int $planningYear Year.
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findByProjectIDAndYear($projectID, $planningYear)
	{
		$projectPlanningCharter_array = array();
		$projectPlanningCharter_db_result = $this->conn->select('*')->from('ProjectPlanningCharter')->where('[Project_ID] = %i AND [Year] = %i', $projectID, $planningYear)->fetchAll();
		foreach ($projectPlanningCharter_db_result as $projectPlanningCharter)
		{
			$projectPlanningCharter_array[] = $this->load($projectPlanningCharter);
		}
		return $projectPlanningCharter_array;
	}
	
	/**
	 * Returns rows related to Project ID with specified Type
	 * 
	 * @param int $projectID Project ID.
	 * @param int $year The year of planning.
	 * @param string $type Type of the records to find, e.g. 'HRS', 'SUB', 'TRV'.
	 * @return 
	 */
	public function findByProjectIDAndYearAndType($projectID, $year, $type)
	{
		$projectPlanningCharter_array = array();
		$projectPlanningCharter_db_result = $this->conn->select('*')->from('ProjectPlanningCharter')->where('[Project_ID] = %i AND [Year] = %i AND [Type] = %s', $projectID, $year, $type)->fetchAll();
		foreach ($projectPlanningCharter_db_result as $result)
		{
			$projectPlanningCharter_array[] = $this->load($result);
		}
		return $projectPlanningCharter_array;
	}
	
	/**
	 * Returns rows related to Project ID with specified Type
	 * 
	 * @param int $projectID Project ID.
	 * @param int $year The year of planning.
	 * @param string $type Type of the records to find, e.g. 'HRS', 'SUB', 'TRV'.
	 * @param string $name Name column. Used for labor tables.
	 * @return 
	 */
	public function findByProjectIDAndYearAndTypeAndName($projectID, $year, $type, $name)
	{
		$projectPlanningCharter_array = array();
		$projectPlanningCharter_db_result = $this->conn->select('*')->from('ProjectPlanningCharter')->where('[Project_ID] = %i AND [Year] = %i AND [Type] = %s AND [Name] = %s', $projectID, $year, $type, $name)->fetchAll();
		foreach ($projectPlanningCharter_db_result as $result)
		{
			$projectPlanningCharter_array[] = $this->load($result);
		}
		return $projectPlanningCharter_array;
	}
	
	/*
	 * Returns years, which are planned for project identified by $projectID
	 * 
	 * @param int $projectID Project ID.
	 * @return array
	 */
	public function findYearsByProjectId($projectID)
	{
		$projectPlanningCharterYears_array = array();
		$projectPlanningCharter_db_result = $this->conn->select('Year')
				->setFlag('DISTINCT')
				->from('ProjectPlanningCharter')->where('[Project_ID] = %i', $projectID)->fetchAll();
		foreach ($projectPlanningCharter_db_result as $result)
		{
			$projectPlanningCharterYears_array[$result->Year] = $result->Year;
		}
		return $projectPlanningCharterYears_array;
	}
	
	/**
	 * Returns all project planning charters.
	 * 
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findAll()
	{
		$projectPlanningCharter_array = array();
		$projectPlanningCharter_db_result = $this->conn->select('*')->from('ProjectPlanningCharter')->fetchAll();
		foreach ($projectPlanningCharter_db_result as $projectPlanningCharter)
		{
			$projectPlanningCharter_array[] = $this->load($projectPlanningCharter);
		}
		return $projectPlanningCharter_array;
	}
}
