<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ProjectPlanningCharterItem represents basic object for project planning charter item.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProjectPlanningCharterItem extends Object
{
	/* possible ProjectPlanningCharterItem types */
	const PROJECT_PLANNING_CHARTER_ITEM_TYPE_PERSON = 'Person';
	const PROJECT_PLANNING_CHARTER_ITEM_TYPE_ORGANIZATION_UNIT = 'OrganizationUnit';
	const PROJECT_PLANNING_CHARTER_ITEM_TYPE_SUBCONTRACTOR = 'Subcontractor';
	const PROJECT_PLANNING_CHARTER_ITEM_TYPE_OTHER = 'Other';

	/** @var int ProjectPlanningCharterItem ID */
	private $id;

	/** @var int ProjectPlanningCharter ID */
	private $projectPlanningCharterID;

	/** @var string EID of person */
	private $personID;

	/** @var int ID of organization unit(only tech. group). */
	private $organizationUnitID;

	/** @var int ID of subcontractor */
	private $subcontractorID;

	/** @var string Type */
	private $type;

	/** @var float Percentage of costs sharing among customer and supplier. */
	private $costShare = 0;
	
	/** @var float Labor rate */
	private $rate;

	public function __construct()
	{
		
	}

	/**
	 * Sets ProjectPlanningCharterItem ID.
	 * 
	 * @param int $id
	 */
	public function setID($id)
	{
		if (is_int($id + 0))
		{
			$this->id = $id;
		}
		else
		{
			throw new InvalidArgumentException('ID must be integer. Wrong param: ' . $id);
		}
	}

	/**
	 * Gets ProjectPlanningCharterItem ID.
	 * 
	 * @return int
	 */
	public function getID()
	{
		return $this->id;
	}

	/**
	 * Sets ProjectPlanningCharterID.
	 * 
	 * @param int $projectPlanningCharterID
	 */
	public function setProjectPlanningCharterID($projectPlanningCharterID)
	{
		if (is_int($projectPlanningCharterID + 0))
		{
			$this->projectPlanningCharterID = $projectPlanningCharterID;
		}
		else
		{
			throw new InvalidArgumentException('ProjectID must be integer. Wrong param: ' . $projectPlanningCharterID);
		}
	}

	/**
	 * Gets ProjectPlanningCharterID.
	 * 
	 * @return int
	 */
	public function getProjectPlanningCharterID()
	{
		return $this->projectPlanningCharterID;
	}

	/**
	 * Set EID of assigned person.
	 * @param string $eid
	 */
	public function setPersonID($eid)
	{
		$this->personID = Person::processEID($eid);
	}

	/**
	 * Gets EID of assigned person.
	 * @return string
	 */
	public function getPersonID()
	{
		return $this->personID;
	}

	/**
	 * Sets organization unit(only tech. group) ID
	 * @param int $technicalGroup Must be positive integer or NULL.
	 */
	public function setOrganizationUnitID($technicalGroup)
	{
		if (is_int($technicalGroup + 0) || ($technicalGroup === NULL))
			$this->organizationUnitID = $technicalGroup;
		else
			throw new InvalidArgumentException('OrganizationUnit ID must be positive integer or NULL. Wrong param: ' . $technicalGroup);
	}

	/**
	 * Gets organization unit(only tech. group) ID.
	 * @return int
	 */
	public function getOrganizationUnitID()
	{
		return $this->organizationUnitID;
	}

	/**
	 * Sets subcontractor ID.
	 * @param int $subcontractorID Must be positive integer or NULL.
	 */
	public function setSubcontractorID($subcontractorID)
	{
		if (is_int($subcontractorID + 0) || ($subcontractorID === NULL))
			$this->subcontractorID = $subcontractorID;
		else
			throw new InvalidArgumentException('Subcontractor ID must be positive integer or NULL. Wrong param: ' . $subcontractorID);
	}

	/**
	 * Gets subcontractor ID.
	 * @return int
	 */
	public function getSubcontractorID()
	{
		return $this->subcontractorID;
	}

	/**
	 * Sets Type.
	 * 
	 * @param string $type
	 */
	public function setType($type)
	{
		if (in_array($type, array_keys(ProjectPlanningCharterItem::getTypes())))
		{
			$this->type = $type;
		}
		else
		{
			throw new InvalidArgumentException('Type must be enum. Wrong param: ' . $type);
		}
	}

	/**
	 * Gets Type.
	 * 
	 * @return string
	 */
	public function getType()
	{
		return $this->type;
	}

	/**
	 * Sets CostShare.
	 * 
	 * @param float $costShare
	 */
	public function setCostShare($costShare)
	{
		if (is_float($costShare + 0.0) && ($costShare >= 0.0) && ($costShare <= 1.0))
		{
			$this->costShare = $costShare;
		}
		else
		{
			throw new InvalidArgumentException('CotShare must be float in range <0,1>. Wrong param: ' . $costShare);
		}
	}

	/**
	 * Gets CostShare.
	 * 
	 * @return float
	 */
	public function getCostShare()
	{
		return $this->costShare;
	}

	/**
	 * Sets rate value
	 * @param float $rate
	 * @throws InvalidArgumentException
	 */
	public function setRate($rate)
	{
		if (is_float($rate + 0.0) || is_null($rate))
		{
			$this->rate = $rate;
		}
		else
		{
			throw new InvalidArgumentException('Rate must be float value. Wrong param: ' . $rate);
		}
	}

	/**
	 * Gets rate value
	 * @return float
	 */
	public function getRate()
	{
		return $this->rate;
	}

	/**
	 * Returns array of available Type.
	 * 
	 * @return array
	 */
	public static function getTypes()
	{
		return array(
			self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_PERSON => self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_PERSON,
			self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_ORGANIZATION_UNIT => self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_ORGANIZATION_UNIT,
			self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_SUBCONTRACTOR => self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_SUBCONTRACTOR,
			self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_OTHER => self::PROJECT_PLANNING_CHARTER_ITEM_TYPE_OTHER
		);
	}

}
