<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project planning charter from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
  */
class ProjectPlanningCharterItemDibiMapper extends DibiMapper implements IProjectPlanningCharterItemMapper
{
	/**
	 * Creates ProjectPlanningCharterItem instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectPlanningCharterItem
	 */
	private function load(DibiRow $row)
	{
		$projectPlanningCharterItem = new ProjectPlanningCharterItem();
		$projectPlanningCharterItem->ID = $row->ProjectPlanningCharterItem_ID;
		$projectPlanningCharterItem->ProjectPlanningCharterID = $row->ProjectPlanningCharter_ID;
		$projectPlanningCharterItem->PersonID = $row->Person_ID;
		$projectPlanningCharterItem->OrganizationUnitID = $row->OrganizationUnit_ID;
		$projectPlanningCharterItem->SubcontractorID = $row->Subcontractor_ID;
		$projectPlanningCharterItem->Type = $row->Type;
		$projectPlanningCharterItem->CostShare = $row->CostShare;
		$projectPlanningCharterItem->Rate = $row->LabourRate;
		return $projectPlanningCharterItem;
	}

	/**
	 * Saves project planning charter.
	 * 
     * @param ProjectPlanningCharterItem $projectPlanningCharterItem Project planning charter
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterItem $projectPlanningCharterItem)
	{
		try
		{
			$array = array();
			$array['ProjectPlanningCharterItem_ID'] = $projectPlanningCharterItem->ID;
			$array['ProjectPlanningCharter_ID'] = $projectPlanningCharterItem->ProjectPlanningCharterID;
			$array['Person_ID'] = $projectPlanningCharterItem->PersonID;
			$array['OrganizationUnit_ID'] = $projectPlanningCharterItem->OrganizationUnitID;
			$array['Subcontractor_ID'] = $projectPlanningCharterItem->SubcontractorID;
			$array['Type'] = $projectPlanningCharterItem->Type;
			$array['CostShare'] = $projectPlanningCharterItem->CostShare;
			$array['LabourRate'] = $projectPlanningCharterItem->Rate;
			$budget_db_result = $this->conn->insert('ProjectPlanningCharterItem', $array)->execute();
			$projectPlanningCharterItem->ID = $this->conn->insertId();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Saving ProjectPlanningCharterItem failed. " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Updates project planning charter.
	 * 
	 * @param ProjectPlanningCharterItem $projectPlanningCharterItem Project planning charter
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterItem $projectPlanningCharterItem)
	{
		try
		{
			$array = array();
			$array['ProjectPlanningCharterItem_ID'] = $projectPlanningCharterItem->ID;
			$array['ProjectPlanningCharter_ID'] = $projectPlanningCharterItem->ProjectPlanningCharterID;
			$array['Person_ID'] = $projectPlanningCharterItem->PersonID;
			$array['OrganizationUnit_ID'] = $projectPlanningCharterItem->OrganizationUnitID;
			$array['Subcontractor_ID'] = $projectPlanningCharterItem->SubcontractorID;
			$array['Type'] = $projectPlanningCharterItem->Type;
			$array['CostShare'] = $projectPlanningCharterItem->CostShare;
			$array['LabourRate'] = $projectPlanningCharterItem->Rate;
			$budget_db_result = $this->conn->update('ProjectPlanningCharterItem', $array)->where('[ProjectPlanningCharterItem_ID] = %i', $projectPlanningCharterItem->getID())->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("ProjectPlanningCharterItem update with ID = " . $projectPlanningCharterItem->getID() . " failed. " . $e->getMessage());
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charter.
	 * 
	 * @param int $projectPlanningCharterItem_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharterItem_id)
	{
		try
		{
			$result = $this->conn->delete('ProjectPlanningCharterItem')->where('[ProjectPlanningCharterItem_ID] = %i', $projectPlanningCharterItem_id)->execute();
			if ($result)
				Debugger::log("Project planning charter item with ID = " . $projectPlanningCharterItem_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project planning charter item with ID = " . $projectPlanningCharterItem_id . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project planning charter item with ID = " . $projectPlanningCharterItem_id . " failed: " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Returns project planning charter by its id.
	 * 
	 * @param int $id ProjectPlanningCharterItem ID.
	 * @return ProjectPlanningCharterItem
	 */
	public function find($id)
	{
		$projectPlanningCharterItem_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')->where('[ProjectPlanningCharterItem_ID] = %i', $id)->fetch();
		if ($projectPlanningCharterItem_db_result !== FALSE)
			return $this->load($projectPlanningCharterItem_db_result);
		else
			return NULL;
	}

	/**
	 * Returns the count of rows with specified type
	 * 
	 * @param int $projectPlanningCharterID Project ID.
	 * @param array $types Type of the records to count, e.g. 'Person', 'OrganizationUnit'.
	 * @return int The count of rows with specified type.
	 */
	public function countRecordsByProjectPlanningCharterIDAndType($projectPlanningCharterID, $types)
	{
		$db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')->where('[ProjectPlaningCharter_ID] = %i %or', $projectPlanningCharterID, $type)->count();
		return $db_result;
	}
	
	/**
	 * Returns rows related to Planning Project Charter ID with specified Type
	 * 
	 * @param int $projectPlanningCharterID Planning Project Charter ID.
	 * @param string $type Type of the records to find, e.g. 'Person', 'OrganizationUnit', 'Subcontractor', 'Other'.
	 * @return array $projectPlanningCharterItem_array
	 */
	public function findByProjectPlanningCharterIDAndType($projectPlanningCharterID, $type)
	{
		$projectPlanningCharterItem_array = array();
		$projectPlanningCharterItem_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')
				->where('[ProjectPlanningCharter_ID] = %i AND [Type] = %s', $projectPlanningCharterID, $type)->fetchAll();
		foreach ($projectPlanningCharterItem_db_result as $result)
		{
			$projectPlanningCharterItem_array[] = $this->load($result);
		}
		return $projectPlanningCharterItem_array;
	}
	
	/**
	 * Returns project planning charter item by project planning charter id.
	 * 
	 * @param int $projectPlanningCharterID ProjectPlanningCharter ID.
	 * @return array of ProjectPlanningCharterItem or empty array if not found
	 */
	public function findByProjectPlanningCharterID($projectPlanningCharterID)
	{
		$projectPlanningCharterItem_array = array();
		$projectPlanningCharterItem_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')->where('[ProjectPlanningCharter_ID] = %i', $projectPlanningCharterID)->fetchAll();
		foreach ($projectPlanningCharterItem_db_result as $projectPlanningCharterItem)
		{
			$projectPlanningCharterItem_array[] = $this->load($projectPlanningCharterItem);
		}
		return $projectPlanningCharterItem_array;
	}
	
	/**
	 * Returns all project planning charters.
	 * 
	 * @return array of ProjectPlanningCharterItem or empty array if not found
	 */
	public function findAll()
	{
		$projectPlanningCharterItem_array = array();
		$projectPlanningCharterItem_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')->fetchAll();
		foreach ($projectPlanningCharterItem_db_result as $projectPlanningCharterItem)
		{
			$projectPlanningCharterItem_array[] = $this->load($projectPlanningCharterItem);
		}
		return $projectPlanningCharterItem_array;
	}
	
	/**
	 * 
	 * @param type $projectPlanningCharterID
	 * @return type
	 */
	public function findAllPeopleByProjectPlanningCharterID($projectPlanningCharterID)
	{
		$projectPlanningCharterItem_array = array();
		$projectPlanningCharterItem_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')
				->where('[ProjectPlanningCharter_ID] = %i AND [Type] = \'Person\'', $projectPlanningCharterID)->fetchAll();
		foreach ($projectPlanningCharterItem_db_result as $result)
		{
			$projectPlanningCharterItem_array[] = $this->load($result);
		}
		return $projectPlanningCharterItem_array;
	}
	
	/**
	 * 
	 * @param type $projectPlanningCharterID
	 * @param type $orgUnitID
	 * @return type
	 */
	public function findAllOrgUnitsByProjectPlanningCharterIDByOrganizationUnit_ID($projectPlanningCharterID, $orgUnitID)
	{
		$projectPlanningCharterItem_array = array();
		$projectPlanningCharterItem_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItem')
				->where('[ProjectPlanningCharter_ID] = %i AND [Type] = \'OrganizationUnit\''
						. 'AND [OrganizationUnit_ID] = %i', $projectPlanningCharterID, $orgUnitID)->fetchAll();
		foreach ($projectPlanningCharterItem_db_result as $result)
		{
			$projectPlanningCharterItem_array[] = $this->load($result);
		}
		return $projectPlanningCharterItem_array;
	}
}
