<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\ProjectReport
 */

/**
 * ProjectEACReport provides reporting of EAC of reports.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectEACReporter extends Object
{
    /** @const */
    const KEY_LABOR_PLANNED_HOURS_MONTH = 'labor_planned_hours_month',
          KEY_LABOR_EAC_HOURS_MONTH = 'labor_eac_hours_month',
          KEY_LABOR_PLAN_EAC_HOURS_MONTH_DELTA = 'labor_plan_eac_hours_month_delta',
          KEY_LABOR_PLANNED_COSTS_MONTH = 'labor_planned_costs_month',
          KEY_LABOR_EAC_COSTS_MONTH = 'labor_eac_costs_month',
          KEY_LABOR_PLAN_EAC_COSTS_MONTH_DELTA = 'labor_plan_eac_costs_month_delta',
          KEY_NON_LABOR_PLANNED_COSTS_MONTH = 'non_labor_planned_costs_month',
          KEY_NON_LABOR_EAC_COSTS_MONTH = 'non_labor_eac_costs_month',
          KEY_NON_LABOR_PLAN_EAC_COSTS_MONTH_DELTA = 'non_labor_plan_eac_costs_month_delta',
	      KEY_PLANNED_COSTS_MONTH_TOTAL = 'planned_costs_month_total',
	      KEY_EAC_COSTS_MONTH_TOTAL = 'eac_costs_month_total',
		  KEY_PLANNED_EAC_TOTAL_DELTA = 'planned_eac_total_delta';
    
	/** @var Project */
	private $project;
	/** @var WorkPackage */
	private $workpackage;
	/** @var Task */
	private $task;
	/** @var int */
	private $year;
	/** @var DIContainer */
	private $context;
	/** @var array Contains all WorkPackages to obtain data from for EAC Control (recursion applied) */
	private $WPsToReport;

	/** @var int Index of fiscal month */
	private $cachedFirstEACMonth;
	/** @var array */
	private $cachedTaskList;
	
    public function __construct(DIContainer $context, $control_item, $year)
    {
        $this->project = new Project();
        $this->workpackage = new WorkPackage();
        $this->task = new Task();
        $this->year = $year;
		$this->context = $context;
        $this->cachedTaskList = NULL;
		$this->WPsToReport = array();
        
        if ($control_item instanceof Project)
            $this->project = $control_item;
        elseif ($control_item instanceof WorkPackage)
            $this->workpackage = $control_item;
        elseif ($control_item instanceof Task)
        	$this->task = $control_item;
        
        if ($this->workpackage->ID)
        {
            // Temporary loading of project id
            // TODO project id should be loaded automatically in find method
            $this->workpackage->ProjectID = $this->context->workPackageRepository->getProjectID($this->workpackage->ID);
            $this->project = $this->context->projectRepository->find($this->workpackage->ProjectID);
        }
        elseif ($this->task->ID)
        	$this->project = $this->context->projectRepository->find($this->task->ProjectID);

        if (!$this->project)
            $this->project = new Project();
    }

	/**
	 * Returns number <1, 12> of updatable month in current year
	 * according to current date and fiscal calendar.
	 * If reporting se to previous years, 0 si returned.
	 * 
	 * @return int value in <1,12> or 0 if no month  (for plan for previous years)
	 */
	public function getFirstEACMonth()
	{
		if (FiscalCalendar::currentYear() <= $this->year)
		{
			if ($this->cachedFirstEACMonth === NULL) // not found yet
			{
				if (FiscalCalendar::currentYear() < $this->year)
					$this->cachedFirstEACMonth = 1;
				else
				{
					$current_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp(time());
					$this->cachedFirstEACMonth = $current_fiscal_month->getIndex() + 1;
				}
			}
			return $this->cachedFirstEACMonth;
		}
		else
			return 0;
	}
    
	/**
	 * Returns all project tasks as array.
	 * Caches task list.
	 * Warning: possible problem when task list become changed during usage of report.
	 * @return array of Task 
	 */
	private function getTaskList()
	{
		if ($this->cachedTaskList === NULL)
        {
            if ($this->workpackage->ID)
			{
				// Tasks just of this WP (not any from nested WPs)
			    $this->cachedTaskList = $this->context->
					taskRepository->findAllWorkPackageTasks($this->workpackage->ID);
				
				// there could be nested Workpackages with more Tasks
				$this->findAllWPsToReport($this->workpackage->ID);

				// now get all of its children WPs and add theirs Tasks to the list
				foreach ($this->WPsToReport as $wp)
				{
					$WPTasks = $this->context->taskRepository
							->findAllWorkPackageTasks($wp->ID);

					$this->cachedTaskList += $WPTasks;
				}
			}
            elseif ($this->task->ID)
            	$this->cachedTaskList = array($this->task->ID => $this->task);
            else
                $this->cachedTaskList = $this->context->taskRepository
                    ->findPlannedProjectTasksInYear($this->project->ID, $this->year);
        }
		return $this->cachedTaskList;
	}
	
    /**
     * Returns Project.
     * @return Project
     */
    public function getProject()
    {
        return $this->project;
    }
    
	/**
	 * Returns totals (sum) of planned hours on project.
	 * 
	 * @return MonthArray with float values with hours for every month in a year.
	 */
	public function getProjectLabourPlannedHoursMonthTotals(CostCenter $costCenter = NULL)
	{
		$totals = new MonthArray((float) 0, false);
		$tasks = $this->getTaskList();
		foreach ($tasks as $task_id => $task) 
		{
			if ($task->isLabourTask())
			{
                if ($costCenter === NULL || ($costCenter->ID == $task->CostCenterID))
                {
                    $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->getID(), $this->year);
                    if ($task_plan)
                    {
                    	for ($month = 1; $month <= 12; $month++)
                        	$totals[$month] += $task_plan->getMonthPlan($month)->getHours();
                    }
                }
			}
		}
		return $totals;
	}

	/**
	 * Returns totals (sum) of actual and assigned hours on project.
	 * Returns actuals until the last finished month and assigned hours
	 * from current month till the end of the year.
	 * 
	 * If year is already finished, returns only actuals.
	 * 
	 * @return MonthArray with float values with hours for every month in a year.
	 */
	public function getProjectLabourEACHoursMonthTotals(CostCenter $costCenter = NULL)
	{
		$totals = new MonthArray((float) 0.0, false);
		$tasks = $this->getTaskList();
		$eac_start_from = $this->getFirstEACMonth();
		if ($eac_start_from == 0) // year already ended
			$eac_start_from = 13;
		foreach ($tasks as $task_id => $task) 
		{
			if ($task->isLabourTask())
			{
                if ($costCenter === NULL || ($costCenter->ID == $task->CostCenterID))
                {
                    // actuals
                    if ($eac_start_from > 1)
                    {
                        $year_work = $this->context->workRepository->findTaskYearWork($task_id, $this->year);
                        for ($month = 1; $month < $eac_start_from; $month++)
                            $totals[$month] += $year_work[$month];
                    }
                    // assignments totals
                    if ($eac_start_from < 13)
                    {
                        $assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignmentTotals($task_id, $this->year);
                        for ($month = $eac_start_from; $month <= 12; $month++)
                            $totals[$month] += $assignments[$month];
                    }
                }
			}
		}
		return $totals;		
	}

    /**
	 * Returns totals (sum) of actual and assigned hours on project for labour tasks.
	 * Returns actuals until the last finished month and then assigned hours
	 * from current month till the end of the year.
	 * 
	 * If year is already finished, returns only actuals.
	 * 
	 * @param Task $task.
	 * @return MonthArray with float values with hours for every month in a year.
	 */
	public function getTaskLabourEACHoursMonthTotals($task)
	{
		$totals = new MonthArray((float) 0.0, false);
		$eac_start_from = $this->getFirstEACMonth();
        
		if ($eac_start_from == 0) // year already ended
			$eac_start_from = 13;
		if ($task->isLabourTask())
		{
            // actuals
            if ($eac_start_from > 1)
            {
                $year_work = $this->context->workRepository->findTaskYearWork($task->ID, $this->year);
                for ($month = 1; $month < $eac_start_from; $month++)
                    $totals[$month] += $year_work[$month];
            }
            // assignments totals
            if ($eac_start_from < 13)
            {
                $assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignmentTotals($task->ID, $this->year);
                for ($month = $eac_start_from; $month <= 12; $month++)
                    $totals[$month] += $assignments[$month];
            }
		}
		return $totals;
	}
    
    /**
	 * Returns totals (sum) of actual and assigned hours on project for person.
	 * Returns actuals until the last finished month and then assigned hours
	 * from current month till the end of the year.
	 * 
	 * If year is already finished, returns only actuals.
     * 
	 * @param string $person_id person ID.
	 * @param Task $task.
     * @param type $person_id
     * @return type 
     */
    public function getPersonLabourEacHoursMonthTotals($tasks, $person_id)
    {
		$totals = new MonthArray((float) 0.0, false);
		$eac_start_from = $this->getFirstEACMonth();
        
		if ($eac_start_from == 0) // year already ended
			$eac_start_from = 13;
		if ($tasks->isLabourTask())
		{
            // actuals
            if ($eac_start_from > 1)
            {                
                for ($month = 1; $month < $eac_start_from; $month++)
                {
                    $year_work = $this->context->workRepository->findPersonTaskMonthWork($person_id, $tasks->ID, $month-1, $this->year);
                    $totals[$month] += $year_work;                    
                }
            }
            // assignments totals
            if ($eac_start_from < 13)
            {
                $assignments = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($tasks->ID, $this->year, $person_id);
                for ($month = $eac_start_from; $month <= 12; $month++)
                    $totals[$month] += $assignments[$month]->hours;
            }
		}
		return $totals;		
    }
    
	/**
	 * Returns totals (sum) of actual spent budget on project for non-labour tasks.
	 * Returns actuals until the last finished month and then planned hours
	 * from current month till the end of the year.
	 * 
	 * If year is already finished, returns only actuals.
     * 
	 * @param Task $task.
	 * @return MonthArray with float values with budget for every month in a year.
	 */
	public function getTaskNonLabourEACCostsMonthTotals($task, $currency)
	{
		$totals = new MonthArray((float) 0.0, false);
		$eac_start_from = $this->getFirstEACMonth();
        
		if ($eac_start_from == 0) // year already ended
			$eac_start_from = 13;
		if (!$task->isLabourTask() || $task->costAsLabour)
		{
            // actuals
            if ($eac_start_from > 1)
            {                
                for ($month = 1; $month < $eac_start_from; $month++)
                    $totals[$month] = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task->ID, $month, $this->year, $currency);                
            }
            // assignment or plan totals
            if ($eac_start_from < 13)
            {
				$assignmentsArray = $this->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
                $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);
				for ($month = $eac_start_from; $month <= 12; $month++)
				{
					if(isset($assignmentsArray[$month-1])) // assignment exists
					{
						$totals[$month] = $assignmentsArray[$month-1]->Cost;
					}
					else // no assignment exists -> plan is default value if exists, otherwise 0 is returned
					{
						if($task_plan != null)
							$totals[$month] += $task_plan->getMonthPlan($month)->Budget;
					}
				}
            }
		}
		return $totals;
	}
	
	/**
	 * Returns totals (sum) of actual spent budget on project for cost as labour tasks.
	 * Returns actuals until the last finished month and then assigned hours
	 * from current month till the end of the year.
	 * 
	 * If year is already finished, returns only actuals.
     * 
	 * @param Task $task.
	 * @return MonthArray with float values with budget for every month in a year.
	 */
	public function getTaskCostAsLabourEACCostMonthTotals($task, $currency)
	{
		$totals = new MonthArray((float) 0.0, false);
		$eac_start_from = $this->getFirstEACMonth();
		$assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignmentTotals($task->ID, $this->year);
		$laborCenter = $this->context->costCenterRepository->findCostCenter($task->CostCenterID);
		$labourRates = $laborCenter->getYearLabourRate($this->year);
		if ($task->costAsLabour)
		{
			for($month = 1; $month < 13; $month++)
			{
				// past years
				if($eac_start_from == 0)
				{
					if($month == 1)
						$totals[$month] = 0;
					else
						$totals[$month] = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task->ID, $month, $this->year, $currency);
				}
				else // present and future year
				{
					if($month == 1)
						$totals[$month] = 0;
					else
					{
						// Cost actuals
						if ($month < $eac_start_from)
						{
							$totals[$month] = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task->ID, $month, $this->year, $currency);
						}
						// Labour actuals
						if($month == $eac_start_from)
						{
							$year_work = $this->context->workRepository->findTaskYearWork($task->ID, $this->year);
							if($eac_start_from == 12)
								$totals[$month] = $year_work[$month-1] * $labourRates->getMonthlyHourRate($month) + $assignments[$month] * $labourRates->getMonthlyHourRate($month);
							else
								$totals[$month] = $year_work[$month-1] * $labourRates->getMonthlyHourRate($month);
						}
						// assignments
						if ($month > $eac_start_from)
						{
							if($month == 12)
								$totals[$month] = $assignments[$month-1] * $labourRates->getMonthlyHourRate($month) + $assignments[$month] * $labourRates->getMonthlyHourRate($month);
							else
								$totals[$month] = $assignments[$month-1] * $labourRates->getMonthlyHourRate($month);
						}
					}	
				}
			}
		}
		return $totals;	
	}
    
	/**
	 * Returns deltas of totals (sum) between SAP planned hours and EAC hours 
	 * on project for every month.
	 * 
	 * If delta value is greater than 0, than there are more SAP planned hours
	 * than assigned/actual. If it is lower than 0, than there there was more
	 * hours spent/assigned on task than planned.
	 * 
	 * @return MonthArray with float values with delta hours for every month in a year.
	 */
	public function getProjectLabourPlanEACMonthDelta()
	{
		$sap = $this->getProjectLabourPlannedHoursMonthTotals();
		$eac = $this->getProjectLabourEACHoursMonthTotals();
		return $sap->minus($eac);
	}

	/**
	 * Returns totals (sum) of planned costs on project for labour tasks
	 * according to hour costs of workers (different task may have
	 * different cost center).
	 * 
	 * @return MonthArray with float values with costs for every month in a year.
	 * @throws InvalidStateException when problems with getting data to count costs
	 */
	public function getProjectLabourPlannedCostsMonthTotals()
	{
		$costs = new MonthArray((float) 0, false);
		
		$centers = $this->context->costCenterRepository->findAllCostCenters();
		$tasks = $this->getTaskList();
		foreach ($tasks as $task_id => $task) 
		{
			if ($task->isLabourTask())
			{
				$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->getID(), $this->year);
				
				if (!isset($centers[$task->getCostCenterID()]))
					throw new InvalidStateException("Unknown cost center ID: ".$task->getCostCenterID(). " for labor task ID: " . $task_id . ".");
				
				$rates = $centers[$task->getCostCenterID()]->getYearLabourRate($this->year);
				if ($rates === NULL)
					throw new InvalidStateException("No rate defined for cost center with ID: ".$task->getCostCenterID()." for year: " . $this->year. " for task ID: " . $task_id . '.');

				if ($task_plan)
				{
					for ($month = 1; $month <= 12; $month++)
						$costs[$month] += $task_plan->getMonthPlan($month)->getHours() * $rates->getMonthlyHourRate($month);
				}
			}
		}
		
		return $costs;
	}
	
	/**
	 * Returns total delta (total of all Planned - total of all EAC).
	 * 
	 * @return MonthArray with float values with costs for every month in a year.
	 */
	public function getProjectPlannedCostsEACCostsMonthTotals()
	{
		$labourPlannedCosts = $this->getProjectLabourPlannedCostsMonthTotals();
		$nonLabourPlannedCosts = $this->getProjectNonLabourPlannedCostsMonthTotals();
		$labourEACCosts = $this->getProjectLabourEACCostsMonthTotals();
		$nonLabourEACCosts = $this->getProjectNonLabourEACCostsMonthTotals();
		
		$planned = $labourPlannedCosts->plus($nonLabourPlannedCosts);
		$eac = $labourEACCosts->plus($nonLabourEACCosts);
		return $planned->minus($eac);
	}
	
	/**
	 * Returns totals (sum) of actual/assigned costs on project for labour tasks
	 * according to hour costs of workers.
	 * Returns actual costs until the last finished month and assigned costs
	 * from current month till the end of the year.
	 * 
	 * If year is already finished, returns only actual costs.
	 * 
	 * @return MonthArray with float values with costs for every month in a year.
	 */
	public function getProjectLabourEACCostsMonthTotals()
	{
		$costs = new MonthArray((float) 0, false);
		$centers = $this->context->costCenterRepository->findAllCostCenters();
		$tasks = $this->getTaskList();
		$eac_start_from = $this->getFirstEACMonth();
		if ($eac_start_from == 0) // year already ended
			$eac_start_from = 13;
		foreach ($tasks as $task_id => $task) 
		{
			if ($task->isLabourTask())
			{
				if (!isset($centers[$task->getCostCenterID()]))
					throw new InvalidStateException("Unknown cost center ID: ".$task->getCostCenterID(). " for labor task ID: " . $task_id .".");
				$rates = $centers[$task->getCostCenterID()]->getYearLabourRate($this->year);
				if ($rates === NULL)
					throw new InvalidStateException("No rate defined for cost center with ID: ".$task->getCostCenterID()." for year: " . $this->year. " for task ID: " . $task_id . '.');
				// actuals
				if ($eac_start_from > 1)
				{
					$year_work = $this->context->workRepository->findTaskYearWork($task_id, $this->year);
					for ($month = 1; $month < $eac_start_from; $month++)
						$costs[$month] += $year_work[$month] * $rates->getMonthlyHourRate($month);
				}
				// assignments totals
				if ($eac_start_from < 13)
				{
					$assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignmentTotals($task_id, $this->year);
					for ($month = $eac_start_from; $month <= 12; $month++)
						$costs[$month] += $assignments[$month] * $rates->getMonthlyHourRate($month);
				}
			}
		}
		return $costs;
	}
	
	/**
	 * Returns deltas of totals (sum) between SAP planned labour costs
	 * and actual/assigned costs on project for every month.
	 * 
	 * If delta value is greater than 0, than there SAP planned costs are bigger 
	 * than spent/assigned. If it is lower than 0, than costs spent/assigned
	 * were bigger than planned.
	 * 
	 * @return MonthArray with float values with delta hours for every month in a year.
	 * @throws InvalidStateException when problems with getting data to count costs
	 */
	public function getProjectLabourPlanEACCostsMonthTotalsDelta()
	{
		$sap = $this->getProjectLabourPlannedCostsMonthTotals();
		$actual = $this->getProjectLabourEACCostsMonthTotals();
		return $sap->minus($actual);
	}
	
	/**
	 * Returns totals (sum) of planned budget on project tasks for non-labour tasks.
	 * 
	 * @return MonthArray with float values with budget for every month in a year.
	 */
	public function getProjectNonLabourPlannedCostsMonthTotals()
	{
		$totals = new MonthArray((float) 0, false);
		$tasks = $this->getTaskList();
		foreach ($tasks as $task_id => $task) 
		{
			if (!$task->isLabourTask())
			{
				$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->getID(), $this->year);
				if ($task_plan)
				{
				    for ($month = 1; $month <= 12; $month++)
					    $totals[$month] += $task_plan->getMonthPlan($month)->getBudget();
				}
			}
		}
		return $totals;
	}
	
	/**
	 * Returns totals (sum) of actual spent budget on project tasks for non-labour tasks.
	 * 
	 * @return MonthArray with float values with budget for every month in a year.
	 */
	public function getProjectNonLabourEACCostsMonthTotals()
	{
		$costs = new MonthArray((float) 0, false);
		$tasks = $this->getTaskList();
		$eac_start_from = $this->getFirstEACMonth();
		if ($eac_start_from == 0) // year already ended
			$eac_start_from = 13;
		foreach ($tasks as $task_id => $task) 
		{
			if (!$task->isLabourTask())
			{
				// actuals
				if ($eac_start_from > 1)
				{
					for ($month = 1; $month < $eac_start_from; $month++)
						$costs[$month] += $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task->getID(), $month, $this->year, 'USD');
				}
				// assignment or plan totals
				if ($eac_start_from < 13)
				{
					$assignmentsArray = $this->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
					$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);
					for ($month = $eac_start_from; $month <= 12; $month++)
					{
						if(isset($assignmentsArray[$month-1])) // assignment exists
						{
							$costs[$month] += $assignmentsArray[$month-1]->Cost;
						}
						else // no assignment exists -> plan is default value if exists, otherwise 0 is returned
						{
							if($task_plan != null)
								$costs[$month] += $task_plan->getMonthPlan($month)->Budget;
						}
					}
				}
			}
		}
		return $costs;
	}
	
	/**
	 * Returns deltas of totals (sum) between SAP planned non-labour costs
	 * and actual costs on project for every month.
	 * 
	 * If delta value is greater than 0, than there is bigger SAP planned budget
	 * than spent/assigned. If it is lower than 0, than there was spent/assigned
	 * more than planned.
	 * 
	 * @return MonthArray with float values with delta hours for every month in a year.
	 */
	public function getProjectNonLabourCostsMonthTotalsDelta()
	{
		$sap = $this->getProjectNonLabourPlannedCostsMonthTotals();
		$actual = $this->getProjectNonLabourEACCostsMonthTotals();
		return $sap->minus($actual);
	}
	
	/**
	 * Returns totals (sum) of planned matterial costs on project.
	 * 
	 * @return MonthArray with float values with costs for every month in a year.
	 * @throws InvalidStateException when problems with getting data to count costs
	 */
	public function getProjectMaterialPlannedCostsMonthTotals()
	{
		$costs = new MonthArray((float) 0, false);
		
		$tasks = $this->getTaskList();
		foreach ($tasks as $task_id => $task) 
		{
			if (!$task->isLabourTask() && (substr($task->Name, 0, 3) == 'MAT'))
			{
				$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->getID(), $this->year);
				if ($task_plan)
				{
				    for ($month = 1; $month <= 12; $month++)
					    $costs[$month] += $task_plan->getMonthPlan($month)->getBudget();
				}
			}
		}
		
		return $costs;
	}
	
	/**
	 * Returns totals (sum) of planned travel costs on project.
	 * 
	 * @return MonthArray with float values with costs for every month in a year.
	 * @throws InvalidStateException when problems with getting data to count costs
	 */
	public function getProjectTravelPlannedCostsMonthTotals()
	{
		$costs = new MonthArray((float) 0, false);
		
		$tasks = $this->getTaskList();
		foreach ($tasks as $task_id => $task) 
		{
			if (!$task->isLabourTask() && (substr($task->Name, 0, 3) == 'TRV'))
			{
				$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->getID(), $this->year);
				if ($task_plan)
				{
				    for ($month = 1; $month <= 12; $month++)
					    $costs[$month] += $task_plan->getMonthPlan($month)->getBudget();
				}
			}
		}
		
		return $costs;
	}
	
	/**
	 * Returns totals (sum) of planned subcontracting costs on project.
	 * 
	 * @return MonthArray with float values with costs for every month in a year.
	 * @throws InvalidStateException when problems with getting data to count costs
	 */
	public function getProjectSubcontractingPlannedCostsMonthTotals()
	{
		$costs = new MonthArray((float) 0, false);
		
		$tasks = $this->getTaskList();
		foreach ($tasks as $task_id => $task) 
		{
			if (!$task->isLabourTask() && !(substr($task->Name, 0, 3) == 'MAT') && !(substr($task->Name, 0, 3) == 'TRV'))
			{
				$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->getID(), $this->year);
				if ($task_plan)
				{
				    for ($month = 1; $month <= 12; $month++)
					    $costs[$month] += $task_plan->getMonthPlan($month)->getBudget();
				}
			}
		}
		
		return $costs;
	}
	
	/**
	 * Recursive function gets all wp to report
	 * 
	 * @param int $workPackageID
	 */
	private function findAllWPsToReport($workPackageID)
	{
		$wpChildren = $this->context->workPackageRepository->findByParentID($workPackageID);
		$this->WPsToReport = array_merge($this->WPsToReport, $wpChildren);
		foreach ($wpChildren as $wp)
		{
			$this->findAllWPsToReport($wp->ID);
		}
	}
}
