<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project repository access class.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectRepository extends Object
{
    /** @var IProjectMapper */
 	private $mapper;
	
	/** @var boolean */
 	private $showFakeProjects;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IProjectMapper $mapper)
    {
        $this->mapper = $mapper;
		/*
		 * Temporary fix!
		 * Operation authenticated via token doesnt have available user stettings.
		 * Therefore SBE virtual leader set as current leader.
		 */
		$eid = Environment::getUser()->getId();
		if(is_null($eid))
			$eid = 'ENN0000';//ATE SBE leader eid hard coded. Needs to find better solution.
		$this->showFakeProjects = Environment::getContext()->userSettingRepository->find($eid)->ShowFakeProjects;
		$this->multilog = new MultiLogger();
		$batch_id = 'Project-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project"));
    }
	
	/**
	 * Function returns array of projects without fake projects.
	 * 
	 * @param array $projectsArray from find methods.
	 * @return array Array of Projects where project ID is key or null if nothing left
	 */
	private function popFakeProjects($projectsArray)
	{
		foreach ($projectsArray as $id => $project)
		{
			if($project->Fake == true)
				unset($projectsArray[$id]);
		}
		if($projectsArray)
			return $projectsArray;
		else
			return array();
	}

	/**
	 * Creates new project.
	 * 
	 * @param Project $project
	 * @return boolean
	 */
	public function save(Project $project)
	{
		if($this->mapper->save($project))
		{
			$this->multilog->log('Project with ID: ' . $project->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}		
	}
	
	/**
	 * Updates existing project.
	 * 
	 * Updates only attributes that are updatable.
	 * 
	 * @param Project $project
	 * @return boolean
	 */
	public function update(Project $project)
	{
		$oldObject = $this->find($project->ID);
		if($this->mapper->update($project))
		{
			$this->multilog->log('Project with ID: ' . $project->ID . ' updated. ' . $this->multilog->differences($oldObject, $project), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
		return $this->mapper->update($project);
	}

	/**
	 * Deletes existing project.
	 * 
	 * @param int $project_id
	 * @return boolean
	 */
	public function delete($project_id)
	{
		if($this->mapper->delete($project_id))
		{
			$this->multilog->log('Project with ID: ' . $project_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets project by id. Alias for findProject.
	 * 
	 * @param int $project_id
	 * @return Project
	 */
	public function find($project_id)
	{
		$project = $this->mapper->find($project_id);
		if($project == null)
			return null;
		if($this->showFakeProjects == false && $project->Fake == true)
			return null;
		else
			return $project;
	}

	/**
	 * Gets project by id. Alias for find.
	 * 
	 * @param int $project_id
	 * @return Project
	 */
	public function findProject($project_id)
	{
		return $this->find($project_id);
	}

	/**
	 * Gets project by SAP ID.
	 * 
	 * @param string $sap_id
	 * @return Project
	 */
	public function findProjectBySAPID($sap_id)
	{
		$project = $this->mapper->findBySAPID($sap_id);
		if($project == null)
			return null;
		elseif($this->showFakeProjects == false && $project->Fake == true)
			return null;
		else
			return $project;
	}

	/**
	 * Gets all Projects.
	 * 
	 * @param boolean $findForReport Decides if find place holder projects for report generaiton.
	 * @return array Array of Projects where Project_ID is key.
	 */
	public function findAllProjects($findForReport = false)
	{
		if($findForReport)
			return $this->mapper->findAll();
		if($this->showFakeProjects == true)
			return $this->mapper->findAll();
		else
			return $this->popFakeProjects($this->mapper->findAll());
	}
    
    /**
	 * Gets all Projects which has taks plan in given year.
	 * 
     * @param int $year year.
	 * @param boolean $findForReport Decides if find place holder projects for report generaiton
	 * @return array Array of Projects where Project_ID is key.
	 */
	public function findAllProjectsInYear($year, $findForReport = false)
	{
		if($findForReport)
			return $this->mapper->findAllProjectsInYear($year);
		if($this->showFakeProjects == true)
			return $this->mapper->findAllProjectsInYear($year);
		else
			return $this->popFakeProjects($this->mapper->findAllProjectsInYear($year));
	}
	
	/**
	 * Gets all Projects which has taks plan in given year and belongs to given SBE unit.
	 * 
     * @param int $year year.
	 * @param int $orgUnitID Organization unit ID.
	 * @return array Array of Projects where Project_ID is key.
	 */
	public function findAllSBEProjectsInYear($year, $orgUnitID, $findForReport = false)
	{
		if($findForReport)
			return $this->mapper->findAllSBEProjectsInYear($year, $orgUnitID);
		if($this->showFakeProjects == true)
			return $this->mapper->findAllSBEProjectsInYear($year, $orgUnitID);
		else
			return $this->popFakeProjects($this->mapper->findAllSBEProjectsInYear($year, $orgUnitID));
    }
    
	/**
	 * Gets project by task id.
	 * 
	 * @param int $task_id
	 * @return Project
	 */
	public function findProjectByTaskId($task_id)
	{
		$project = $this->mapper->findProjectByTaskId($task_id);
		if($project == null)
			return null;
		if($this->showFakeProjects == false && $project->Fake == true)
			return null;
		else
			return $project;
	}
	
    /**
     * Gets all years of project.
     * 
     * @param int $project_id
     * @return array Array of available years
     */
    public function findProjectYears($project_id)
    {
        return $this->mapper->findProjectYears($project_id);
    }
	
    /**
     * Gets all projects of person in given year.
     * @param string $person_id person ID.
     * @param int $year year.
     * @return array of Projects. Return array if find succesfull, otherwise returns null. 
     */
    public function findPersonAssignedAtProjectsInYear($person_id, $year)
    {
		if($this->showFakeProjects == true)
			return $this->mapper->findPersonAssignedAtProjectsInYear($person_id, $year);
		else
			return $this->popFakeProjects($this->mapper->findPersonAssignedAtProjectsInYear($person_id, $year));
    }
	
    /**
     * Gets all projects of person.
     * @param string $person_id person ID.
     * @param int $year year.
     * @return array of Projects. Return array if find succesfull, otherwise returns null. 
     */
    public function findPersonAssignedAtProjectsInYearAndFuture($person_id, $year)
    {
		if($this->showFakeProjects == true)
			return $this->mapper->findPersonAssignedAtProjectsInYearAndFuture($person_id, $year);
		else
			return $this->popFakeProjects($this->mapper->findPersonAssignedAtProjectsInYearAndFuture($person_id, $year));
    }
	
	/**
     * Gets all projects of person in given year where is not assigned on any task.
     * @param string $person_id person ID.
     * @param int $year year.
     * @return array of Projects. Return array if find succesfull, otherwise returns null. 
     */
    public function findPersonNotAssignedAtProjectsInYear($person_id, $year)
    {
		if($this->showFakeProjects == true)
			return $this->mapper->findPersonNotAssignedAtProjectsInYear($person_id, $year);
		else
			return $this->popFakeProjects($this->mapper->findPersonNotAssignedAtProjectsInYear($person_id, $year));
    }

    /**
     * Gets all projects where given person is involved as PI, PM, CL or PP&C
     *
     * @param string $person_id
     * @return array List of projects or empty array
     */
    public function findByPerson($person_id)
    {
    	if($this->showFakeProjects == true)
			return $this->mapper->findByPerson($person_id);
		else
			return $this->popFakeProjects($this->mapper->findByPerson($person_id));
    }
	
	/**
     * Gets Projects by ProjectFilter. Returns every project that matches all set conditions.
     * @param ProjectFilter $filter
     * @return array Of Projects or empty array
    */
	public function findByFilter(ProjectFilter $filter)
	{
		if($this->showFakeProjects == true)
			return $this->mapper->findByFilter($filter);
		else
			return $this->popFakeProjects($this->mapper->findByFilter($filter));
	}
	
	/**
	 * Gets next project id.
	 * 
	 * @return int Project ID;
	 */
	public function getNextProjectID()
	{
		return $this->mapper->getNextProjectID();
	}
	
	/**
	 * Finds out if given SBE has active or planned project.
	 * 
	 * @param int $orgUnitID Organization unit ID.
	 * @return boolean if active or planned project found
	 */
	public function hasOrgUnitActiveOrPlannedProject($orgUnitID)
	{
		return $this->mapper->hasOrgUnitActiveOrPlannedProject($orgUnitID);
    }
}