<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Global setting for application stored in database. 
 * 
 * Uses Dibi library for accesing database. 
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Can be optimized by having internal cache of data implemented
 *		so we do not need to load the same key twice from database.
 *		Cache would be updated with updates by set(), reset(), remove() methods.
 */
class DatabaseGlobalSetting implements IGlobalSetting
{
	/** @var DibiConnection */
	private $conn;
	
    public function __construct(DibiConnection $conn = NULL)
    {
		if ($conn == NULL)
			$this->conn = dibi::getConnection();
		else
			$this->conn = $conn;
    }
	
	/**
	 * Gets setting as an array with keyd values.
	 * 
	 * @param string $key Key/index/name of the setting value.
	 * @return array	Returns array or NULL if not found.
	 */
	private function load($key)
	{
		$db_setting = $this->conn->query('SELECT * FROM [GlobalSetting] WHERE [Key] = %s', $key)->fetch();
		if ($db_setting)
		{
			$setting = array(
				'key' => $db_setting->Key,
				'val' => $db_setting->Value,
				'def' => $db_setting->Default,
				'typ' => $db_setting->DataType,
			);
			return $setting;
		}
		else 
			return NULL;
	}
	
	/**
	 * Gets setting value by key if the is known.
	 * 
	 * @param string $key Key/index/name of the setting value.
	 * @return mixed Returns value.
	 * @throws RangeException on unknown key.
	 */
	public function get($key)
	{
		$setting = $this->load($key);
		if ($setting !== NULL)
		{
			$value = (($setting['val'] === NULL) ? $setting['def'] : $setting['val']);
			if ($setting['typ'] == self::OPTVAL_BOOLEAN)
				return (($value) ? TRUE : FALSE); // transform 0/1 => FALSE/TRUE
			else
				return $value;
		}
		else 
			throw new RangeException ("Key $key does not exist.");
	}
	
	/**
	 * Test whether entered datatype is ok (on of IGlobalSetting::OPTVAL_* constants).
	 * 
	 * @param string $key Key/index/name of the setting value.
	 * @return boolean Returns TRUE if datatype is valid.
	 * @throws RangeException on unknown key.
	 */
	private function validDataType($datatype)
	{
		if (($datatype == self::OPTVAL_STRING)
			|| ($datatype == self::OPTVAL_FLOAT)
			|| ($datatype == self::OPTVAL_UNSIGNED_FLOAT)
			|| ($datatype == self::OPTVAL_INTEGER)
			|| ($datatype == self::OPTVAL_UNSIGNED_INT)
			|| ($datatype == self::OPTVAL_BOOLEAN))
			return true;
		else
			return false;
	}

	/**
	 * Test whether entered value is of given data type.
	 * NULL values are always valid.
	 * 
	 * @param mixed $value 
	 * @return string $datatype
	 * @throws RangeException on unknown key.
	 */
	private function validValue($value, $datatype)
	{
		if ($value === NULL)
			return TRUE;
		switch ($datatype)
		{
			case self::OPTVAL_STRING: 
				return TRUE;
			case self::OPTVAL_FLOAT: 
				return is_numeric($value);
			case self::OPTVAL_UNSIGNED_FLOAT: 
				if (is_numeric($value))
					return (($value >= 0.0) ? TRUE : FALSE);
				else
					return FALSE;
				return TRUE;
			case self::OPTVAL_INTEGER: 
				if (is_numeric($value))
					return ((intval($value) == $value) ? TRUE : FALSE);
				else
					return FALSE;
			case self::OPTVAL_UNSIGNED_INT: 
				if (is_numeric($value))
					return (((intval($value) == $value) && ($value >= 0)) ? TRUE : FALSE);
				else
					return FALSE;
			case self::OPTVAL_BOOLEAN: 
				return is_bool($value);
			default:
				throw new InvalidArgumentException('Data type is not defined/allowed. Invalid value: ' . $type);
		}
	}
	
	/**
	 * Sets setting user value by key.
	 * 
	 * If it does not exist yet, it is created as a default value of given type
	 * but only if $autocreate is TRUE.
	 * 
	 * @param string $key Key/index/name of the setting value.
	 * @param mixed $value User value of the setting.
	 * @param string $autocreate Whether it should be created if it does not exist
	 * @param string $autocreate Data type that should 
	 * @throws RangeException on unknown key if $autocreate is FALSE.
	 * @throws InvalidArgumentException on invalid datatype of $value.
	 * @todo Should there be "add" method instead of set-autocreate functionality?
	 */
	public function set($key, $value, $autocreate = FALSE, $type = self::OPTVAL_STRING)
	{
		$stored_type = NULL;
		$setting = $this->load($key);
		if ($setting !== NULL)
			$stored_type = $setting['typ'];
		else
			if (!$autocreate)
				throw new RangeException("Key $key does not exist.");
		if ($stored_type !== NULL) // key exists => UPDATE
		{
			// TODO: optimize if no change of value => no need to update
			if ($this->validValue($value, $stored_type))
			{
				if ($stored_type == self::OPTVAL_BOOLEAN)
					$value = ($value) ? '1' : '0';
				$this->conn->query('UPDATE [GlobalSetting] SET [Value] = %s WHERE [Key] = %s', $value, $key);
			}
			else
				throw new InvalidArgumentException('Stored data is not of correct data type '.$stored_type.'. Invalid value: ' . $value);
		}
		else // key does not exist and autocreate must be true => INSERT
		{
			if (!$this->validDataType($type))
				throw new InvalidArgumentException('Data type for new key is not defined/allowed. Invalid value: ' . $type);
			if ($this->validValue($value, $type))
			{
				if ($stored_type == self::OPTVAL_BOOLEAN)
					$value = ($value) ? '1' : '0';
				$this->conn->query('INSERT INTO [GlobalSetting]', array('Key' => $key, 'Default' => $value, 'DataType' => $type));
			}
			else
				throw new InvalidArgumentException('Stored data is not of correct data type '.$type.'. Invalid value: ' . $value);
		}
	}
	
	/**
	 * Resets value to default value (NULLifies user-value).
	 * @param string $key Key/index/name of the setting value.
	 * @throws RangeException on unknown key.
	 */
	public function reset($key)
	{
		// test of key existence (if it does not exist, exception will be thrown
		$value = $this->get($key);
		$this->conn->query('UPDATE [GlobalSetting] SET [Value] = NULL WHERE [Key] = %s', $key);
	}
	
	/**
	 * Gets default value for key.
	 * @param string $key Key/index/name of the setting value.
	 * @return mixed 
	 * @throws RangeException on unknown key.
	 */
	public function getDefault($key)
	{
		$setting = $this->load($key);
		if ($setting !== NULL)
			if ($setting['typ'] == self::OPTVAL_BOOLEAN)
				return (($setting['def']) ? TRUE : FALSE); // transform 0/1 => FALSE/TRUE
			else
				return $setting['def'];
		else 
			throw new RangeException("Key $key does not exist.");
	}

	/**
	 * Tests whether value for key is defined (test whether any value is
	 * defined i.e. whether user-defined or default value is set)
	 * 
	 * @param string $key Key/index/name of the setting value.
	 * @return boolean
	 */
	public function exists($key)
	{
		return (($this->load($key) === NULL) ? FALSE : TRUE);
	}

	/**
	 * Removes key-value pair from settings (not only value NULLifiing).
	 * @param string $key Key/index/name of the setting value.
	 * @throws RangeException on unknown key.
	 */
	public function remove($key)
	{
		$this->get($key);
		$this->conn->query('DELETE FROM [GlobalSetting] WHERE [Key] = %s', $key);
	}
}


