<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Task of a project.
 * 
 * Tasks are organised to WorkPackages.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class Task extends Object implements WBSElement
{
	/* possible task actuals import data sources */
	const TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP = 'iASAP';
	const TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE = 'E-Charge';
	const TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL = 'Manual';
	const TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC = 'SAP';
	
	/* possible task actuals import data sources */
	const TASK_TYPE_SAP = 0;
	const TASK_TYPE_PLACEHOLDER = 1;
	const TASK_TYPE_PLANNING = 2;

	/** @var int */
	private $id = 0;
	/** @var string External Task ID in SAP. */
	private $sapID = '';
	/** @var string First 10 chars from SAP ID. */
	private $network = '';
	/** @var string Last 4 chars from SAP ID. */
	private $definition = '';
	/** @var int ID of project this task belongs to */
	private $projectID = 0;
	/** @var int ID of workpackage this task belongs to */
	private $workPackageID = 0;
	/** @var string */
	private $name = '';
	/** @var string Date of task creation */
	private $date = NULL;
	/** @var string ID of a cost center (work center in SAP) this task belongs to */
	private $costCenterID = NULL;
	/** @var boolean Whether this task is labour task */
	private $labourTask = true;
	/** @var string Description and remarks */
	private $remark = '';
	/** @var string Percentage of costs sharing among customer and supplier. Real cost share */
	private $costShare = NULL;
	/** @var string Percentage of costs sharing among customer and supplier. From SAP */
	private $sapCostShare = 0;
	/** @var boolean Whether this task should be considered as labour task */
	private $costAsLabour = false;
	/** @var boolean Fake Decides if task is fake or not.*/ 
 	private $fake = 0;
	/** @var boolean Decides if import actuals or not.*/ 
 	private $actualsImport = 0;
	/** @var boolean Decides if import actuals from iASAP or from E-Charge.*/ 
 	private $actualsImportDataSource;
	/** @var string Date of task start */
	private $startDate = NULL;
	/** @var string Date of task finish */
	private $finishDate = NULL;
	/** @var boolean Task assignment lock */
	private $locked  = FALSE;
	
	public function __construct()
	{}
	
   /**
     * Set ID of task.
     * 
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('Task ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of task.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}

    /**
     * Gets Definition ID.
     * 
     * @return string
     */
	public function getSAPID()
	{
		return $this->sapID;
	}
	
    /**
     * Sets SAP ID.
     * 
     * @param string $id
     */
	public function setSAPID($id)
	{
		$this->sapID = $id;
	}

    /**
     * Gets SAP ID.
     * 
     * @return string
     */
	public function getNetwork()
	{
		return $this->network;
	}
	
	/**
     * Sets Network ID.
     * 
     * @param string $id
     */
	public function setNetwork($id)
	{
		$this->network = $id;
	}

    /**
     * Gets Network ID.
     * 
     * @return string
     */
	public function getDefinition()
	{
		return $this->definition;
	}
	
	/**
     * Sets Definition ID.
     * 
     * @param string $id
     */
	public function setDefinition($id)
	{
		$this->definition = $id;
	}
    
    /**
     * Gets SAP ID with dash '-' insted of space ' ',
     * eg. '7001358419-0010' instead of '7001358419 0010'.
     * 
     * @return type 
     */
    public function getSAPIDWithDash()
    {
        return str_replace(' ', '-', $this->sapID);
    }

    /**
     * Gets SAP ID for usage in ECharge.
     * 
     * @return string
     */
	public function getEChargeID()
	{
		return $this->getSAPIDWithDash();
	}
    
    /**
     * Gets SAP ID for usage in Concur in case this task is travel
     * task which means that it starts with the right prefix.
     * 
     * @return string
     * @throws InvalidStateException in case the SAP ID is not as expected
     * @todo By far all tasks are in CostCenter (SAP) Brno -> 3320
     */
	public function getConcurID()
	{
        if (strpos($this->name, 'TRV') !== 0)
            return NULL;
        
        // "7002329123 0020" => "3320-007002329123/0020"
        // "402080" => "3320-000000402080"
        $id_breakdown = explode(' ', $this->sapID);
        if (count($id_breakdown) > 2)
            throw new InvalidStateException('Task SAP ID cannot be parsed. Format is invalid: ' . $this->sapID);
        $sap_id = str_pad($id_breakdown[0], 12, '0', STR_PAD_LEFT);
        if (count($id_breakdown) > 1)
            $sap_id .= '/'.$id_breakdown[1];
        // TODO: solve better, this is temporary solution or Brno Cost Center (Financial Center): 3320
        return '3320-' . $sap_id; 
	}
    
	/**
     * Set ID of project this task belongs to.
     * 
     * @param int $id Must be positive integer.
     */
	public function setProjectID($id)
	{
		if ($id > 0)
			$this->projectID = $id;
		else
			throw new InvalidArgumentException('Project ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of project this task belongs to.
     * 
     * @return int
     */
	public function getProjectID()
	{
		return $this->projectID;
	}

	/**
     * Set ID of WorkPackage this task belongs to.
     * 
     * @param int $id Must be positive integer.
     */
	public function setWorkPackageID($id)
	{
		if ($id > 0)
			$this->workPackageID = $id;
		else
			throw new InvalidArgumentException('WorkPackage ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of WorkPackage this task belongs to.
     * 
     * @return int
     */
	public function getWorkPackageID()
	{
		return $this->workPackageID;
	}

    /**
     * Sets name.
     * 
     * @param string $name
     */
	public function setName($name)
	{
		$this->name = $name;
	}

    /**
     * Gets name.
     * 
     * @return string
     */
	public function getName()
	{
		return $this->name;
	}	
	
    /**
     * Set date of creation.
     * 
     * @param string $datetime Date time in YYYY-MM-DD hh:mm:ss format.
	 * @todo Check format of $datetime.
     */
	public function setDate($datetime)
	{
		$this->date = $datetime;
	}
	
    /**
     * Gets date of creation.
     * 
     * @return string
     */
	public function getDate()
	{
		return $this->date;
	}

    /**
     * Sets cost center ID.
     * 
     * @param string $id
     */
	public function setCostCenterID($id)
	{
		$this->costCenterID = $id;
	}

    /**
     * Gets cost center ID.
     * 
     * @return string
     */
	public function getCostCenterID()
	{
		return $this->costCenterID;
	}		

    /**
     * Sets whether this task is labour (assigned to labourers/persons).
	 * If false task is non-labour and is usually named with a prefix:
	 * TRV - travelling, MAT - material, OTC - other costs)
	 * 
	 * @param boolean $mark
     */
    public function setLabourTaskFlag($mark)
    {
		$this->labourTask = ($mark) ? TRUE : FALSE;
    }

    /**
     * Gets whether this task is labour (assigned to labourers/persons).
	 * If false task is non-labour and is usually named with a prefix:
	 * TRV - travelling, MAT - material, OTC - other costs)
	 * 
     * @return boolean
     */
    public function getLabourTaskFlag()
    {
        return $this->labourTask;
    }

	/**
     * Sets whether this task should be considered as labour.
	 * 
	 * @param boolean $mark
     */
    public function setCostAsLabour($mark)
    {
		$this->costAsLabour = ($mark) ? TRUE : FALSE;
    }
	
    /**
     * Gets whether this task should be considered as labour.
	 * 
     * @return boolean
     */
    public function getCostAsLabour()
    {
		return $this->costAsLabour;
    }

    /**
     * Gets whether this task is labour (assigned to labourers/persons).
	 * If false task is non-labour and is usually named with a prefix:
	 * TRV - travelling, MAT - material, OTC - other costs)
	 * 
     * @return boolean
     */
    public function isLabourTask()
	{
		return $this->getLabourTaskFlag();
	}

    /**
     * Sets remarks/description of a task.
     * 
     * @param string $name
     */
	public function setRemark($remark)
	{
		$this->remark = $remark;
	}

    /**
     * Gets remarks/description of a task.
     * 
     * @return string
     */
	public function getRemark()
	{
		return $this->remark;
	}

    /**
	 * Sets cost share.
	 * Precision will be rounded to 2 decimal positions.
	 * Value 1.6 means 160%
	 *
	 * @param float $value Should be number in range <0,10)
	 * @throws InvalidArgumentException
	 */
	public function setCostShare($value)
	{
		if ((is_numeric($value) && ($value >= 0) && ($value < 10)) || ($value === null))
			$this->costShare = $value;
		else
			throw new InvalidArgumentException('Cost sharing must be float in range <0,10).');
	}

	/**
	 * Sets cost share.
	 * Treats with given value as it would be percentage representation of cost share.
	 * Value 160 will be transformed to 1.6
	 *
	 * @param int $value Should be number in range <0,1000)
	 * @throws InvalidArgumentException
	 */
	public function setCostSharePercentage($value)
	{
        if (is_numeric($value) && ($value >= 0) && ($value < 1000))
        	$this->costShare = $value / 100;
        elseif ($value === null)
        	$this->costShare = null;
        else
        	throw new InvalidArgumentException('Cost share must be integer in range <0,1000).');
	}

	/**
     * Gets cost share.
     * Returns real cost share in case it is inherited from project, SAP cost share otherwise.
     * 
     * @return float
     */
	public function getCostShare()
	{
		if ($this->costShare !== null)
			return $this->costShare;

		return $this->sapCostShare;
	}

	/**
     * Gets real cost share
     * 
     * @return float
     */
	public function getRealCostShare()
	{
		return $this->costShare;
	}

	/**
	 * Gets cost share as percentage value
	 * Returns real cost share in case it is inherited from project, SAP cost share otherwise.
	 *
	 * @return float
	 */
	public function getCostSharePercentage()
	{
		return ($this->getCostShare() !== null) ? ($this->getCostShare() * 100) : null;
	}

	/**
	 * Sets SAP cost share.
	 * Precision will be rounded to 2 decimal positions.
	 * Value 1.6 means 160%
	 *
	 * @param float $value Should be number in range <0,10)
	 * @throws InvalidArgumentException
	 */
	public function setSAPCostShare($value)
	{
        if ((is_numeric($value) && ($value >= 0) && ($value < 10)) || ($value === null))
        	$this->sapCostShare = ($value === null) ? 0 : $value;
        else
        	throw new InvalidArgumentException('Cost Share must be float in range <0,10).');
	}

	/**
	 * Sets SAP cost share.
	 * Treats with given value as it would be percentage representation of cost share.
	 * Value 160 will be transformed to 1.6
	 *
	 * @param int $value Should be number in range <0,1000)
	 * @throws InvalidArgumentException
	 */
	public function setSAPCostSharePercentage($value)
	{
        if (is_numeric($value) && ($value >= 0) && ($value < 1000))
        	$this->sapCostShare = $value / 100;
        elseif ($value === null)
        	$this->sapCostShare = 0;
        else
        	throw new InvalidArgumentException('Cost share must be integer in range <0,1000).');
	}

	/**
	 * Gets SAP cost share
	 *
	 * @return float
	 */
	public function getSAPCostShare()
	{
		return $this->sapCostShare;
	}
	
	/**
	 * Gets SAP cost share as percentage value
	 *
	 * @return float
	 */
	public function getSAPCostSharePercentage()
	{
		return ($this->getSAPCostShare() !== null) ? ($this->getSAPCostShare() * 100) : null;
	}
    
    /**
     * Returns category of a Task guessed from it's name prefix.
     * 
     * @return string [LABOR|MATERIAL|TRAVEL|SUBCONTRACTING]
     */
    public function getCategory()
    {
        $prefix = strtolower(substr($this->name, 0, 3));
        $category = '';
        switch ($prefix)
        {
            case 'mat':
                $category = 'MATERIAL';
                break;
            case 'trv':
                $category = 'TRAVEL';
                break;
            default:
                if ($this->labourTask)
                {
                    $category = 'LABOR';
                }
                else
                {
                    $category = 'SUBCONTRACTING';
                }
                break;
        }
        return $category;
    }
	
	/**
	 * Sets fake flag. 
	 * 
	 * @param boolean $fake 
	 */ 
	public function setFake($fake) 
	{ 
		if (in_array($fake, array_keys(Task::getTaskTypes())))
			$this->fake = $fake;
		else
			throw new InvalidArgumentException('Unknown task type: ' . $fake . '.');
	} 

	/** 
	 * Gets fake flag. 
	 * 
	 * @return boolean 
	 */ 
	public function getFake() 
	{ 
		return $this->fake; 
	}
	
	/**
	 * Sets actualsImport flag. 
	 * 
	 * @param boolean $actualsImport 
	 */ 
	public function setActualsImport($actualsImport) 
	{ 
		$this->actualsImport = $actualsImport; 
	} 

	/** 
	 * Gets actualsImport flag. 
	 * 
	 * @return boolean 
	 */ 
	public function getActualsImport() 
	{ 
		return ($this->actualsImport) ? TRUE : FALSE; 
	}
	
	/**
	 * Sets actualsImportDataSource. 
	 * 
	 * @param boolean $actualsImportDataSource 
	 */ 
	public function setActualsImportDataSource($actualsImportDataSource) 
	{ 
		if (is_null($actualsImportDataSource) || in_array($actualsImportDataSource, array_keys(Task::getActualsImportDataSources())))
			$this->actualsImportDataSource = $actualsImportDataSource;
		else
			throw new InvalidArgumentException('Unknown task actuals import data source: ' . $actualsImportDataSource . '.');
	} 

	/** 
	 * Gets actualsImportDataSource. 
	 * 
	 * @return boolean 
	 */ 
	public function getActualsImportDataSource() 
	{ 
		return $this->actualsImportDataSource;
	}
	
    /**
     * Set start date.
     * 
     * @param string $datetime Date time in YYYY-MM-DD format.
	 * @todo Check format of $datetime.
     */
	public function setStartDate($datetime)
	{
		$this->startDate = $datetime;
	}
	
    /**
     * Gets start date.
     * 
     * @return string
     */
	public function getStartDate()
	{
		return $this->startDate;
	}
	
	/**
     * Set finish date.
     * 
     * @param string $datetime Date time in YYYY-MM-DD format.
	 * @todo Check format of $datetime.
     */
	public function setFinishDate($datetime)
	{
		$this->finishDate = $datetime;
	}
	
    /**
     * Gets finish date.
     * 
     * @return string
     */
	public function getFinishDate()
	{
		return $this->finishDate;
	}
	
	/**
     * Gets task duration.
	 * Returns:
	 * If strat and finish date is defined - (From: 20-01-2015 to: 20-12-2015)
	 * If finish date is not defined - (From: 20-01-2015 to: -)
	 * if strat date is not defined - null
     * 
     * @return string
     */
	public function getTaskDuration()
	{
		if(!is_null($this->startDate) && !is_null($this->finishDate))
			return '(From: ' . $this->startDate->format('Y-m-d') . ' To: ' . $this->finishDate->format('Y-m-d') . ')';
		elseif(!is_null($this->startDate) && is_null($this->finishDate)) 
			return '(From: ' . $this->startDate->format('Y-m-d') . ' To: -)';
		else
			return null;
	}
	
    /**
     * Returns array of available actuals import data sources
     * @return array arr[string] = string .. arr[TASK_ACTUALS_IMPORT_DATA_SOURCE_*] = TASK_ACTUALS_IMPORT_DATA_SOURCE_*
     */
	public static function getActualsImportDataSources()
	{
            $arrStatuses = array(
                self::TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE => self::TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE,
                self::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP => self::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP,
				self::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL => self::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL,
				self::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC => self::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC
            );
            asort($arrStatuses);
            return $arrStatuses;
	}
	
/**
	 * Sets task assignment lock.
     * @param boolean $locked
     */
    public function setLocked($locked)
    {
        $this->locked = ($locked) ? TRUE : FALSE;
    }

    /**
	 * Gets task assignment lock.
     * @return boolean
     */
    public function getLocked()
    {
        return $this->locked;
    }
	
    /**
     * Returns array of available task types
     * @return array arr[int] = string .. arr[TASK_TYPE_*] = TASK_TYPE_*
     */
	public static function getTaskTypes()
	{
            $arrTypes = array(
                self::TASK_TYPE_SAP => self::TASK_TYPE_SAP,
                self::TASK_TYPE_PLACEHOLDER => self::TASK_TYPE_PLACEHOLDER,
				self::TASK_TYPE_PLANNING => self::TASK_TYPE_PLANNING
            );
            return $arrTypes;
	}
}
