<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing tasks from database using Dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class TaskDibiMapper extends DibiMapper implements ITaskMapper
{
	/**
	 * Creates Task instance from DibiRow.
	 * 
	 * @param $row DibiRow
	 * @return	Task 
	 */
    private function loadTask(DibiRow $row)
    {
		$task = new Task();
		$task->ID = $row->Task_ID;
		$task->SAPID = $row->SAP_ID;
		$sapID = explode(" ", $row->SAP_ID);
		if(isset($sapID[0]))
			$task->Network = $sapID[0];
		else
			$task->Network = '0';
		if(isset($sapID[1]))
			$task->Definition = $sapID[1];
		else
			$task->Definition = '0';
		$task->ProjectID = $row->Project_ID;
		$task->WorkPackageID = $row->WorkPackage_ID;
		$task->Name = $row->Name;
		$task->Date = $row->Date;
		$task->CostCenterID = $row->CostCenter_ID;
		$task->LabourTaskFlag = $row->LabourTask;
		$task->Remark = $row->Remark;
		$task->CostShare = $row->RealCostShare;
		$task->SAPCostShare = $row->CostShare;
		$task->CostAsLabour = $row->CostAsLabour;
		$task->Fake = $row->Fake;
		$task->ActualsImport = $row->ActualsImport;
		$task->ActualsImportDataSource = $row->ActualsImportDataSource;
		$task->StartDate = $row->StartDate;
		$task->FinishDate = $row->FinishDate;
		$task->Locked = $row->Locked;
		return $task;
    }	
	
	/**
	 * Saves new task.
	 * 
	 * @param Task $task
	 */
	public function save(Task $task)
	{
        try
        {
            $array = array();
            $array['SAP_ID'] = $task->getSAPID();
            $array['Project_ID'] = $task->getProjectID();
            $array['WorkPackage_ID'] = $task->getWorkPackageID();
            $array['Name'] = $task->getName();
            $array['Date'] = $task->getDate();
            $array['CostCenter_ID'] = $task->getCostCenterID();                
            $array['Remark'] = $task->getRemark();
            $array['CostShare'] = $task->getSAPCostShare();
            $array['RealCostShare'] = $task->getRealCostShare();
			$array['CostAsLabour'] = $task->getCostAsLabour();
			$array['Fake'] = $task->getFake();
			$array['ActualsImport'] = $task->getActualsImport();
			$array['ActualsImportDataSource'] = $task->getActualsImportDataSource();
			$array['StartDate'] = $task->getStartDate();
			$array['FinishDate'] = $task->getFinishDate();
            // LabourTask column is controlled by DecideLabourTask trigger on TaskPlan table
            if ($task->LabourTaskFlag)
			    $array['LabourTask'] = 1;
            else
			    $array['LabourTask'] = 0;
			$array['Locked'] = $task->Locked;
            $this->conn->insert('Task', $array)->execute();
			$task->ID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert of new task with SAP ID = ".$task->getSAPID()." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Updates existing task.
	 * 
	 * Updates only attributes that are updatable.
	 * 
	 * @param Task $task
	 */
	public function update(Task $task)
	{
        try
        {
			// NOTE: changing of WorkPackage_ID column triggers Project_ID column update
            $array = array();
            $array['Name'] = $task->getName();
			$array['SAP_ID'] = $task->getSAPID();
            $array['Remark'] = $task->getRemark();
            $array['CostShare'] = $task->getSAPCostShare();
            $array['RealCostShare'] = $task->getRealCostShare();
            $array['CostCenter_ID'] = $task->getCostCenterID();
			$array['CostAsLabour'] = $task->getCostAsLabour();
			$array['Fake'] = $task->getFake();
			$array['ActualsImport'] = $task->getActualsImport();
			$array['ActualsImportDataSource'] = $task->getActualsImportDataSource();
			$array['StartDate'] = $task->getStartDate();
			$array['FinishDate'] = $task->getFinishDate();
			if ($task->LabourTaskFlag) 
				$array['LabourTask'] = 1; 
			else     
				$array['LabourTask'] = 0;
			$array['Locked'] = $task->Locked;
			$assigment_db_result = $this->conn->update('Task', $array)->where('[Task_ID] = %u', $task->getID())->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of task with ID = ".$task->getID()." failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Updates value of real cost share of each task which belongs to project of given id
	 *
	 * @param float $cost_share New value
	 * @param int $project_id
	 * @return boolean True if update was successful, false otherwise
	 */
	public function updateRealCostShare($cost_share, $project_id)
	{
		try
        {
            $array = array(
            	'RealCostShare' => $cost_share
            );

			$db_result = $this->conn
			        ->update('Task', $array)
			        ->where('[Project_ID] = %i', $project_id)
			        ->execute();

            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of tasks with Project_ID = " . $project_id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing task.
	 * 
	 * @param int $task_id
	 * @return boolean
	 */
	public function delete($task_id)
	{
		try
		{
		    $result = $this->conn->delete('Task')->where('[Task_ID] = %i', $task_id)->execute();
			if ($result)
				Debugger::log("Task with ID = " . $task_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of task with ID = " . $task_id . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of task with ID = " . $task_id . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds task by id.
	 * 
	 * @param int $id task ID.
	 * @return	Task or NULL if not found.
	 */
	public function find($id)
	{
		$db_row = $this->conn
		        ->query('
			        SELECT [Task].*
			        FROM [Task]
			        WHERE [Task_ID] = %i
		        ', $id)
		        ->fetch();

		if ($db_row)
			return $this->loadTask($db_row);
		else
			return NULL;
	}
    
	/**
	 * Finds task by SAP id.
	 * 
	 * @param string $sap_id task SAP ID.
	 * @return Task or NULL if not found.
	 */
	public function findBySAPID($sap_id) 
	{
		$task_db_result = $this->conn
		        ->query('
		        	SELECT [Task].*
		        	FROM [Task]
		        	WHERE [Task.SAP_ID] = %s
		        ', $sap_id)
		        ->fetch();

		if ($task_db_result !== FALSE)
			return $this->loadTask($task_db_result);
		else
			return NULL;
	}

	/**
	 * Returns all task that are part of a project ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllProjectTasks($project_id)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT [Task].*
		        	FROM [Task]
		        	WHERE [Task.Project_ID] = %i
		        	ORDER BY [Task.Name] ASC
		        ', $project_id)
		        ->fetchAll();

		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}
	
	/**
	 * Returns all task that are part of a workpackage ordered by name.
	 * 
	 * @param int $workpackage_ID ID of a project.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllWorkPackageTasks($workpackage_ID)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT [Task].*
		        	FROM [Task]
		        	WHERE [Task.WorkPackage_ID] = %i
		        	ORDER BY [Task.Name] ASC
		        ', $workpackage_ID)
		        ->fetchAll();

		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}
	
	/**
	 * Returns all task that are part of a project and are active
	 * in selected year which means task that have some persons
	 * assigned.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAssignedProjectTasksInYear($project_id, $year)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT DISTINCT [Task].*
		        	FROM [Task], [Person_Task]
		        	WHERE [Person_Task.Task_ID] = [Task.Task_ID] AND
		        	      [Task.Project_ID] = %i AND
		        	      [Person_Task.Year] = %i
		        	ORDER BY [Task.Name] ASC
		        ', $project_id, $year)
		        ->fetchAll();
		
		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}

	/**
	 * Returns all tasks that are part of a project and have
	 * plan for selected year.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findPlannedProjectTasksInYear($project_id, $year)
	{
		$db_result = $this->conn
		        ->query('
                    SELECT [Task].*
                    FROM [Task], [TaskPlan]
                    WHERE [Task.Task_ID] = [TaskPlan.Task_ID] AND
                          [Task.Project_ID] = %i AND
                          [TaskPlan.Year] = %i
                    ORDER BY [Task.Name] ASC
		        ', $project_id, $year)
		        ->fetchAll();
		
		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}

	/**
	 * Returns all tasks that are part of a workpackage and have
	 * plan for selected year.
	 * 
	 * @param int $wp_id ID of a workpackage.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findPlannedWorkpackageTasksInYear($wp_id, $year)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT [Task].*
		        	FROM [Task], [TaskPlan]
		        	WHERE [Task.Task_ID] = [TaskPlan.Task_ID] AND
		        	      [Task.WorkPackage_ID] = %i AND
		        	      [TaskPlan.Year] = %i
		        	ORDER BY [Task.Name] ASC
		        ', $wp_id, $year)
		        ->fetchAll();
		
		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}
    
	/**
	 * Returns all tasks that are active in a project in given year even if they
	 * are not planned for the current year. 
	 * 
	 * Returns only tasks that exist in the system . Deleted are not returned
	 * even if there is some activity (e.g. invoicing or work).
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findActiveProjectTasksInYear($project_id, $year)
	{
		// find SAP project id
		$project_sap_id = $this->conn->query('SELECT [SAP_ID] FROM [Project] 
				WHERE [Project_ID] = %u', $project_id)->fetchSingle();
		if ($project_sap_id === FALSE)
			throw new InvalidArgumentException('Project with ID=' . $project_id . ' not found!');
		// first bunch will be looked for using ActualsCost table
		$db_result = $this->conn
		        ->query('
                    SELECT DISTINCT [Task].*
                    FROM [Task], [ActualsCost] 
				    WHERE [Task.Task_ID] = [ActualsCost.Task_ID] AND
				          [ActualsCost].[Project_SAP_ID] = %s AND
				          [ActualsCost].[Year] = %i
					ORDER BY [Task.Name] ASC
		        ', $project_sap_id, $year)
		        ->fetchAll();

		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		// second bunch will be looked for using Actuals table
		$db_result = $this->conn
		        ->query('
		        	SELECT DISTINCT [Task].*
		        	FROM [Task], [Actuals] 
				    WHERE [Task.SAP_ID] = [Actuals.ChargeNumber] AND
				          [Actuals.Project] = %s AND
				          [Actuals.Year] = %i
					ORDER BY [Task.Name] ASC
		        ', $project_sap_id, $year)
		        ->fetchAll();

		foreach ($db_result as $row)
		{
			if (!isset($tasks[$row->Task_ID]))
			{
				$task = $this->loadTask($row);
				$tasks[$task->ID] = $task;
			}
		}
		
		return $tasks;
	}
    
    /**
     * Returns distinct task SAP IDs.
     * 
     * @return array Array of all task SAP IDs 
     */
    public function findDistinctSAPIDs()
    {
        return $this->conn->query('SELECT DISTINCT [SAP_ID] FROM [Task]')->fetchPairs();
    }
	
    /**
     * Returns distinct labour task SAP IDs.
     * 
     * @return array Array of labour task SAP IDs 
     */
    public function findDistinctLaborTaskSAPIDs()
    {
        $db_result = $this->conn->query('SELECT DISTINCT * FROM [Task]
				WHERE [LabourTask] = %i AND [ActualsImport] = %i', 1, 0)->fetchAll();
		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$row->SAP_ID] = $task;
		}
		return $tasks;
    }
	
    /**
     * Returns distinct labour activity.
     * 
     * @return array Array of labour task
     */
    public function findDistinctLaborTaskActivity()
    {
        $db_result = $this->conn->query('SELECT DISTINCT * FROM [Task]
				WHERE [LabourTask] = %i', 1)->fetchAll();
		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$row->SAP_ID] = $task;
		}
		return $tasks;
    }
    
	/**
	 * Returns all task that are part of a project ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllProjectTasksInYear($project_id, $year)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT DISTINCT [Task].*
		        	FROM [Task] JOIN [TaskPlan] ON [Task.Task_ID] = [TaskPlan.Task_ID]
                    WHERE [Task.Project_ID] = %i AND
                          [TaskPlan.Year] = %i
                    ORDER BY [Task.Name] ASC
		        ', $project_id, $year)
		        ->fetchAll();

		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}
	
	/**
	 * Returns all task that are part of a project ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllProjectTasksInYearAndFuture($project_id, $year)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT DISTINCT [Task].*
		        	FROM [Task] JOIN [TaskPlan] ON [Task.Task_ID] = [TaskPlan.Task_ID]
                    WHERE [Task.Project_ID] = %i AND
                          [TaskPlan.Year] >= %i
                    ORDER BY [Task.Name] ASC
		        ', $project_id, $year)
		        ->fetchAll();

		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}
	
	/**
	 * Returns all Tasks' IDs and theirs SAP IDs.
	 * 
	 * @param bool $removeSpaces Remove all spaces from key strings.
	 * @return array associative array, where Task_ID is key and its SAP_ID is the value
	 */
	public function findAllProjectTasksSAPIDs($removeSpaces = FALSE)
	{
		$db_result = $this->conn
				->query('
					SELECT [SAP_ID], [Task_ID]
					FROM [Task]
					')
					->fetchAll();
		
		$tasks;
		if ($db_result !== FALSE)
		{
			foreach ($db_result as $row)
			{
				if ($removeSpaces)
					$tasks[str_replace(' ', '', $row->SAP_ID)] = $row->Task_ID;
				else
					$tasks[$row->SAP_ID] = $row->Task_ID;
			}
			return $tasks;
		}
		else
			return NULL;
	}
    
    /**
     * Gets all tasks of person in given project and year.
     * @param string $person_id person ID.
	 * @param int $project_id
     * @param int $year year.
     * @return array of tasks. Return array if find succesfull, otherwise returns null.
     */
    public function findPersonTasksInYear($person_id, $project_id, $year)
    {
        $db_result = $this->conn
                ->query('
                	SELECT DISTINCT [Task].*
                	FROM [Person_Task] JOIN [Task] ON [Person_Task.Task_ID] = [Task.Task_ID]
                                       JOIN [Project] ON [Task.Project_ID] = [Project.Project_ID]
                    WHERE [Project.Project_ID] = %i AND
                          [Person_Task.Year] = %i AND
                          [Person_Task.EID] = %s
                    ORDER BY [Task.Name] ASC
                ', $project_id, $year, $person_id)
                ->fetchAll();

        $tasks = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
              $tasks[$row->Task_ID] = $this->loadTask($row);
            return $tasks;
        }
        else
            return NULL;
    }
	
    /**
     * Gets all tasks where is person assigned.
     * @param string $person_id person ID.
     * @return array of tasks. Return array if find succesfull, otherwise returns null.
     */
    public function findAllTasksWhereIsPersonAssigned($person_id)
    {
        $db_result = $this->conn
                ->query('
                	SELECT DISTINCT [Task].*
                	FROM [Person_Task] JOIN [Task] ON [Person_Task.Task_ID] = [Task.Task_ID]
                    WHERE [Person_Task.EID] = %s
                    ORDER BY [Task.Name] ASC
                ', $person_id)
                ->fetchAll();

        $tasks = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
              $tasks[$row->Task_ID] = $this->loadTask($row);
            return $tasks;
        }
        else
            return NULL;
    }

    /**
     * Gets tasks where person charged some hours in given project and year.
     * @param string $person_id person ID.
	 * @param int $project_id
     * @param int $year year.
     * @return array of tasks. Return array if find succesfull, otherwise returns null.
     */
    public function findPersonActiveTasksInYear($person_id, $project_id, $year)
    {
        $db_result = $this->conn
                ->query('
                	SELECT DISTINCT [Task].*
                	FROM [Actuals] JOIN [Task] ON [Actuals.ChargeNumber] = [Task.SAP_ID]
			        WHERE [Actuals.EID] = %s AND
			              [Actuals.Year] = %i AND
			              [Task.Project_ID] = %i
                ', $person_id, $year, $project_id)
                ->fetchAll();

        $tasks = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
              $tasks[$row->Task_ID] = $this->loadTask($row);
            return $tasks;
        }
        else
            return NULL;
    }
	
	/**
	 * Gets next task id.
	 * 
	 * @return int Task ID;
	 */
	public function getNextTaskID()
	{
		$db_result = $this->conn->query('SELECT MAX([Task_ID]) FROM [Task]')->fetch();
		if ($db_result === FALSE)
			return '1';
		else		
			return sprintf('%010s', $db_result['MAX(`Task_ID`)'] + 1);
	}
	
	/**
     * Gets all placeholder tasks for actuals import.
	 * @param string $importFrom iASAP or E-Charge(Task constants).
     * @return array of tasks. Return array if find succesfull, otherwise returns empty array.
     */
    public function findAllPlaceholderTasksForImport($importFrom)
	{
		$db_result = $this->conn
		        ->query('
		        	SELECT [Task].*
		        	FROM [Task] 
			        WHERE [Task.ActualsImport] = %i AND
			              [Task.ActualsImportDataSource] = %s
		        ', 1, $importFrom)
		        ->fetchAll();

		$tasks = array();
		foreach ($db_result as $row)
		{
			$task = $this->loadTask($row);
			$tasks[$task->ID] = $task;
		}
		return $tasks;
	}
	
	/**
	 * Returns all labour tasks with task plan by given year.
	 * 
	 * @param int $year Year.
	 * @return	array Returns associative array of tasks (key is TaskID).
	 */
	public function findLabourTaskWithTaskPlanForYear($year)
	{
		$tasks = array();
		$tasks_db_result = $this->conn->query('SELECT [Task].* FROM [Task]
				LEFT JOIN [TaskPlan] ON [Task].[Task_ID] = [TaskPlan].[Task_ID]
				WHERE [TaskPlan].[Year] = %u AND [Task].[LabourTask] = %u
				ORDER BY [Task].[Name] ASC', $year, 1)->fetchAll();
		foreach ($tasks_db_result as $task_db_result)
		{
			$tasks[$task_db_result->Task_ID] = $this->loadTask($task_db_result);;
		}
		return $tasks;
	}
	
    /**
     * Gets all years of task.
     * 
     * @param int $taskID
     * @return array Array of available years
     */
    public function findTaskYears($taskID)
    {
		$db_result = $this->conn->query('SELECT DISTINCT [TaskPlan].[Year] FROM [TaskPlan]
                                                WHERE [TaskPlan].[Task_ID] = %u ORDER BY [TaskPlan].[Year] ASC', $taskID)->fetchAll();
        $years = array();        
		foreach ($db_result as $year)		
			array_push($years, $year->Year);       
        return $years;
    }
}
