<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Description of UserMonthlyReportService
 * 
 * Contract of the class:
 * Use method prepareUserMonthlyTaskReport() first for report preparation
 * and then use getReport() and getTotals() to obtain the data.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Do not inject context into class but rather inject single repositories
 */
class UserMonthlyReportService
{

	/** prepared report */
	private $report = NULL;
			
	/** prepared report totals */
	private $totals = NULL;
	
	/** @var DIContainer Access to model repositories */
	private $context;

    public function __construct(DIContainer $context)
    {
		$this->context = $context;
	}
	
   /**
     * For selected user prepares report with user's tasks he should be working 
	 * on required fiscal month and saves data in associative arrays for
	 * later readout using methods getReport() and getTotals().
	 * 
     * @param	$employee_id	string	    User ID / Employee ID.
     * @param	$fiscal_month	FiscalMonth Fiscal month.
	 * @return void	
     */
    public function prepareUserMonthlyTaskReport($employee_id, FiscalMonth $fiscal_month) 
    {
		$month = $fiscal_month->Index + 1;
		$month_assignments = $this->context->personTaskAssignmentRepository->findPersonMonthAssignments($employee_id, $month, $fiscal_month->Year);
		$all_work = $this->context->workRepository->findTasksPersonWorkedOnInMonth($employee_id, $month, $fiscal_month->Year);
		$this->report = array();
		$this->totals = array("plan" => 0, "actual" => 0.0, "delta" => 0.0, "actual_extra" => 0.0);
		$task_list = array();
		$projects = array();
		foreach (array_keys($month_assignments) as $task_id) 
		{
			$task = $this->context->taskRepository->find($task_id);
			$task_list[$task_id] = $task;
			if (!isset($projects[$task->ProjectID])) // load if not loaded already
				$this->projects[$task->ProjectID] = $this->context->projectRepository->find($task->ProjectID);
		}
		foreach (array_keys($all_work) as $task_id) 
		{
			if (!isset($task_list[$task_id])) // load if not among planned tasks
			{
				$task = $this->context->taskRepository->find($task_id);
				$task_list[$task_id] = $task;
				if (!isset($projects[$task->ProjectID])) // load if not loaded already
					$this->projects[$task->ProjectID] = $this->context->projectRepository->find($task->ProjectID);
			}
		}
		// TODO sort tasks by project

		$last_project_id = 0;
		$last_project_sapid = '';
		$last_wp_id = 0;
		foreach ($task_list as $task_id => $task) 
		{
			// project change
			if ($last_project_id != $task->ProjectID && isset($last_project_id))
			{
				$last_project_id = $task->ProjectID;
				$project = $this->projects[$last_project_id];
				if($project)
				{
					$last_project_sapid = $project->SAPID;
					$this->report[] = array(
						'type' => 'project',
						'name' => $project->Name,
						'id' => $project->ID,
						'sap' => $project->SAP_ID,
						'plan' => '',
						'actual' => '',
						'delta' => '',
						'fake' => $project->Fake
					);
				}
				else
				{
					break;
				}
			}
			// load workpackage tree structure if necessary ...
			if ($last_wp_id != $task->WorkPackageID) 
			{
				$last_wp_id = $wp_parent_id = $task->WorkPackageID;
				$wp_parent_arr = array();
				while ($wp_parent_id > 0) // there is a wp structure
				{
					$wp_parent = $this->context->workPackageRepository->find($wp_parent_id);
					if ($wp_parent->SAPID != $last_project_sapid) // top-level work-packages have the same SAP ID and name -> omitting
					{
						$wp_parent_arr[] = array(
							'type' => 'workpackage',
							'name' => $wp_parent->Name,
							'id' => $wp_parent->ID,
							'sap' => $wp_parent->SAPID,
							'plan' => '',
							'actual' => '',
							'delta' => ''
						);
					}
					$wp_parent_id = $wp_parent->ParentID;
				}
				// ...and add it to the report
				$wp_parent_arr_rev = array_reverse($wp_parent_arr);
				foreach ($wp_parent_arr_rev as $wp_parent)
					$this->report[] = $wp_parent;
			}
			$planned_task = isset($month_assignments[$task_id]);
			$plan = ($planned_task) ? $month_assignments[$task_id]->Hours : 0;
			$actual = (isset($all_work[$task_id])) ? $all_work[$task_id] : 0;
			$delta = round($plan - $actual, 1);
			$this->report[] = array(
				'type' => 'task',
				'name' => $task->Name,
				'id' => $task->ID,
				'sap' => $task->SAPID,
				'plan' => $plan,
				'actual' => $actual,
				'delta' => $delta,
				'extra' => (($planned_task) ? 0 : 1),
				'fake' => $task->Fake
			);
			
			// update totals
			if (!$planned_task)
				$this->totals["actual_extra"] += $actual;
			$this->totals["plan"] += $plan;
			$this->totals["actual"] += $actual;
			$this->totals["delta"] += $delta;
		}
		
		foreach ($all_work as $task_id => $actual) 
		{
			if (!isset($task_list[$task_id])) // work on a task without plan
			{
				$task = $this->context->taskRepository->find($task_id);
				$this->report[] = array(
					'type' => 'task',
					'name' => $task->Name,
					'id' => $task->ID,
					'sap' => $task->SAPID,
					'plan' => 0,
					'actual' => $actual,
					'delta' => -$actual
				);
				$this->totals["actual_extra"] += $actual;
			}
		}
		
		$aop = $this->context->aopRepository->findForPerson($fiscal_month->Year, $employee_id);
		$this->totals["workable"] = $aop->getMonthAOPHours($month);
		$this->totals["coverage"] = $this->totals['workable'] ? round($this->totals['plan'] / $this->totals['workable'] * 100)  : 0;
		$this->totals["yield"] = $this->totals['workable'] ? round($this->totals['actual'] / $this->totals['workable'] * 100) : 0;
	}

    /**
     * Returns report prepared by prepareUserMonthlyTaskReport method.
	 * It is an array with mixed data of project, work packages and task
	 * sorted for easy displaying in a (HTML) table.
	 * 
	 * Associative arrays have following keyed values:
	 * 'type' - can be one of three string values 'project'|'workpackage'|'task'
	 * 'id' - object ID in the database (according to 'type')
	 * 'name' - object name (e.g. project name, task name, ...)
	 * 'sap' - SAP external ID of the object
	 * 'plan' - planned hours count (only for records of type 'task')
	 * 'actual' - worked hours count (only for records of type 'task')
	 * 'delta' - diffrence in plan vs. actual hours count (only for records of type 'task')
	 * 'extra' - 0 if planned task, 1 if not planned but charged/worked on
	 * 
	 * @return array
     */
    public function getReport() 
    {
		if ($this->report === NULL) {
			throw new InvalidStateException("Wrong object usage. Call 'prepareUserMonthlyTaskReport' method first!");
		} 
		return $this->report;
	}

    /**
     * Returns totals of the report prepared by prepareUserMonthlyTaskReport method.
	 * 
	 * Associative arrays have following keyed values:
	 * 'plan' - total planned hours count for all assigned tasks
	 * 'actual' - total worked hours on assigned tasks
	 * 'actual_extra' - total worked hours on non-assigned tasks (is included in actual field)
	 * 'delta' - diffrence in plan vs. actual hours count
	 * 'workable' - workable hours in the month
	 * 'coverage' - planned coverage (ratio between plan and workable hours) as percentage
	 * 'yield' - yield (ratio between worked (actuals) and workable hours) as percentage
	 * 
	 * @return array
     */
    public function getTotals() 
    {
		if ($this->totals === NULL) {
			throw new InvalidStateException("Wrong object usage. Call 'prepareUserMonthlyTaskReport' method first!");
		} 
		return $this->totals;
	}
}

