<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\WBS
 */

/**
 * Encapsulation of WBS Tree Elements - top level object.
 * 
 * Creation of three is expected in following manner: 
 * 1) Items should be added using addItem method in the order
 * so the tree is constructible meaning you cannot add
 * an item that does not have a parent in the tree already. 
 * 
 * 2) Only first added item can have parent UUID as NULL value
 * meaning that it will become root item of the tree.
 * 
 * 3) Root can be setup only once.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Decide: Should WBS tree check that WP are parent nodes to Tasks and Tasks have not subitems/subnodes?
 */
class WBSTree extends Object
{

    /** @var WBSTreeElementItem root item */
	private $root = NULL;
	
	public function __construct() 
	{
	}	
	
	public function getRootElement()
	{
		return $this->root;
	}
	
	/**
	 * Adds new WBSTreeElementItem. 
	 * 
	 * @param WBSTreeElementItem $elem WBS element to add.
	 * @param string $parent_item_uuid UUID of the parent element
	 * @return WBSTreeElementItem parent item where element was added.
	 * @throws InvalidOrderException if element cannot be added into tree because its parent is missing
	 * @throws InvalidStateException if root is added for the second time
	 */
	public function addItem(WBSTreeElementItem $elem, $parent_item_uuid = NULL)
	{
		$parent_item = NULL;
		if ($parent_item_uuid == NULL)
		{
			if ($this->root != NULL)
				throw new InvalidStateException('Root item can be added only once.');
			else
			{
				$this->root = $elem;
				$parent_item = $this->root;
			}
		}
		else
		{
			$parent_item = ($this->search($parent_item_uuid));
			if ($parent_item == NULL)
				throw new InvalidOrderException('Parent with UUID '.$parent_item_uuid. ' not found.');
			else
				$parent_item->addChild($elem);
		}
		$elem->setTree($this);
		return $parent_item;
	}

	/**
	 * Searches item in the tree by its UUID.
	 * @param string $item_uuid Item UUID.
	 * @return WBSTreeElementItem or NULL if not found.
	 */
	public function search($item_uuid)
	{
		if ($this->root == NULL)
			return NULL;
		return $this->searchInternal($this->root, $item_uuid);
	}
	
	/**
	 *  Internal method for element searching. Recursive method.
	 */
	private function searchInternal(WBSTreeElementItem $elem, $uuid)
	{
		if ($uuid == $elem->getUUID())
			return $elem;
		foreach ($elem->getChildren() as $subitem)
		{
			$found = $this->searchInternal($subitem, $uuid);
			if ($found != NULL)
				return $found;
		}
		return NULL;
	}
	
	
	/* DFS traversal for resolving element level in the WBS tree */
	public function getElementLevel(WBSTreeElementItem $elem)
	{
		if (!isset($this->root))
			throw new InvalidStateException('Root not set. Cannot determine element level.');
		$level = 0;
		if ($this->root->getUUID() == $elem->getUUID())
			return $level;
		$level++;
		foreach ($this->root->getChildren() as $subitem)
		{
			$level_found = $this->getElementLevelInternal($elem, $subitem, $level);
			if ($level_found !== NULL)
				return $level_found;
		}
		return NULL;
	}

	/**
	 *  Internal method for element level searching. Recursive method.
	 */
	private function getElementLevelInternal(WBSTreeElementItem $elem, WBSTreeElementItem $item, $level)
	{
		if ($item->getUUID() == $elem->getUUID())
			return $level;
		$level++;
		foreach ($item->getChildren() as $subitem)
		{
			$level_found = $this->getElementLevelInternal($elem, $subitem, $level);
			if ($level_found !== NULL)
				return $level_found;
		}
		return NULL;
	}
	
    /**
     * Returns iterator to traverse over children.
	 * 
	 * @param int $tasks_only If TRUE only (leaves) tasks are returned, when FALSE workpackages are returned to.
     * @return RecursiveIteratorIterator
     */
	public function getTreeIterator($leaves_only = FALSE)
	{
		if ($this->root == NULL)
		{
			// return iterator over empty array
			$array = array();
			return new RecursiveIteratorIterator(new RecursiveArrayIterator($array));
		}
		if ($leaves_only)
			return new RecursiveIteratorIterator(
				new WBSTreeElementRecursiveIterator($this->root), 
				RecursiveIteratorIterator::LEAVES_ONLY);
		else
			return new RecursiveIteratorIterator(
				new WBSTreeElementRecursiveIterator($this->root), 
				RecursiveIteratorIterator::SELF_FIRST);
	}
	
}
