<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * TypedArray is an object encapsulating array of values of same type.
 * 
 * It enables array access for "for" and "foreach" statements.
 * 
 * It checkes data type of added values are of defined type
 * same as initialization object.
 * For detailed description of initiation by type and datatype
 * testing see constructor API documentation.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Consider extending ArrayIterator
 */
class TypedArray implements ArrayAccess, Countable, IteratorAggregate
{
	/** array Internal storage for values is array */
	protected $values = array();

	/** string Textual representation of type */
	private $type;

	/** string Name of the class if Array holds objects */
	private $class = '';
	
	/** boolean Whether type checking is strict */
	private $strict = FALSE;
	
	/**
	 * Creates new array object of the same type as $init_value.
	 * NULL values and resources are not accepted as init values.
	 * 
	 * If $strict is FALSE, then if added value "contains" data of 
	 * the defined value, data is retyped to correct datatype
	 * e.g. if array is initated with int and then string of
	 * text "10" is added it is silently retyped to int.
	 * Warning: For classes $strict means that the object
	 * must be the same class or child class.
	 * 
	 * If $strict is TRUE, then added data must be of the same type.
	 * Warning: For classes it means that it must be the same class.
	 * Child classes are not accepted.
	 * 
	 * Example of creation of object:
	 * $array_of_int = new TypedArray((int) 0, false);
	 * $array_of_float = new TypedArray((float) 0, true);
	 * $array_of_typed_arrays = new TypedArray(new TypedArray('string', false), true);
	 * 
	 * @param mixed $init_value Value for initialization of type of values.
	 * @param boolean $strict Whether testing of data types should be strict.
	 * @return TypedArray
	 */
    public function __construct($init_value, $strict = false)
    {
		if ($init_value === NULL)
			throw new InvalidArgumentException("Initiation value not supplied. NULL is not acceptable as initiation value.");
		if (is_resource($init_value))
			throw new InvalidArgumentException("Resource is not acceptable as initiation value.");
		$this->type = gettype($init_value);
		if ($this->type == 'object')
			$this->class = get_class($init_value);
		$this->strict = ($strict) ? TRUE : FALSE;
	}


	/**
	 * Returns text describing data type ("integer", "double" (for float), "string", "array", "object").
	 * 
	 * @return string
	 */
	public function getDataType()
	{
		return $this->type;
	}

	/**
	 * Returns text describing data type.
	 * For objects it returns text "object(ClassName)".
	 * 
	 * @return string
	 */
	protected function getDataTypeExt()
	{
		if ($this->getDataType() == 'object')
			return ($this->getDataType() . '(' . $this->getClass() . ')');
		else
			return $this->getDataType();
	}	
	
	/**
	 * If data type is "object", returns name of class accepted as data,
	 * otherwise returns empty string.
	 * 
	 * @return string
	 */
	public function getClass()
	{
		return $this->class;
	}

	/**
	 * Tests whether type of objects in array are subclasses or classes
	 * of $classname.
	 * Returns FALSE for scalar types.
	 * 
	 * @param $classname Name of class or interface.
	 * @return boolean
	 * @todo Test method.
	 */
	public function isSubclass($classname)
	{
		if ($this->getDataType() != 'object')
			return FALSE;
		if (!class_exists($classname))
			throw new InvalidArgumentException('Class name '.$classname.' is not valid. Class not defined.');
		if ($this->class == $classname)
			return TRUE;
		$reflection = new ReflectionClass($this->class);
		if ($reflection->isSubclassOf($classname) || $reflection->implementsInterface($classname))
			return TRUE;
		return FALSE;
	}
	
	/**
	 * Retuns TRUE if type testing of data values is strict.
	 * 
	 * For more information about strict testing see __construct API doc.
	 * 
	 * @return boolean
	 */
	public function isStrict()
	{
		return $this->strict;
	}

	/**
	 * Returns items count.
	 * 
	 * @return int
	 */
	public function count()
	{
		return count($this->values);
	}	

	/**
	 * Test data type of data.
	 * 
	 * @param  mixed $value
	 * @return boolean
	 */
	protected function testData($value)
	{
		if ($this->isStrict())
		{
			switch ($this->getDataType())
			{
				case 'integer':
					return is_int($value);
					break;
				case 'double':
				case 'float':
					return is_float($value);
					break;
				case 'string':
					return is_string($value);
					break;
				case 'boolean':
					return is_bool($value);
					break;
				case 'array':
					return is_array($value);
					break;
				case 'object':
					if (is_object($value) && (get_class($value) == $this->getClass()))
						return TRUE;
					else
						return FALSE;
					break;
				default:
					return FALSE;
			}
		}
		else
		{
			switch ($this->getDataType())
			{
				case 'integer':
					return is_numeric($value);
					break;
				case 'double':
				case 'float':
					return is_numeric($value);
					break;
				case 'string':
					return is_scalar($value);
					break;
				case 'boolean':
					return (is_bool($value) || ($value == 1) || ($value == 0));
					break;
				case 'array':
					return is_array($value);
					break;
				case 'object':
					$class_name = $this->getClass();
					if (is_object($value) && ($value instanceof $class_name))
						return TRUE;
					else
						return FALSE;
					break;
				default:
					return FALSE;
			}		
		}
	}
	
	/**
	 * Replaces or appends a item.
	 * 
	 * @param  mixed
	 * @param  mixed
	 * @return void
	 * @throws InvalidArgumentException if data type of $value is not acceptable
	 */
	public function offsetSet($index, $value)
	{
		if (!is_scalar($index))
			throw new InvalidArgumentException("Index must be scalar value. Invalid data: ". $index);
		if ($index === NULL) 
			$this->values[] = $value;
		elseif ($this->testData($value))
			$this->values[$index] = $value;
		else
			throw new InvalidArgumentException("Data are of invalid data type. Expected " . $this->getDataTypeExt());
	}

	/**
	 * Returns a item on defined index.
	 * @param  mixed $index
	 * @return mixed
	 * @throws OutOfRangeException
	 * @throws InvalidArgumentException
	 */
	public function offsetGet($index)
	{
		if (isset($this->values[$index]))
			throw new OutOfRangeException("Offset $index invalid or out of range.");
		return $this->values[$index];
	}



	/**
	 * Determines whether a item exists.
	 * @param  mixed $index
	 * @return bool
	 */
	public function offsetExists($index)
	{
		return isset($this->values[$index]);
	}



	/**
	 * Removes the element at the specified position in this list.
	 * @param  mixed $index
	 * @return void
	 * @throws OutOfRangeException
	 */
	public function offsetUnset($index)
	{
		if (isset($this->values[$index]))
			throw new OutOfRangeException("Offset $index invalid or out of range");
		$this->values[$index] = NULL;
	}	
	
	/**
	 * Returns an iterator over all items.
	 * @return ArrayIterator
	 */
	public function getIterator()
	{
		return new ArrayIterator($this->values);
	}	
}


